@@ -32,7 +32,7 @@ ChColor hcPlain_Folds = 0x0A;
 ChColor hcPlain_HilitWord = 0x0D;
 
 /* LIST */
-ChColor hcList_Border    = 0x03;
+//ChColor hcList_Border    = 0x03;
 ChColor hcList_Status    = 0x70;
 ChColor hcList_Normal    = 0x07;
 ChColor hcList_Selected  = 0x1F;
@@ -57,8 +57,8 @@ ChColor hcChoice_ActiveChar = 0x2F;
 ChColor hcChoice_NormalItem = 0x1F;
 ChColor hcChoice_NormalChar = 0x1E;
 
-struct {
-    char *Name;
+static const struct {
+    const char *Name;
     ChColor *C;
 } Colors[] = 
 {
@@ -73,7 +73,6 @@ struct {
     { "Entry.Prompt",      &hcEntry_Prompt },
     { "Entry.Selection",   &hcEntry_Selection },
 
-    { "LIST.Border",       &hcList_Border },
     { "LIST.Status",       &hcList_Status },
     { "LIST.Normal",       &hcList_Normal },
     { "LIST.Selected",     &hcList_Selected },
@@ -109,20 +108,18 @@ struct {
 };
 #define NCOLORS (sizeof(Colors)/sizeof(Colors[0]))
 
-int SetColor(char *ColorV, char *Value) {
-    unsigned int i;
+int SetColor(const char *ColorV, const char *Value) {
     unsigned int Col;
+    unsigned int ColBg, ColFg;
     ChColor C;
+
+    if (sscanf(Value, "%1X %1X", &ColFg, &ColBg) != 2)
+        return 0;
+
+    Col = ColFg | (ColBg << 4);
     
-    if (*Value == '-') {
-	Value++;
-	if (sscanf(Value, "%1X", &Col) != 1) return 0;
-	Col |= (hcPlain_Background & 0xF0);
-    } else {
-	if (sscanf(Value, "%2X", &Col) != 1) return 0;
-    }
     C = ChColor(Col);
-    for (i = 0; i < NCOLORS; i++) {
+    for (unsigned int i = 0; i < NCOLORS; i++) {
         if (strcmp(ColorV, Colors[i].Name) == 0) {
             *Colors[i].C = C;
             return 1;
