@@ -11,11 +11,11 @@
 
 typedef struct _CurPos {
     int sz;
-    char *a;
-    char *c;
-    char *z;
+    const char *a;
+    const char *c;
+    const char *z;
     int line;
-    char *name; // filename
+    const char *name; // filename
 } CurPos;
 
 #ifdef CONFIG_INDENT_C
@@ -28,6 +28,10 @@ extern int C_ClassDelta;
 extern int C_ColonOfs;
 extern int C_CommentOfs;
 extern int C_CommentDelta;
+extern int C_FirstLevelWidth;
+extern int C_FirstLevelIndent;
+extern int C_Continuation;
+extern int C_ParenDelta;
 #endif
 
 #ifdef CONFIG_INDENT_REXX
@@ -48,7 +52,11 @@ int OpenAfterClose = 1;
 int SelectPathname = 0;
 char DefaultModeName[32] = "";
 RxNode *CompletionFilter;
+#if defined(DOS) || defined(DOSP32)
+char PrintDevice[MAXPATH] = "PRN";
+#else
 char PrintDevice[MAXPATH] = "\\DEV\\PRN";
+#endif
 char CompileCommand[256] = "make";
 int KeepHistory = 0;
 int LoadDesktopOnEntry = 0;
@@ -64,14 +72,16 @@ int PMDisableAccel = 0;
 int SevenBit = 0;
 int WeirdScroll = 0;
 int LoadDesktopMode = 0;
+char HelpCommand[128] = "man -a";
+char *ConfigSourcePath = 0;
 
 #ifdef CONFIG_SYNTAX_HILIT
-int AddKeyword(ColorKeywords *tab, char color, char *keyword) {
+int AddKeyword(ColorKeywords *tab, char color, const char *keyword) {
     int len;
-    
+
     len = strlen(keyword);
     if (len < 1 || len >= CK_MAXLEN) return 0;
-    
+
     if (tab->key[len]) {
         int lx = strlen(tab->key[len]);
         tab->key[len] = (char *)realloc(tab->key[len], lx + len + 1 + 1);
@@ -84,7 +94,7 @@ int AddKeyword(ColorKeywords *tab, char color, char *keyword) {
         assert(tab->key[len] != 0);
         strcpy(tab->key[len], keyword);
         tab->key[len][len] = color;
-	tab->key[len][len + 1] = 0;
+        tab->key[len][len + 1] = 0;
     }
     tab->count[len]++;
     tab->TotalCount++;
@@ -100,37 +110,39 @@ int SetModeNumber(EMode *mode, int what, int number) {
     return 0;
 }
 
-int SetModeString(EMode *mode, int what, char *string) {
+int SetModeString(EMode *mode, int what, const char *string) {
     int j = what;
-    
+
 #ifdef CONFIG_SYNTAX_HILIT
     if (j == BFI_Colorizer) {
         mode->fColorize = FindColorizer(string);
     } else
 #endif
         if (j == BFI_EventMap) {
-        mode->fEventMap = FindEventMap(string);
-    } else if (j == BFI_IndentMode) {
-        mode->Flags.num[j] = GetIndentMode(string);
-    } else if (j == BFS_WordChars) {
-        SetWordChars(mode->Flags.WordChars, string);
-    } else if (j == BFS_FileNameRx) {
-        if (mode->MatchName)
-            free(mode->MatchName);
-        if (mode->MatchNameRx)
-            RxFree(mode->MatchNameRx);
-        mode->MatchName = strdup(string);
-        mode->MatchNameRx = RxCompile(string);
-    } else if (j == BFS_FirstLineRx) {
-        if (mode->MatchLine)
-            free(mode->MatchLine);
-        if (mode->MatchLineRx)
-            RxFree(mode->MatchLineRx);
-        mode->MatchLine = strdup(string);
-        mode->MatchLineRx = RxCompile(string);
-    } else {
-        mode->Flags.str[j & 0xFF] = strdup(string);
-    }
+            mode->fEventMap = FindEventMap(string);
+        } else if (j == BFI_IndentMode) {
+            mode->Flags.num[j] = GetIndentMode(string);
+        } else if (j == BFS_WordChars) {
+            SetWordChars(mode->Flags.WordChars, string);
+        } else if (j == BFS_CapitalChars) {
+            SetWordChars(mode->Flags.CapitalChars, string);
+        } else if (j == BFS_FileNameRx) {
+            if (mode->MatchName)
+                free(mode->MatchName);
+            if (mode->MatchNameRx)
+                RxFree(mode->MatchNameRx);
+            mode->MatchName = strdup(string);
+            mode->MatchNameRx = RxCompile(string);
+        } else if (j == BFS_FirstLineRx) {
+            if (mode->MatchLine)
+                free(mode->MatchLine);
+            if (mode->MatchLineRx)
+                RxFree(mode->MatchLineRx);
+            mode->MatchLine = strdup(string);
+            mode->MatchLineRx = RxCompile(string);
+        } else {
+            mode->Flags.str[j & 0xFF] = strdup(string);
+        }
     return 0;
 }
 
@@ -146,6 +158,10 @@ int SetGlobalNumber(int what, int number) {
     case FLAG_C_ColonOfs:        C_ColonOfs = number; break;
     case FLAG_C_CommentOfs:      C_CommentOfs = number; break;
     case FLAG_C_CommentDelta:    C_CommentDelta = number; break;
+    case FLAG_C_FirstLevelIndent: C_FirstLevelIndent = number; break;
+    case FLAG_C_FirstLevelWidth: C_FirstLevelWidth = number; break;
+    case FLAG_C_Continuation:    C_Continuation = number; break;
+    case FLAG_C_ParenDelta:      C_ParenDelta = number; break;
 #endif
 #ifdef CONFIG_INDENT_REXX
     case FLAG_REXX_Indent:       REXX_Base_Indent = number; break;
@@ -184,20 +200,21 @@ int SetGlobalNumber(int what, int number) {
     return 0;
 }
 
-int SetGlobalString(long what, char *string) {
+int SetGlobalString(long what, const char *string) {
     switch (what) {
     case FLAG_DefaultModeName: strcpy(DefaultModeName, string); break;
     case FLAG_CompletionFilter: if ((CompletionFilter = RxCompile(string)) == NULL) return -1; break;
     case FLAG_PrintDevice: strcpy(PrintDevice, string); break;
     case FLAG_CompileCommand: strcpy(CompileCommand, string); break;
     case FLAG_WindowFont: strcpy(WindowFont, string); break;
+    case FLAG_HelpCommand: strcpy(HelpCommand, string); break;
     default:
         return -1;
     }
     return 0;
 }
 
-int SetEventString(EEventMap *Map, long what, char *string) {
+int SetEventString(EEventMap *Map, long what, const char *string) {
     switch (what) {
     case EM_MainMenu:
     case EM_LocalMenu:
@@ -210,7 +227,7 @@ int SetEventString(EEventMap *Map, long what, char *string) {
 }
 
 #ifdef CONFIG_SYNTAX_HILIT
-int SetColorizeString(EColorize *Colorize, long what, char *string) {
+int SetColorizeString(EColorize *Colorize, long what, const char *string) {
     switch (what) {
     case COL_SyntaxParser:
         Colorize->SyntaxParser = GetHilitMode(string);
@@ -226,70 +243,74 @@ unsigned char GetObj(CurPos &cp, unsigned short &len) {
     len = 0;
     if (cp.c + 3 <= cp.z) {
         unsigned char c;
+        unsigned char l[2];
         c = *cp.c++;
-        memcpy(&len, cp.c, 2);
+        memcpy(&l, cp.c, 2);
+        len = (l[1] << 8) + l[0];
         cp.c += 2;
         return c;
     }
     return 0xFF;
 }
 
-char *GetCharStr(CurPos &cp, unsigned short len) {
-    char *p = cp.c;
+const char *GetCharStr(CurPos &cp, unsigned short len) {
+    const char *p = cp.c;
     if (cp.c + len > cp.z)
         return 0;
     cp.c += len;
     return p;
 }
 
-char *GetString(CurPos &cp) {
-    unsigned short len;
-    
-    if (GetObj(cp, len) == 0xFF) return 0;
-    return GetCharStr(cp, len);
-}
-
 int GetNum(CurPos &cp, long &num) {
+    unsigned char n[4];
     if (cp.c + 4 > cp.z) return 0;
-    memcpy(&num, cp.c, 4);
+    memcpy(&n, cp.c, 4);
+    num =
+        (n[3] << 24) +
+        (n[2] << 16) +
+        (n[1] << 8) +
+        n[0];
+
+    if ((n[3] > 127) && sizeof(long) > 4)
+        num = num | (~0xFFFFFFFFUL);
     cp.c += 4;
     return 1;
 }
 
-int ReadCommands(CurPos &cp, char *Name) {
+int ReadCommands(CurPos &cp, const char *Name) {
     unsigned char obj;
     unsigned short len;
     long Cmd = NewCommand(Name);
     long cmdno;
-    
+
     if (GetObj(cp, len) != CF_INT) return-1;
     if (GetNum(cp, cmdno) == 0) return -1;
     if (cmdno != (Cmd | CMD_EXT)) {
         fprintf(stderr, "Bad Command map %s -> %ld != %ld\n", Name, Cmd, cmdno);
         return -1;
     }
-    
+
     while ((obj = GetObj(cp, len)) != 0xFF) {
         switch (obj) {
         case CF_COMMAND:
             {
-  //              char *s;
+                //              char *s;
                 long cnt;
                 long ign;
                 long cmd;
-                
-//                if ((s = GetCharStr(cp, len)) == 0) return -1;
+
+                //                if ((s = GetCharStr(cp, len)) == 0) return -1;
                 if (GetNum(cp, cmd) == 0) return -1;
                 if (GetObj(cp, len) != CF_INT) return -1;
                 if (GetNum(cp, cnt) == 0) return -1;
                 if (GetObj(cp, len) != CF_INT) return -1;
                 if (GetNum(cp, ign) == 0) return -1;
-                
-//                if (cmd != CmdNum(s)) {
-//                    fprintf(stderr, "Bad Command Id: %s -> %d\n", s, cmd);
-//                    return -1;
-//                }
-                
+
+                //                if (cmd != CmdNum(s)) {
+                //                    fprintf(stderr, "Bad Command Id: %s -> %d\n", s, cmd);
+                //                    return -1;
+                //                }
+
                 if (AddCommand(Cmd, cmd, cnt, ign) == 0) {
                     if (Name == 0 || strcmp(Name, "xx") != 0) {
                         fprintf(stderr, "Bad Command Id: %ld\n", cmd);
@@ -300,8 +321,8 @@ int ReadCommands(CurPos &cp, char *Name) {
             break;
         case CF_STRING:
             {
-                char *s = GetCharStr(cp, len);
-                
+                const char *s = GetCharStr(cp, len);
+
                 if (s == 0) return -1;
                 if (AddString(Cmd, s) == 0) return -1;
             }
@@ -309,11 +330,22 @@ int ReadCommands(CurPos &cp, char *Name) {
         case CF_INT:
             {
                 long num;
-                
+
                 if (GetNum(cp, num) == 0) return -1;
                 if (AddNumber(Cmd, num) == 0) return -1;
             }
             break;
+        case CF_VARIABLE:
+            {
+                long num;
+
+                if (GetNum(cp, num) == 0) return -1;
+                if (AddVariable(Cmd, num) == 0) return -1;
+            }
+            break;
+        case CF_CONCAT:
+            if (AddConcat(Cmd) == 0) return -1;
+            break;
         case CF_END:
             return Cmd;
         default:
@@ -323,10 +355,10 @@ int ReadCommands(CurPos &cp, char *Name) {
     return -1;
 }
 
-int ReadMenu(CurPos &cp, char *MenuName) {
+int ReadMenu(CurPos &cp, const char *MenuName) {
     unsigned char obj;
     unsigned short len;
-    
+
     int menu = -1, item = -1;
 
     menu = NewMenu(MenuName);
@@ -338,7 +370,7 @@ int ReadMenu(CurPos &cp, char *MenuName) {
                 if (len == 0) {
                     item = NewItem(menu, 0);
                 } else {
-                    char *s = GetCharStr(cp, len);
+                    const char *s = GetCharStr(cp, len);
                     int Cmd;
                     if (s == 0) return -1;
                     item = NewItem(menu, s);
@@ -350,9 +382,9 @@ int ReadMenu(CurPos &cp, char *MenuName) {
             break;
         case CF_SUBMENU:
             {
-                char *s = GetCharStr(cp, len);
-                char *w;
-                
+                const char *s = GetCharStr(cp, len);
+                const char *w;
+
                 if ((obj = GetObj(cp, len)) != CF_STRING) return -1;
                 if ((w = GetCharStr(cp, len)) == 0) return -1;
                 item = NewSubMenu(menu, s, GetMenuId(w), SUBMENU_NORMAL);
@@ -361,15 +393,15 @@ int ReadMenu(CurPos &cp, char *MenuName) {
 
         case CF_SUBMENUCOND:
             {
-                char *s = GetCharStr(cp, len);
-                char *w;
-                
+                const char *s = GetCharStr(cp, len);
+                const char *w;
+
                 if ((obj = GetObj(cp, len)) != CF_STRING) return -1;
                 if ((w = GetCharStr(cp, len)) == 0) return -1;
                 item = NewSubMenu(menu, s, GetMenuId(w), SUBMENU_CONDITIONAL);
             }
             break;
-            
+
         case CF_END:
             return 0;
         default:
@@ -379,17 +411,17 @@ int ReadMenu(CurPos &cp, char *MenuName) {
     return -1;
 }
 
-int ReadColors(CurPos &cp, char *ObjName) {
+int ReadColors(CurPos &cp, const char *ObjName) {
     unsigned char obj;
     unsigned short len;
-    
+
     while ((obj = GetObj(cp, len)) != 0xFF) {
         switch (obj) {
         case CF_STRING:
             {
                 char cl[30];
-                char *sname = GetCharStr(cp, len);
-                char *svalue;
+                const char *sname = GetCharStr(cp, len);
+                const char *svalue;
                 if (sname == 0) return -1;
                 if ((obj = GetObj(cp, len)) != CF_STRING) return -1;
                 if ((svalue = GetCharStr(cp, len)) == 0) return -1;
@@ -409,17 +441,17 @@ int ReadColors(CurPos &cp, char *ObjName) {
 }
 
 #ifdef CONFIG_SYNTAX_HILIT
-int ReadHilitColors(CurPos &cp, EColorize *Colorize, char *ObjName) {
+int ReadHilitColors(CurPos &cp, EColorize *Colorize, const char * /*ObjName*/) {
     unsigned char obj;
     unsigned short len;
-    
+
     while ((obj = GetObj(cp, len)) != 0xFF) {
         switch (obj) {
         case CF_INT:
             {
                 long cidx;
-                char *svalue;
-                
+                const char *svalue;
+
                 if (GetNum(cp, cidx) == 0) return -1;
                 if ((obj = GetObj(cp, len)) != CF_STRING)
                     return -1;
@@ -441,12 +473,12 @@ int ReadHilitColors(CurPos &cp, EColorize *Colorize, char *ObjName) {
 int ReadKeywords(CurPos &cp, ColorKeywords *keywords, int color) {
     unsigned char obj;
     unsigned short len;
-    
+
     while ((obj = GetObj(cp, len)) != 0xFF) {
         switch (obj) {
         case CF_STRING:
             {
-                char *kname = GetCharStr(cp, len);
+                const char *kname = GetCharStr(cp, len);
                 if (kname == 0) return -1;
                 if (AddKeyword(keywords, (char) color, kname) != 1) return -1;
             }
@@ -461,18 +493,18 @@ int ReadKeywords(CurPos &cp, ColorKeywords *keywords, int color) {
 }
 #endif
 
-int ReadEventMap(CurPos &cp, EEventMap *Map, char *MapName) {
+int ReadEventMap(CurPos &cp, EEventMap *Map, const char * /*MapName*/) {
     unsigned char obj;
     unsigned short len;
-    
+
     while ((obj = GetObj(cp, len)) != 0xFF) {
         switch (obj) {
         case CF_KEY:
             {
                 EKey *Key;
-                char *s;
+                const char *s;
                 int Cmd;
-                
+
                 if ((s = GetCharStr(cp, len)) == 0) return -1;
                 if ((Key = SetKey(Map, s)) == 0) return -1;
                 if ((obj = GetObj(cp, len)) != CF_KEYSUB) return -1;
@@ -485,10 +517,10 @@ int ReadEventMap(CurPos &cp, EEventMap *Map, char *MapName) {
         case CF_ABBREV:
             {
                 EAbbrev *Ab;
-                char *s;
-                char *x;
+                const char *s;
+                const char *x;
                 int Cmd;
-                
+
                 if ((s = GetCharStr(cp, len)) == 0) return -1;
                 obj = GetObj(cp, len);
                 if (obj == CF_KEYSUB) {
@@ -505,28 +537,28 @@ int ReadEventMap(CurPos &cp, EEventMap *Map, char *MapName) {
             }
             break;
 #endif
-            
+
         case CF_SETVAR:
             {
                 long what;
-                
+
                 if (GetNum(cp, what) == 0) return -1;
                 switch (GetObj(cp, len)) {
                 case CF_STRING:
                     {
-                        char *val = GetCharStr(cp, len);
+                        const char *val = GetCharStr(cp, len);
                         if (len == 0) return -1;
                         if (SetEventString(Map, what, val) != 0) return -1;
                     }
                     break;
-/*                case CF_INT:
-                    {
-                        long num;
-                        
-                        if (GetNum(cp, num) == 0) return -1;
-                        if (SetModeNumber(Mode, what, num) != 0) return -1;
-                    }
-	            break;*/
+                    /*                case CF_INT:
+                     {
+                     long num;
+
+                     if (GetNum(cp, num) == 0) return -1;
+                     if (SetModeNumber(Mode, what, num) != 0) return -1;
+                     }
+                     break;*/
                 default:
                     return -1;
                 }
@@ -542,40 +574,33 @@ int ReadEventMap(CurPos &cp, EEventMap *Map, char *MapName) {
 }
 
 #ifdef CONFIG_SYNTAX_HILIT
-int ReadColorize(CurPos &cp, EColorize *Colorize, char *ModeName) {
+int ReadColorize(CurPos &cp, EColorize *Colorize, const char *ModeName) {
     unsigned char obj;
     unsigned short len;
 
     long LastState = -1;
-    
+
     while ((obj = GetObj(cp, len)) != 0xFF) {
         switch (obj) {
         case CF_COLOR:
             if (ReadHilitColors(cp, Colorize, ModeName) == -1) return -1;
             break;
-        
+
         case CF_KEYWORD:
             {
-                char *colorstr;
-                int color;
-                
+                const char *colorstr;
+
                 if ((colorstr = GetCharStr(cp, len)) == 0) return -1;
 
-                color = hcPlain_Keyword;
+                unsigned int Col;
+                unsigned int ColBg, ColFg;
 
-                if (strcmp(colorstr, "-") != 0) {
-                    char *Value = colorstr;
-                    int Col;
-                    
-                    if (*Value == '-') {
-                        Value++;
-                        if (sscanf(Value, "%1X", &Col) != 1) return -1;
-                        Col |= (hcPlain_Background & 0xF0);
-                    } else {
-                        if (sscanf(Value, "%2X", &Col) != 1) return -1;
-                    }
-                    color = Col;
-                }
+                if (sscanf(colorstr, "%1X %1X", &ColFg, &ColBg) != 2)
+                    return 0;
+
+                Col = ColFg | (ColBg << 4);
+
+                int color = ChColor(Col);
                 if (ReadKeywords(cp, &Colorize->Keywords, color) == -1) return -1;
             }
             break;
@@ -584,7 +609,7 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, char *ModeName) {
             {
                 long stateno;
                 long color;
-                
+
                 if (Colorize->hm == 0)
                     Colorize->hm = new HMachine();
 
@@ -617,7 +642,7 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, char *ModeName) {
                 HTrans newTrans;
                 long nextState;
                 long matchFlags;
-                char *match;
+                const char *match;
                 long color;
 
                 if (GetNum(cp, nextState) == 0)
@@ -640,7 +665,7 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, char *ModeName) {
                 newTrans.matchFlags = matchFlags;
                 newTrans.nextState = nextState;
                 newTrans.color = color;
-                
+
                 if ((newTrans.matchFlags & MATCH_SET) ||
                     (newTrans.matchFlags & MATCH_NOTSET))
                 {
@@ -662,7 +687,7 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, char *ModeName) {
                 long nextKwdNotMatchedState;
                 long nextKwdNoCharState;
                 long options;
-                char *wordChars;
+                const char *wordChars;
 
                 obj = GetObj(cp, len);
                 assert(obj == CF_INT);
@@ -693,7 +718,7 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, char *ModeName) {
                 Colorize->hm->LastState()->nextKwdMatchedState = nextKwdMatchedState;
                 Colorize->hm->LastState()->nextKwdNotMatchedState = nextKwdNotMatchedState;
                 Colorize->hm->LastState()->nextKwdNoCharState = nextKwdNoCharState;
-                
+
                 if (wordChars && *wordChars) {
                     assert((Colorize->hm->LastState()->wordChars = (char *)malloc(256/8)) != 0);
                     SetWordChars(Colorize->hm->LastState()->wordChars, wordChars);
@@ -703,17 +728,17 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, char *ModeName) {
 
         case CF_HWORDS:
             {
-                char *colorstr;
+                const char *colorstr;
                 int color;
 
                 if ((colorstr = GetCharStr(cp, len)) == 0) return -1;
-                
+
                 color = hcPlain_Keyword;
-                
+
                 if (strcmp(colorstr, "-") != 0) {
-                    char *Value = colorstr;
+                    const char *Value = colorstr;
                     int Col;
-                    
+
                     if (*Value == '-') {
                         Value++;
                         if (sscanf(Value, "%1X", &Col) != 1) return -1;
@@ -734,24 +759,24 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, char *ModeName) {
         case CF_SETVAR:
             {
                 long what;
-                
+
                 if (GetNum(cp, what) == 0) return -1;
                 switch (GetObj(cp, len)) {
                 case CF_STRING:
                     {
-                        char *val = GetCharStr(cp, len);
+                        const char *val = GetCharStr(cp, len);
                         if (len == 0) return -1;
                         if (SetColorizeString(Colorize, what, val) != 0) return -1;
                     }
                     break;
-/*                case CF_INT:
-                    {
-                        long num;
-                        
-                        if (GetNum(cp, num) == 0) return -1;
-                        if (SetModeNumber(Mode, what, num) != 0) return -1;
-                    }
-	            break;*/
+                    /*                case CF_INT:
+                     {
+                     long num;
+
+                     if (GetNum(cp, num) == 0) return -1;
+                     if (SetModeNumber(Mode, what, num) != 0) return -1;
+                     }
+                     break;*/
                 default:
                     return -1;
                 }
@@ -767,21 +792,21 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, char *ModeName) {
 }
 #endif
 
-int ReadMode(CurPos &cp, EMode *Mode, char *ModeName) {
+int ReadMode(CurPos &cp, EMode *Mode, const char * /*ModeName*/) {
     unsigned char obj;
     unsigned short len;
-    
+
     while ((obj = GetObj(cp, len)) != 0xFF) {
         switch (obj) {
         case CF_SETVAR:
             {
                 long what;
-                
+
                 if (GetNum(cp, what) == 0) return -1;
                 switch (GetObj(cp, len)) {
                 case CF_STRING:
                     {
-                        char *val = GetCharStr(cp, len);
+                        const char *val = GetCharStr(cp, len);
                         if (len == 0) return -1;
                         if (SetModeString(Mode, what, val) != 0) return -1;
                     }
@@ -789,11 +814,11 @@ int ReadMode(CurPos &cp, EMode *Mode, char *ModeName) {
                 case CF_INT:
                     {
                         long num;
-                        
+
                         if (GetNum(cp, num) == 0) return -1;
                         if (SetModeNumber(Mode, what, num) != 0) return -1;
                     }
-	            break;
+                    break;
                 default:
                     return -1;
                 }
@@ -808,10 +833,10 @@ int ReadMode(CurPos &cp, EMode *Mode, char *ModeName) {
     return -1;
 }
 
-int ReadObject(CurPos &cp, char *ObjName) {
+int ReadObject(CurPos &cp, const char *ObjName) {
     unsigned char obj;
     unsigned short len;
-    
+
     while ((obj = GetObj(cp, len)) != 0xFF) {
         switch (obj) {
         case CF_COLOR:
@@ -821,8 +846,8 @@ int ReadObject(CurPos &cp, char *ObjName) {
         case CF_COMPRX:
             {
                 long file, line, msg;
-                char *regexp;
-                
+                const char *regexp;
+
                 if (GetObj(cp, len) != CF_INT) return -1;
                 if (GetNum(cp, file) == 0) return -1;
                 if (GetObj(cp, len) != CF_INT) return -1;
@@ -831,7 +856,7 @@ int ReadObject(CurPos &cp, char *ObjName) {
                 if (GetNum(cp, msg) == 0) return -1;
                 if (GetObj(cp, len) != CF_REGEXP) return -1;
                 if ((regexp = GetCharStr(cp, len)) == 0) return -1;
-                
+
                 if (AddCRegexp(file, line, msg, regexp) == 0) return -1;
             }
             break;
@@ -840,11 +865,11 @@ int ReadObject(CurPos &cp, char *ObjName) {
             {
                 long what;
                 if (GetNum(cp, what) == 0) return -1;
-                
+
                 switch (GetObj(cp, len)) {
                 case CF_STRING:
                     {
-                        char *val = GetCharStr(cp, len);
+                        const char *val = GetCharStr(cp, len);
                         if (len == 0) return -1;
                         if (SetGlobalString(what, val) != 0) return -1;
                     }
@@ -852,11 +877,11 @@ int ReadObject(CurPos &cp, char *ObjName) {
                 case CF_INT:
                     {
                         long num;
-                        
+
                         if (GetNum(cp, num) == 0) return -1;
                         if (SetGlobalNumber(what, num) != 0) return -1;
                     }
-	            break;
+                    break;
                 default:
                     return -1;
                 }
@@ -870,41 +895,51 @@ int ReadObject(CurPos &cp, char *ObjName) {
     }
     return -1;
 }
-            
+
 int ReadConfigFile(CurPos &cp) {
     unsigned char obj;
     unsigned short len;
-    
+
+    {
+        const char *p;
+
+        obj = GetObj(cp, len);
+        assert(obj == CF_STRING);
+        if ((p = GetCharStr(cp, len)) == 0)
+            return -1;
+        ConfigSourcePath = strdup(p);
+    }
+
     while ((obj = GetObj(cp, len)) != 0xFF) {
         switch (obj) {
         case CF_SUB:
             {
-                char *CmdName = GetCharStr(cp, len);
-                
-                if (ReadCommands(cp, strdup(CmdName)) == -1) return -1;
+                const char *CmdName = GetCharStr(cp, len);
+
+                if (ReadCommands(cp, CmdName) == -1) return -1;
             }
             break;
         case CF_MENU:
             {
-                char *MenuName = GetCharStr(cp, len);
-                
+                const char *MenuName = GetCharStr(cp, len);
+
                 if (ReadMenu(cp, MenuName) == -1) return -1;
             }
             break;
         case CF_EVENTMAP:
             {
                 EEventMap *EventMap = 0;
-                char *MapName = GetCharStr(cp, len);
-                char *UpMap = 0;
-                
+                const char *MapName = GetCharStr(cp, len);
+                const char *UpMap = 0;
+
                 if ((obj = GetObj(cp, len)) != CF_PARENT) return -1;
                 if (len > 0)
                     if ((UpMap = GetCharStr(cp, len)) == 0) return -1;
-            
+
                 // add new mode
                 if ((EventMap = FindEventMap(MapName)) == 0) {
                     EEventMap *OrgMap = 0;
-                    
+
                     if (strcmp(UpMap, "") != 0)
                         OrgMap = FindEventMap(UpMap);
                     EventMap = new EEventMap(MapName, OrgMap);
@@ -919,13 +954,13 @@ int ReadConfigFile(CurPos &cp) {
         case CF_COLORIZE:
             {
                 EColorize *Mode = 0;
-                char *ModeName = GetCharStr(cp, len);
-                char *UpMode = 0;
-                
+                const char *ModeName = GetCharStr(cp, len);
+                const char *UpMode = 0;
+
                 if ((obj = GetObj(cp, len)) != CF_PARENT) return -1;
                 if (len > 0)
                     if ((UpMode = GetCharStr(cp, len)) == 0) return -1;
-                
+
                 // add new mode
                 if ((Mode = FindColorizer(ModeName)) == 0)
                     Mode = new EColorize(ModeName, UpMode);
@@ -941,24 +976,24 @@ int ReadConfigFile(CurPos &cp) {
         case CF_MODE:
             {
                 EMode *Mode = 0;
-                char *ModeName = GetCharStr(cp, len);
-                char *UpMode = 0;
-                
+                const char *ModeName = GetCharStr(cp, len);
+                const char *UpMode = 0;
+
                 if ((obj = GetObj(cp, len)) != CF_PARENT) return -1;
                 if (len > 0)
                     if ((UpMode = GetCharStr(cp, len)) == 0) return -1;
-                
+
                 // add new mode
                 if ((Mode = FindMode(ModeName)) == 0) {
                     EMode *OrgMode = 0;
                     EEventMap *Map;
-                    
+
                     if (strcmp(UpMode, "") != 0)
                         OrgMode = FindMode(UpMode);
                     Map = FindEventMap(ModeName);
                     if (Map == 0) {
                         EEventMap *OrgMap = 0;
-                        
+
                         if (strcmp(UpMode, "") != 0)
                             OrgMap = FindEventMap(UpMode);
                         Map = new EEventMap(ModeName, OrgMap);
@@ -976,8 +1011,8 @@ int ReadConfigFile(CurPos &cp) {
             break;
         case CF_OBJECT:
             {
-                char *ObjName;
-                
+                const char *ObjName;
+
                 if ((ObjName = GetCharStr(cp, len)) == 0)
                     return -1;
                 if (ReadObject(cp, ObjName) == -1)
@@ -993,12 +1028,12 @@ int ReadConfigFile(CurPos &cp) {
     return -1;
 }
 
-int LoadConfig(int argc, char *argv[], char *CfgFileName) {
+int LoadConfig(int /*argc*/, char ** /*argv*/, char *CfgFileName) {
     int fd, rc;
     char *buffer = 0;
     struct stat statbuf;
     CurPos cp;
-    
+
     if ((fd = open(CfgFileName, O_RDONLY | O_BINARY)) == -1)
         return -1;
     if (fstat(fd, &statbuf) != 0) {
@@ -1015,28 +1050,37 @@ int LoadConfig(int argc, char *argv[], char *CfgFileName) {
         return -1;
     }
     close(fd);
-    
-    if (*(long *)buffer != CONFIG_ID) {
+
+    unsigned char l[4];
+    unsigned long ln;
+
+    memcpy(l, buffer, 4);
+    ln = (l[3] << 24) + (l[2] << 16) + (l[1] << 8) + l[0];
+
+    if (ln != CONFIG_ID) {
         DieError(0, "Bad .CNF signature");
         return -1;
     }
 
-    if (((long *)buffer)[1] != VERNUM) {
+    memcpy(l, buffer + 4, 4);
+    ln = (l[3] << 24) + (l[2] << 16) + (l[1] << 8) + l[0];
+
+    if (ln != VERNUM) {
         DieError(0, "Bad .CNF version.");
         return -1;
     }
-    
+
     cp.name = CfgFileName;
     cp.sz = statbuf.st_size;
     cp.a = buffer;
-    cp.c = cp.a + 2 * sizeof(long);
+    cp.c = cp.a + 2 * 4;
     cp.z = cp.a + cp.sz;
     cp.line = 1;
-    
+
     rc = ReadConfigFile(cp);
 
     free(buffer);
-    
+
     if (rc == -1) {
         DieError(1, "Error %s offset %d\n", CfgFileName, cp.c - cp.a);
     }
@@ -1048,16 +1092,16 @@ int LoadConfig(int argc, char *argv[], char *CfgFileName) {
 int UseDefaultConfig() {
     CurPos cp;
     int rc;
-    
+
     cp.name = "Internal Configuration";
     cp.sz = sizeof(DefaultConfig);
     cp.a = (char *)DefaultConfig;
-    cp.c = (char *)DefaultConfig + 2 * sizeof(long);
+    cp.c = (char *)DefaultConfig + 2 * 4;
     cp.z = cp.a + cp.sz;
     cp.line = 1;
-    
+
     rc = ReadConfigFile(cp);
-    
+
     if (rc == -1)
         DieError(1, "Error %s offset %d\n", cp.name, cp.c - cp.a);
     return rc;
