@@ -12,6 +12,12 @@
 
 #ifdef CONFIG_HISTORY
 
+#ifdef UNIX
+#define HISTORY_NAME ".fte-history"
+#else
+#define HISTORY_NAME "fte.his"
+#endif
+
 typedef struct {
     char *FileName;
     int Row, Col;
