@@ -22,13 +22,13 @@ EBufferFlags DefaultBufferFlags = {
         INDENT_PLAIN,       // IndentMode
         0,                  // ShowTab
         10,                 // LineChar
-#if defined(OS2) || defined(NT)
+#if !defined(UNIX)
         13,                 // StripChar
 #else
         -1,
 #endif
         1,                  // AddLine
-#if defined(OS2) || defined(NT)
+#if !defined(UNIX)
         1,                  // AddStrip
 #else
         0,
@@ -62,6 +62,12 @@ EBufferFlags DefaultBufferFlags = {
         1,                  // PersistentBlocks
         0,                  // InsertKillBlock
         0,                  // UndoMoves
+#ifdef UNIX
+        0,                  // DetectLineSep
+#else
+        1,
+#endif
+        0                   // trim on save
     },
     {
         0,                  // Routine Regexp
@@ -71,10 +77,11 @@ EBufferFlags DefaultBufferFlags = {
         0,                  // comment end (folds)
         0,                  // filename rx
         0,                  // firstline rx
+        0                   // compile command
     }
 };
 
-EMode *GetModeForName(char *FileName) {
+EMode *GetModeForName(const char *FileName) {
     //    char ext[10];
     //    char *p;
     int l, i;
