@@ -50,8 +50,6 @@ typedef unsigned char ChColor;
 #define INDENT_REXX   2
 #define INDENT_SIMPLE 3
 
-#define BFI_COUNT              44
-
 #define BFI_AutoIndent          0
 #define BFI_Insert              1
 #define BFI_DrawOn              2
@@ -101,8 +99,10 @@ typedef unsigned char ChColor;
 #define BFI_InsertKillBlock    41
 #define BFI_EventMap           42
 #define BFI_UndoMoves          43
+#define BFI_DetectLineSep      44
+#define BFI_TrimOnSave         45
 
-#define BFS_COUNT               7
+#define BFI_COUNT              46
 
 #define BFS_RoutineRegexp       (0 | 256)
 #define BFS_DefFindOpt          (1 | 256)
@@ -111,8 +111,12 @@ typedef unsigned char ChColor;
 #define BFS_CommentEnd          (4 | 256)
 #define BFS_FileNameRx          (5 | 256)
 #define BFS_FirstLineRx         (6 | 256)
+#define BFS_CompileCommand      (7 | 256)
+
+#define BFS_COUNT               8
 
 #define BFS_WordChars           (100 | 256) // ext
+#define BFS_CapitalChars        (101 | 256)
 
 #define BFI(y,x) ((y)->Flags.num[(x) & 0xFF])
 #define BFS(y,x) ((y)->Flags.str[(x) & 0xFF])
@@ -129,6 +133,7 @@ typedef struct {
     int num[BFI_COUNT];
     char *str[BFS_COUNT];
     char WordChars[32];
+    char CapitalChars[32];
 } EBufferFlags;
 
 extern EBufferFlags DefaultBufferFlags;
@@ -177,6 +182,11 @@ extern EBufferFlags DefaultBufferFlags;
 #define FLAG_SevenBit           46
 #define FLAG_WeirdScroll        47
 #define FLAG_LoadDesktopMode    48
+#define FLAG_HelpCommand        49
+#define FLAG_C_FirstLevelIndent 50
+#define FLAG_C_FirstLevelWidth  51
+#define FLAG_C_Continuation     52
+#define FLAG_C_ParenDelta       32
 
 #define EM_MENUS 2
 #define EM_MainMenu 0
@@ -234,4 +244,18 @@ extern EBufferFlags DefaultBufferFlags;
 #define STATE_TAGASNEXT    0x0002
 #define STATE_NOGRAB       0x0004
 
+typedef enum {
+    mvFilePath = 1,  /* directory + name + extension */
+    mvFileName,      /* name + extension */
+    mvFileDirectory, /* directory + '/' */
+    mvFileBaseName,  /* without the last extension */
+    mvFileExtension, /* the last one */
+    mvCurDirectory,
+    mvCurRow,
+    mvCurCol,
+    mvChar,
+    mvWord,
+    mvLine
+} MacroVariable;
+
 #endif
