@@ -1,6 +1,6 @@
 /*    con_os2.cpp
  *
- *    Copyright (c) 1994-1996, Marko Macek
+ *    Copyright (c) 1994-1998, Marko Macek
  *
  *    You may distribute under the terms of either the GNU General Public
  *    License or the Artistic License, as specified in the README file.
@@ -238,6 +238,9 @@ int ReadKbdEvent(TEvent *Event, int Wait) {
                         break;
                     }
             } else {
+                if (CharCode < 32)
+                    if (KeyFlags & kfCtrl)
+                        CharCode += 64;
                 KeyCode = CharCode;
             }
         }
@@ -303,8 +306,12 @@ int ReadMouseEvent(TEvent *Event, ULONG EventMask) {
             Event->What = evMouseUp;
         else
             Event->What = evMouseDown;
-    } else
+    } else {
         Event->What = evMouseMove;
+        if (Event->Mouse.X == LastMouseEvent.Mouse.X &&
+            Event->Mouse.Y == LastMouseEvent.Mouse.Y)
+            return 0;
+    }
     Event->Mouse.Buttons = Buttons;
     Event->Mouse.Count = 1;
     PrevState = State;
@@ -382,7 +389,7 @@ int ConPutBox(int X, int Y, int W, int H, PCell Cell) {
                     DrawMouse(0);
                     MouseHidden = 1;
                 }
-        VioWrtCellStr(p, (USHORT)(W << 1), (USHORT)(Y + I), (USHORT)X, 0);
+        VioWrtCellStr((PCH)p, (USHORT)(W << 1), (USHORT)(Y + I), (USHORT)X, 0);
 
         if (MouseHidden) {
             DrawMouse(1);
@@ -404,13 +411,13 @@ int ConGetBox(int X, int Y, int W, int H, PCell Cell) {
         ConQueryMousePos(&MX, &MY);
 
     for (I = 0; I < H; I++) {
-    if (MouseVisible)
-        if (Y + I == MY)
-            if (MX >= X && MX < X + W) {
-                DrawMouse(0);
-                MouseHidden = 1;
-            }
-        VioReadCellStr((char *)p, &WW, (USHORT)(Y + I), (USHORT)X, 0);
+        if (MouseVisible)
+            if (Y + I == MY)
+                if (MX >= X && MX < X + W) {
+                    DrawMouse(0);
+                    MouseHidden = 1;
+                }
+        VioReadCellStr((PCH)p, &WW, (USHORT)(Y + I), (USHORT)X, 0);
 
         if (MouseHidden) {
             DrawMouse(1);
@@ -436,7 +443,7 @@ int ConPutLine(int X, int Y, int W, int H, PCell Cell) {
                 DrawMouse(0);
                 MouseHidden = 1;
             }
-        VioWrtCellStr(p, (USHORT)(W << 1), (USHORT)(Y + I), (USHORT)X, 0);
+        VioWrtCellStr((PCH)p, (USHORT)(W << 1), (USHORT)(Y + I), (USHORT)X, 0);
 
         if (MouseHidden) {
             DrawMouse(1);
@@ -461,7 +468,7 @@ int ConSetBox(int X, int Y, int W, int H, TCell Cell) {
                     DrawMouse(0);
                     MouseHidden = 1;
                 }
-        VioWrtNCell(p, (USHORT)(W), (USHORT)(Y + I), (USHORT)X, 0);
+        VioWrtNCell((PCH)p, (USHORT)(W), (USHORT)(Y + I), (USHORT)X, 0);
 
         if (MouseHidden) {
             DrawMouse(1);
@@ -917,7 +924,7 @@ static int CreatePipeChild(PID &pid, HPIPE &hfPipe, char *Command) {
     return 0;
 }
 
-static void PipeThread(void *p) {
+static void _LNK_CONV PipeThread(void *p) {
     GPipe *pipe = (GPipe *)p;
     int rc;
     ULONG ulPostCount;
@@ -1018,10 +1025,17 @@ int GUI::OpenPipe(char *Command, EModel *notify) {
                 return -1;
             }
 
+#if defined(__MT__) || defined(__MULTI__)
             Pipes[i].tid = _beginthread(PipeThread,
                                         FAKE_BEGINTHREAD_NULL
                                         16384, &Pipes[i]);
-
+#else
+            DosCreateThread(Pipes[i].tid,
+                            (PFNTHREAD)PipeThread,
+                            &Pipes[i],
+                            0, /* immediate */
+                            16384);
+#endif
             Pipes[i].used = 1;
             //fprintf(stderr, "Pipe Open: %d\n", i);
             return i;
@@ -1104,7 +1118,7 @@ int GUI::ClosePipe(int id) {
     return Pipes[id].RetCode;
 }
 
-int GUI::RunProgram(char *Command) {
+int GUI::RunProgram(int mode, char *Command) {
     int rc, W, H, W1, H1;
 
     ConQuerySize(&W, &H);
