@@ -16,7 +16,7 @@
 int EBuffer::SetBB(EPoint M) {
     EPoint OldBB = BB;
     int MinL, MaxL;
-    
+
     if (BB.Row == M.Row && BB.Col == M.Col) return 1;
 #ifdef CONFIG_UNDOREDO
     if (PushBlockData() == 0) return 0;
@@ -34,7 +34,7 @@ int EBuffer::SetBB(EPoint M) {
 int EBuffer::SetBE(EPoint M) {
     EPoint OldBE = BE;
     int MinL, MaxL;
-  
+
     if (BE.Row == M.Row && BE.Col == M.Col) return 1;
 #ifdef CONFIG_UNDOREDO
     if (PushBlockData() == 0) return 0;
@@ -85,7 +85,7 @@ int EBuffer::BlockRedraw() {
 
 int EBuffer::BlockBegin() {
     EPoint X;
-    
+
     X.Row = VToR(CP.Row);
     X.Col = CP.Col;
     CheckBlock();
@@ -95,7 +95,7 @@ int EBuffer::BlockBegin() {
 
 int EBuffer::BlockEnd() {
     EPoint X;
-    
+
     X.Row = VToR(CP.Row);
     X.Col = CP.Col;
     CheckBlock();
@@ -105,7 +105,7 @@ int EBuffer::BlockEnd() {
 
 int EBuffer::BlockUnmark() {
     EPoint Null(-1,-1);
-    
+
     SetBB(BE);
     SetBE(Null);
     SetBB(Null);
@@ -122,7 +122,7 @@ int EBuffer::BlockCopy(int Append) {
     EPoint B, E;
     int L;
     int SL, OldCount;
-    
+
     AutoExtend = 0;
     if (CheckBlock() == 0) return 0;
     if (RCount == 0) return 0;
@@ -147,7 +147,7 @@ int EBuffer::BlockCopy(int Append) {
             SL++;
         }
         break;
-        
+
     case bmColumn:
         for (L = B.Row; L < E.Row; L++) {
             if (SS->InsLine(SL, 0) == 0) return 0;
@@ -156,7 +156,7 @@ int EBuffer::BlockCopy(int Append) {
             SL++;
         }
         break;
-        
+
     case bmStream:
         if (B.Row == E.Row) {
             if (SS->InsLine(SL, 0) == 0) return 0;
@@ -179,7 +179,7 @@ int EBuffer::BlockCopy(int Append) {
         break;
     }
     if (SystemClipboard)
-	PutPMClip();
+        PutPMClip();
     return 1;
 }
 
@@ -201,9 +201,9 @@ int EBuffer::BlockPasteColumn() {
 int EBuffer::BlockPaste() {
     EPoint B, E;
     int L, BL;
-    
+
     if (SystemClipboard)
-	GetPMClip();
+        GetPMClip();
 
     if (SS == 0) return 0;
     if (SS->RCount == 0) return 0;
@@ -228,12 +228,12 @@ int EBuffer::BlockPaste() {
         SetBB(B);
         SetBE(E);
         break;
-    
+
     case bmColumn:
         for (L = 0; L < SS->RCount; L++) {
             if (AssertLine(BL) == 0) return 0;
             if (InsLineText(BL, B.Col, SS->LineLen(L), 0, SS->RLine(L)) == 0) return 0;
-	    if (TrimLine(BL) == 0) return 0;
+            if (TrimLine(BL) == 0) return 0;
             BL++;
         }
         if (AssertLine(BL) == 0) return 0;
@@ -242,7 +242,7 @@ int EBuffer::BlockPaste() {
         SetBB(B);
         SetBE(E);
         break;
-    
+
     case bmStream:
         if (SS->RCount > 1)
             if (SplitLine(B.Row, B.Col) == 0) return 0;
@@ -265,21 +265,21 @@ int EBuffer::BlockPaste() {
         SetBE(E);
         break;
     }
-    return 1;    
+    return 1;
 }
 
 int EBuffer::BlockKill() {
     EPoint B, E;
     int L;
     int Y = -1;
-    
+
     AutoExtend = 0;
     if (CheckBlock() == 0) return 0;
     if (RCount <= 0) return 0;
     B = BB;
     E = BE;
     Draw(B.Row, -1);
-//    if (MoveToPos(B.Col, B.Row) == 0) return 0;
+    //    if (MoveToPos(B.Col, B.Row) == 0) return 0;
 
 #ifdef CONFIG_UNDOREDO
     if (BFI(this, BFI_Undo) == 1) {
@@ -288,7 +288,7 @@ int EBuffer::BlockKill() {
         if (PushUChar(ucPosition) == 0) return 0;
     }
 #endif
-    
+
     switch (BlockMode) {
     case bmLine:
         Y = VToR(CP.Row);
@@ -302,7 +302,7 @@ int EBuffer::BlockKill() {
         for (L = B.Row; L < E.Row; L++)
             if (DelLine(B.Row) == 0) return 0;
         break;
-        
+
     case bmColumn:
         Y = VToR(CP.Row);
         if (Y >= B.Row && Y < E.Row) {
@@ -317,10 +317,10 @@ int EBuffer::BlockKill() {
         for (L = B.Row; L < E.Row; L++)
             if (DelText(L, B.Col, E.Col - B.Col) == 0) return 0;
         break;
-        
+
     case bmStream:
         Y = VToR(CP.Row);
-        
+
         if (B.Row == E.Row) {
             if (Y == B.Row) {
                 if (CP.Col >= B.Col) {
@@ -369,7 +369,7 @@ int EBuffer::ClipClear() {
 int EBuffer::BlockIndent() {
     EPoint B, E;
     int L;
-    
+
     AutoExtend = 0;
     if (CheckBlock() == 0) return 0;
     if (RCount <= 0) return 0;
@@ -401,7 +401,7 @@ int EBuffer::BlockIndent() {
 int EBuffer::BlockUnindent() {
     EPoint B, E;
     int L;
-    
+
     AutoExtend = 0;
     if (CheckBlock() == 0) return 0;
     if (RCount <= 0) return 0;
@@ -415,7 +415,7 @@ int EBuffer::BlockUnindent() {
         case bmLine:
             if (L < E.Row || E.Col != 0) {
                 int I = LineIndented(L) - 1;
-                if (I >= 0) 
+                if (I >= 0)
                     IndentLine(L, I);
             }
             break;
@@ -473,7 +473,7 @@ int EBuffer::BlockExtendBegin() {
     ExtendGrab = 0;
     AutoExtend = 0;
     int Y = VToR(CP.Row);
-    
+
     switch (BlockMode) {
     case bmStream:
         if ((Y == BB.Row) && (CP.Col == BB.Col)) ExtendGrab |= 1;
@@ -490,21 +490,21 @@ int EBuffer::BlockExtendBegin() {
         if (CP.Col == BE.Col) ExtendGrab |= 8;
         break;
     }
-    
+
     if (ExtendGrab == 0) {
         BlockBegin();
         BlockEnd();
         if (BlockMode == bmColumn)
             ExtendGrab = 1 | 2 | 4 | 8;
-        else 
+        else
             ExtendGrab = 1 | 2;
-    }    
+    }
     return 1;
 }
 
 int EBuffer::BlockExtendEnd() {
     EPoint T, B, E;
-    
+
     CheckBlock();
     B = BB;
     E = BE;
@@ -513,9 +513,9 @@ int EBuffer::BlockExtendEnd() {
         if (ExtendGrab & 1) { B.Row = VToR(CP.Row); B.Col = 0; }
         else if (ExtendGrab & 2) { E.Row = VToR(CP.Row); E.Col = 0; }
         if (B.Row > E.Row) {
-           T = B;
-           B = E;
-           E = T;
+            T = B;
+            B = E;
+            E = T;
         }
         break;
     case bmStream:
@@ -523,9 +523,9 @@ int EBuffer::BlockExtendEnd() {
         else if (ExtendGrab & 2) { E.Col = CP.Col; E.Row = VToR(CP.Row); }
         if ((B.Row > E.Row) ||
             ((B.Row == E.Row) && (B.Col > E.Col))) {
-           T = B;
-           B = E;
-           E = T;
+            T = B;
+            B = E;
+            E = T;
         }
         break;
     case bmColumn:
@@ -535,14 +535,14 @@ int EBuffer::BlockExtendEnd() {
         else if (ExtendGrab & 8) E.Col = CP.Col;
         if (B.Row > E.Row) {
             int T;
-            
+
             T = B.Row;
             B.Row = E.Row;
             E.Row = T;
         }
         if (B.Col > E.Col) {
             int T;
-            
+
             T = B.Col;
             B.Col = E.Col;
             E.Col = T;
@@ -564,7 +564,7 @@ int EBuffer::BlockIsMarked() {
 int EBuffer::BlockReIndent() {
     EPoint P = CP;
     EPoint B, E;
-    
+
     AutoExtend = 0;
     if (CheckBlock() == 0) return 0;
     if (RCount <= 0) return 0;
@@ -583,15 +583,15 @@ int EBuffer::BlockSelectWord() {
     PELine L = RLine(Y);
     int P;
     int C;
-    
+
     if (BlockUnmark() == 0) return 0;
     BlockMode = bmStream;
-    
+
     P = CharOffset(L, CP.Col);
-    
+
     if (P >= L->Count) return 0;
     C = ChClassK(L->Chars[P]);
-    
+
     while ((P > 0) && (C == ChClassK(L->Chars[P - 1]))) P--;
     if (SetBB(EPoint(Y, ScreenPos(L, P))) == 0) return 0;
     while ((P < L->Count) && (C == ChClassK(L->Chars[P]))) P++;
@@ -603,7 +603,7 @@ int EBuffer::BlockSelectLine() {
     int Y = VToR(CP.Row);
     if (BlockUnmark() == 0) return 0;
     BlockMode = bmStream;
-    
+
     if (SetBB(EPoint(Y, 0)) == 0) return 0;
     if (Y == RCount - 1) {
         if (SetBE(EPoint(Y, LineLen(Y))) == 0) return 0;
@@ -618,40 +618,41 @@ int EBuffer::BlockSelectPara() {
 }
 
 int EBuffer::BlockWriteTo(char *AFileName, int Append) {
+    //int error = 0;
     EPoint B, E;
-    unsigned int L;
+    int L;
     PELine LL;
-    unsigned int A, Z;
+    int A, Z;
     FILE *fp;
     int bc = 0, lc = 0, oldc = 0;
-    
+
     AutoExtend = 0;
     if (CheckBlock() == 0) return 0;
     if (RCount == 0) return 0;
     B = BB;
     E = BE;
-    Msg(INFO, "Writing %s...", AFileName);
+    Msg(S_INFO, "Writing %s...", AFileName);
     fp = fopen(AFileName, Append ? "ab" : "wb");
-    if (fp == NULL) goto erroropen;
+    if (fp == NULL) goto error;
     setvbuf(fp, FileBuffer, _IOFBF, sizeof(FileBuffer));
     for (L = B.Row; L <= E.Row; L++) {
         A = -1;
         Z = -1;
         LL = RLine(L);
         switch (BlockMode) {
-          case bmLine:
+        case bmLine:
             if (L < E.Row) {
                 A = 0;
                 Z = LL->Count;
             }
             break;
-          case bmColumn:
+        case bmColumn:
             if (L < E.Row) {
                 A = CharOffset(LL, B.Col);
                 Z = CharOffset(LL, E.Col);
             }
             break;
-          case bmStream:
+        case bmStream:
             if (B.Row == E.Row) {
                 A = CharOffset(LL, B.Col);
                 Z = CharOffset(LL, E.Col);
@@ -672,7 +673,7 @@ int EBuffer::BlockWriteTo(char *AFileName, int Append) {
                 if (Z > LL->Count)
                     Z = LL->Count;
                 if (Z > A) {
-                    if (fwrite(LL->Chars + A, 1, Z - A, fp) != Z - A) {
+                    if ((int)fwrite(LL->Chars + A, 1, Z - A, fp) != Z - A) {
                         goto error;
                     } else
                         bc += Z - A;
@@ -690,18 +691,20 @@ int EBuffer::BlockWriteTo(char *AFileName, int Append) {
                     lc++;
                 }
             if (bc > 65536 + oldc) {
-                Msg(INFO, "Writing %s, %d lines, %d bytes.", AFileName, lc, bc);
+                Msg(S_INFO, "Writing %s, %d lines, %d bytes.", AFileName, lc, bc);
                 oldc = bc;
             }
         }
     }
     fclose(fp);
-    Msg(INFO, "Wrote %s, %d lines, %d bytes.", AFileName, lc, bc);
+    Msg(S_INFO, "Wrote %s, %d lines, %d bytes.", AFileName, lc, bc);
     return 1;
 error:
-    fclose(fp);
-    unlink(AFileName);
-erroropen:
+    if(fp != NULL)
+    {
+        fclose(fp);
+        unlink(AFileName);
+    }
     View->MView->Win->Choice(GPC_ERROR, "Error", 1, "O&K", "Failed to write block to %s", AFileName);
     return 0;
 }
@@ -715,8 +718,8 @@ int EBuffer::BlockReadFrom(char *AFileName, int blockMode) {
         View->MView->Win->Choice(GPC_ERROR, "Error", 1, "O&K", "File not found: %s", AFileName);
         return 0;
     }
-    
-    B = new EBuffer((EModel **)&SS, AFileName);
+
+    B = new EBuffer(0, (EModel **)&SS, AFileName);
     if (B == 0) return 0;
     B->SetFileName(AFileName, 0);
     if (B->Load() == 0) {
@@ -733,7 +736,7 @@ int EBuffer::BlockReadFrom(char *AFileName, int blockMode) {
     default:
     case bmStream: rc = BlockPasteStream(); break;
     }
-        
+
     SystemClipboard = savesys;
 
     if (rc == 0)
@@ -750,7 +753,7 @@ static int SortMaxRow;
 static int SortMinCol;
 static int SortMaxCol;
 
-static int SortProc(const void *A, const void *B) {
+static int _LNK_CONV SortProc(const void *A, const void *B) {
     int *AA = (int *)A;
     int *BB = (int *)B;
     ELine *LA = SortBuffer->RLine(*AA);
@@ -813,7 +816,7 @@ static int SortProc(const void *A, const void *B) {
 int EBuffer::BlockSort(int Reverse) {
     int rq;
     ELine *oldL;
-    
+
     if (CheckBlock() == 0) return 0;
     if (RCount == 0) return 0;
 
@@ -863,7 +866,7 @@ int EBuffer::BlockSort(int Reverse) {
     for (rq = 0; rq <= SortMaxRow - SortMinRow; rq++)
         if (DelLine(SortMinRow) == 0)
             return 0;
-    
+
     free(SortRows);
     return 1;
 }
@@ -872,7 +875,7 @@ int EBuffer::BlockUnTab() {
     EPoint B, E;
     ELine *L;
     int O, C;
-    
+
     AutoExtend = 0;
     if (CheckBlock() == 0) return 0;
     if (RCount <= 0) return 0;
@@ -886,7 +889,7 @@ int EBuffer::BlockUnTab() {
         while (O < L->Count) {
             if (L->Chars[O] == '\t') {
                 C = NextTab(C, BFI(this, BFI_TabSize));
-                
+
                 if (DelChars(i, O, 1) != 1)
                     return 0;
                 if (InsChars(i, O, C - O, 0) != 1)
@@ -906,7 +909,7 @@ int EBuffer::BlockEnTab() {
     ELine *L;
     int O, C, O1, C1;
     char tab = '\t';
-    
+
     AutoExtend = 0;
     if (CheckBlock() == 0) return 0;
     if (RCount <= 0) return 0;
@@ -990,7 +993,7 @@ int EBuffer::FindFunction(int delta, int way) {
     }
 
     //** Scan backwards from the current cursor position,
-    Msg(BUSY, "Matching %s", BFS(this, BFS_RoutineRegexp));
+    Msg(S_BUSY, "Matching %s", BFS(this, BFS_RoutineRegexp));
     line = VToR(CP.Row) + delta;
     while (line >= 0 && line < RCount) {
         L = RLine(line);
