@@ -13,8 +13,8 @@ EBuffer *SS = 0; // scrap buffer (clipboard)
 
 ///////////////////////////////////////////////////////////////////////////////
 
-EBuffer::EBuffer(EModel **ARoot, char *AName)
-    :EModel(ARoot), TP(0,0), CP(0,0), BB(-1,-1), BE(-1,-1),
+EBuffer::EBuffer(int createFlags, EModel **ARoot, const char * /*AName*/)
+    :EModel(createFlags, ARoot), TP(0,0), CP(0,0), BB(-1,-1), BE(-1,-1),
      PrevPos(-1, -1), SavedPos(-1, -1), Match(-1, -1)
 {
     Modified = 0;
@@ -83,6 +83,8 @@ EBuffer::~EBuffer() {
     if (FileName != 0 && Loaded)
         UpdateFPos(FileName, VToR(CP.Row), CP.Col);
 #endif
+    if (FileName && Loaded)
+        markIndex.storeForBuffer(this);
     
     Clear();
     free(LL);
@@ -166,7 +168,7 @@ int EBuffer::FreeUndo() {
 
 int EBuffer::Modify() {
     if (BFI(this, BFI_ReadOnly)) {
-        Msg(ERROR, "File is read-only.");
+        Msg(S_ERROR, "File is read-only.");
         return 0;
     }
     if (Modified == 0) {
@@ -205,7 +207,7 @@ int EBuffer::Modify() {
     return 1;
 }
 
-int EBuffer::LoadRegion(EPoint *A, int FH, int StripChar, int LineChar) {
+int EBuffer::LoadRegion(EPoint * /*A*/, int /*FH*/, int /*StripChar*/, int /*LineChar*/) {
     return 0;
 }
 
@@ -394,7 +396,7 @@ int EBuffer::AssertLine(int Row) {
     return 1;
 }
 
-int EBuffer::SetFileName(char *AFileName, char *AMode) {
+int EBuffer::SetFileName(const char *AFileName, const char *AMode) {
     FileOk = 0;
 
     free(FileName);
@@ -595,7 +597,7 @@ int EBuffer::DelLine(int Row, int DoMark) {
     if (VAllocated - VAllocated / 2 > VCount) {
         memmove(VV + VGap + GapSize - VAllocated / 3,
                 VV + VGap + GapSize,
-                sizeof(int) * (VCount - VGap));
+                sizeof(VV[0]) * (VCount - VGap));
         if (AllocVis(VAllocated - VAllocated / 3) == 0) return 0;
     }
     return 1;
@@ -655,7 +657,7 @@ int EBuffer::InsLine(int Row, int DoAppend, int DoMark) {
         if (AllocVis(VCount ? (VCount * 2) : 1) == 0) return 0;
         memmove(VV + VAllocated - (VCount - VGap),
                 VV + VGap,
-                sizeof(int) * (VCount - VGap));
+                sizeof(VV[0]) * (VCount - VGap));
     }
     if (VGap != VLine)
         if (MoveVGap(VLine) == 0) return 0;
@@ -664,7 +666,7 @@ int EBuffer::InsLine(int Row, int DoAppend, int DoMark) {
     VCount++;
         
 /*    if (AllocVis(VCount + 1) == 0) return 0;
-    memmove(VV + VLine + 1, VV + VLine, sizeof(int) * (VCount - VLine));
+    memmove(VV + VLine + 1, VV + VLine, sizeof(VV[0]) * (VCount - VLine));
     VCount++;
     Vis(VLine, Row - VLine);*/
     return 1;
@@ -764,7 +766,7 @@ int EBuffer::UnTabPoint(int Row, int Col) {
     return 1;
 }
 
-int EBuffer::ChgChars(int Row, int Ofs, int ACount, char *Buffer) {
+int EBuffer::ChgChars(int Row, int Ofs, int ACount, char * /*Buffer*/) {
     PELine L;
     
     assert(Row >= 0 && Row < RCount && Ofs >= 0);
@@ -891,8 +893,7 @@ int EBuffer::InsLineText(int Row, int Col, int ACount, int LCol, PELine Line) {
     }
     //fprintf(stderr, "B = %d, C = %d\n", B, C);
     C -= B;
-    if (C == 0) return 1;
-    assert(C > 0);
+    if (C <= 0) return 1;
     if (InsText(Row, Col, C, Line->Chars + B) == 0) return 0;
 //    printf("OK\n");
     return 1;
