@@ -30,7 +30,7 @@
 
 typedef unsigned char TransTable[256];
 
-#ifdef DOS
+#ifdef DOS /* 16 bit, sometime ;-) */
 #define RWBUFSIZE     8192
 #else
 #define RWBUFSIZE     32768
@@ -207,7 +207,7 @@ public:
 #endif
     
     // constructors
-    EBuffer(EModel **ARoot, char *AName);
+    EBuffer(int createFlags, EModel **ARoot, const char *AName);
     ~EBuffer();
     virtual void DeleteRelated();
     
@@ -215,7 +215,7 @@ public:
     EEditPort *GetViewVPort(EView *V);
     EEditPort *GetVPort();
     virtual int CanQuit();
-    virtual int ConfQuit(GxView *V);
+    virtual int ConfQuit(GxView *V, int multiFile = 0);
     
     virtual int GetContext();
     virtual EEventMap *GetEventMap();
@@ -341,7 +341,7 @@ public:
     int Save();
     int Reload();
     int FilePrint();
-    int SetFileName(char *AFileName, char *AMode);
+    int SetFileName(const char *AFileName, const char *AMode);
     
     int SetPos(int Col, int Row, int tabMode = tmNone);
     int SetPosR(int Col, int Row, int tabMode = tmNone);
@@ -460,6 +460,14 @@ public:
     int     MoveWordEndRight();
     int     MoveWordEndPrev();
     int     MoveWordEndNext();
+    int     MoveWordOrCapLeft();
+    int     MoveWordOrCapRight();
+    int     MoveWordOrCapPrev();
+    int     MoveWordOrCapNext();
+    int     MoveWordOrCapEndLeft();
+    int     MoveWordOrCapEndRight();
+    int     MoveWordOrCapEndPrev();
+    int     MoveWordOrCapEndNext();
 //    int     MoveWordStart();
 //    int     MoveWordEnd();
     int     MoveLineStart();
@@ -499,12 +507,17 @@ public:
     int     MoveFoldTop();
     int     MoveFoldPrev();
     int     MoveFoldNext();
+    int     MoveBeginOrNonWhite();
+    int     MoveBeginLinePageFile();
+    int     MoveEndLinePageFile();
     
     int     KillLine();
     int     KillChar();
     int     KillCharPrev();
     int     KillWord();
     int     KillWordPrev();
+    int     KillWordOrCap();
+    int     KillWordOrCapPrev();
     int     KillToLineStart();
     int     KillToLineEnd();
     int     KillBlock();
@@ -545,6 +558,9 @@ public:
     int     LineNew();
     int     LineIndent();
     int     LineTrim();
+    int     LineCenter();
+    int     FileTrim();
+    int     BlockTrim();
 
 #ifdef CONFIG_UNDOREDO
     int     CanUndo();
@@ -629,7 +645,7 @@ public:
     int     ShowPosition();
 
     int     Search(ExState &State, char *aString, int Options, int CanResume = 0);
-    int     SearchAgain(ExState &State, int Options);
+    int     SearchAgain(ExState &State, unsigned int Options);
     int     SearchReplace(ExState &State, char *aString, char *aReplaceString, int Options);
     int     Search(ExState &State);
     int     SearchB(ExState &State);
@@ -641,9 +657,9 @@ public:
     int     SearchReplaceRx(ExState &State);
     
 #ifdef CONFIG_WORD_HILIT
-    int     HilitAddWord(char *Word);
-    int     HilitFindWord(char *Word);
-    int     HilitRemoveWord(char *Word);
+    int     HilitAddWord(const char *Word);
+    int     HilitFindWord(const char *Word);
+    int     HilitRemoveWord(const char *Word);
     int     HilitWord();
 #endif
     int     SearchWord(int Flags);
@@ -717,9 +733,11 @@ public:
 
 #ifdef CONFIG_TAGS
     int FindTag(ExState &State);
-    int FindTagWord();
+    int FindTagWord(ExState &State);
 #endif
 
+    int SetCIndentStyle(ExState &State);
+
     int FindFunction(int delta, int way);
     int BlockMarkFunction();
     int IndentFunction();
@@ -727,11 +745,24 @@ public:
     int MoveFunctionNext();
     int InsertDate(ExState& state);
     int InsertUid();
+
+    int ShowHelpWord(ExState &State);
+
+    int PlaceGlobalBookmark(ExState &State);
+    int PushGlobalBookmark();
+
+    virtual int GetStrVar(int var, char *str, int buflen);
+    virtual int GetIntVar(int var, int *value);
+
+    int SetIndentWithTabs(ExState &State);
+    int FoldCreateAtRoutines();
 };
 
 extern EBuffer *SS;
 extern SearchReplaceOptions LSearch;
 
+extern int suspendLoads;
+
 int DoneEditor();
 
 EBuffer *FindFile(char *FileName);
