@@ -16,17 +16,24 @@ int EBuffer::BlockPrint() {
     int L;
     int A, Z;
     PELine LL;
-    int fd;
+    FILE *fp;
     int bc = 0, lc = 0;
+    int error = 0;
     
     AutoExtend = 0;
     if (CheckBlock() == 0) return 0;
     if (RCount == 0) return 0;
     B = BB;
     E = BE;
-    Msg(INFO, "Printing to %s...", PrintDevice);
-    fd = open(PrintDevice, O_RDWR | O_BINARY | O_CREAT | O_TRUNC, 0666);
-    if (fd == -1) goto erroropen;
+    Msg(S_INFO, "Printing to %s...", PrintDevice);
+    if (PrintDevice[0] == '|')
+        fp = popen(PrintDevice + 1, "w");
+    else
+        fp = fopen(PrintDevice, "w");
+    if (fp == NULL) {
+        Msg(S_INFO, "Failed to write to %s", PrintDevice);
+        return 0;
+    }
     for (L = B.Row; L <= E.Row; L++) {
         A = -1;
         Z = -1;
@@ -65,36 +72,46 @@ int EBuffer::BlockPrint() {
                 if (Z > LL->Count)
                     Z = LL->Count;
                 if (Z > A) {
-                    if (write(fd, LL->Chars + A, Z - A) != Z - A) {
-                        goto error;
+                    if (int(fwrite(LL->Chars + A, 1, Z - A, fp)) != Z - A) {
+                        error++;
+                        break;
                     } else
                         bc += Z - A;
                 }
             }
             if (BFI(this, BFI_AddCR) == 1)
-                if (write(fd, &cr, 1) != 1)
-                    goto error;
-                else
+                if (fwrite(&cr, 1, 1, fp) != 1) {
+                    error++;
+                    break;
+                } else
                     bc++;
             if (BFI(this, BFI_AddLF) == 1)
-                if (write(fd, &lf, 1) != 1)
-                    goto error;
-                else {
+                if (fwrite(&lf, 1, 1, fp) != 1) {
+                    error++;
+                    break;
+                } else {
                     bc++;
                     lc++;
                 }
             if ((lc % 200) == 0)
-                Msg(INFO, "Printing, %d lines, %d bytes.", lc, bc);
+                Msg(S_INFO, "Printing, %d lines, %d bytes.", lc, bc);
 
         }
     }
-    close(fd);
-    Msg(INFO, "Printing %d lines, %d bytes.", lc, bc);
-    return 1;
-error:
-    close(fd);
-erroropen:
-    Msg(INFO, "Failed to write to %s", PrintDevice);
+    if (!error) {
+        fwrite("\f\n", 2, 1, fp);
+        if (PrintDevice[0] == '|')
+            pclose(fp);
+        else
+            fclose(fp);
+        Msg(S_INFO, "Printing %d lines, %d bytes.", lc, bc);
+        return 1;
+    }
+    if (PrintDevice[0] == '|')
+        pclose(fp);
+    else
+        fclose(fp);
+    Msg(S_INFO, "Failed to write to %s", PrintDevice);
     return 0;
 }
 
@@ -103,51 +120,63 @@ int EBuffer::FilePrint() {
     static char cr = 13;
     static char lf = 10;
     int l;
-    int fd;
-    long ByteCount = 0;
+    FILE *fp;
+    unsigned long ByteCount = 0;
     int BChars;
   
-    Msg(INFO, "Printing %s to %s...", FileName, PrintDevice);
-    fd = open(PrintDevice, O_RDWR | O_BINARY | O_CREAT | O_TRUNC, 0666);
-    if (fd == -1) goto erroropen;
+    Msg(S_INFO, "Printing %s to %s...", FileName, PrintDevice);
+    if (PrintDevice[0] == '|')
+        fp = popen(PrintDevice + 1, "w");
+    else
+        fp = fopen(PrintDevice, "w");
+    if (fp == NULL) {
+        Msg(S_ERROR, "Error printing %s to %s.", FileName, PrintDevice);
+        return 0;
+    }
     BChars = 0;
     for (l = 0; l < RCount; l++) {
         if ((int) sizeof(FileBuffer) - (BChars + 2) < RLine(l)->Count) {
             if (BChars) {
                 ByteCount += BChars;
-                Msg(INFO, "Printing: %d lines, %d bytes.", l, ByteCount);
-                if (write(fd, FileBuffer, BChars) != BChars) goto fail;
+                Msg(S_INFO, "Printing: %d lines, %d bytes.", l, ByteCount);
+                if (int(fwrite(FileBuffer, 1, BChars, fp)) != BChars) goto fail;
                 BChars = 0;
             }
         }
-        if (RLine(l)->Count > sizeof(FileBuffer) - 2) {
+        if (RLine(l)->Count > int(sizeof(FileBuffer)) - 2) {
             assert(BChars == 0);
             ByteCount += RLine(l)->Count;
-            Msg(INFO, "Printing: %d lines, %d bytes.", l, ByteCount);
-            if (write(fd, RLine(l)->Chars, RLine(l)->Count) != RLine(l)->Count) goto fail;
+            Msg(S_INFO, "Printing: %d lines, %d bytes.", l, ByteCount);
+            if (int(fwrite(RLine(l)->Chars, 1, RLine(l)->Count, fp)) != RLine(l)->Count) goto fail;
         } else {
             memcpy(FileBuffer + BChars, RLine(l)->Chars, RLine(l)->Count);
             BChars += RLine(l)->Count;
         }
         if ((l < RCount - 1) || BFI(this, BFI_ForceNewLine)) {
-            assert(sizeof(FileBuffer) >= BChars + 2);
+            assert(int(sizeof(FileBuffer)) >= BChars + 2);
             if (BFI(this, BFI_AddCR) == 1) FileBuffer[BChars++] = cr;
             if (BFI(this, BFI_AddLF) == 1) FileBuffer[BChars++] = lf;
         }
     }
     if (BChars) {
         ByteCount += BChars;
-        Msg(INFO, "Printing: %d lines, %d bytes.", l, ByteCount);
-        if (write(fd, FileBuffer, BChars) != BChars) goto fail;
+        Msg(S_INFO, "Printing: %d lines, %d bytes.", l, ByteCount);
+        if (int(fwrite(FileBuffer, 1, BChars, fp)) != BChars) goto fail;
     }
     BChars = 0;
-    if (close(fd) != 0) goto fail;
-    Msg(INFO, "Printed %s.", FileName);
+    if (PrintDevice[0] == '|')
+        pclose(fp);
+    else
+        fclose(fp);
+    Msg(S_INFO, "Printed %s.", FileName);
     return 1;
 fail:
-    close(fd);
-    return 0;
-erroropen:
-    Msg(ERROR, "Error printing %s to %s.", FileName, PrintDevice);
+    if (fp != NULL) {
+        if (PrintDevice[0] == '|')
+            pclose(fp);
+        else
+            fclose(fp);
+    }
+    Msg(S_ERROR, "Error printing %s to %s.", FileName, PrintDevice);
     return 0;
 }
