@@ -11,18 +11,18 @@
 
 int EBuffer::GetMap(int Row, int *StateLen, hsState **StateMap) {
     hlState State = 0;
-    
+
     Rehilit(Row);
-    
+
     *StateLen = LineChars(Row);
     if (Row > 0) State = RLine(Row - 1)->StateE;
     if (*StateLen > 0) {
         PELine L = RLine(Row);
         int ECol;
-        
+
         *StateMap = (hsState *) malloc(*StateLen);
         if (*StateMap == 0) return 0;
-        
+
 #ifdef CONFIG_SYNTAX_HILIT
         if (BFI(this, BFI_HilitOn) == 1 && HilitProc != 0)
             HilitProc(this, Row, 0, 0, *StateLen, L, State, *StateMap, &ECol);
@@ -44,7 +44,7 @@ int EBuffer::GetMap(int Row, int *StateLen, hsState **StateMap) {
 void EBuffer::FullRedraw() { // redraw all views
     EView *V = View;
     EEditPort *W;
-    
+
     while (V) {
         W = GetViewVPort(V);
         Draw(W->TP.Row, W->TP.Row + W->Rows);
@@ -68,29 +68,29 @@ void EBuffer::Rehilit(int ToRow) {
     int HilitX;
     PELine L;
     int ECol;
-    
+
     if (StartHilit == -1)   // all ok
         return ;
-    
+
     if (BFI(this, BFI_MultiLineHilit) == 0) // everything handled in redisplay
         return;
-    
+
     if (ToRow <= StartHilit) // will be handled in redisplay
         return;
-    
+
     if (ToRow >= RCount)
         ToRow = RCount;
-    
+
     HilitX = 1;
     while ((StartHilit < RCount) && ((StartHilit < ToRow) || HilitX)) {
         L = RLine(StartHilit);
-        
+
         HilitX = 0;
         if (StartHilit > 0)
             State = RLine(StartHilit - 1)->StateE;
         else
             State = 0;
-        
+
         if (BFI(this, BFI_HilitOn) == 1 && HilitProc != 0) {
             HilitProc(this, StartHilit, 0, 0, 0, L, State, 0, &ECol);
         } else {
@@ -129,7 +129,7 @@ void EBuffer::Draw(int Row0, int RowE) {
 void EBuffer::DrawLine(TDrawBuffer B, int VRow, int C, int W, int &HilitX) {
     hlState State;
     int StartPos, EndPos;
-    
+
     HilitX = 0;
     MoveChar(B, 0, W, ' ', hcPlain_Background, W);
     //    if ((VRow == VCount - 1) && !BFI(this, BFI_ForceNewLine)) {
@@ -140,7 +140,7 @@ void EBuffer::DrawLine(TDrawBuffer B, int VRow, int C, int W, int &HilitX) {
         int Row = VToR(VRow);
         PELine L = RLine(Row);
         int ECol = 0;
-        
+
         if (Row > 0) State = RLine(Row - 1)->StateE;
         else State = 0;
 #ifdef CONFIG_SYNTAX_HILIT
@@ -162,7 +162,7 @@ void EBuffer::DrawLine(TDrawBuffer B, int VRow, int C, int W, int &HilitX) {
             int Folded = 0;
             static char fold[20];
             int l;
-            
+
             f = FindFold(Row);
             if (f != -1) {
                 if (FF[f].open == 1) {
@@ -210,8 +210,8 @@ void EBuffer::DrawLine(TDrawBuffer B, int VRow, int C, int W, int &HilitX) {
                 }
                 break;
             default:
-		StartPos = EndPos = 0;
-		break;
+                StartPos = EndPos = 0;
+                break;
             }
             if (BFI(this, BFI_SeeThruSel))
                 MoveBgAttr(B, StartPos, W, hcPlain_Selected, EndPos - StartPos);
@@ -241,19 +241,19 @@ void EBuffer::Redraw() {
     char s[256];
     ChColor SColor;
     int RowA, RowZ;
-    
+
     {
         int W1, H1;
         if (!(View && View->MView))
             return;
         View->MView->ConQuerySize(&W1, &H1);
-        
+
         if (H1 < 1 || W1 < 1) return;
     }
     //    printf("Redraw\n");
     if (CP.Row >= VCount) CP.Row = VCount - 1;
     if (CP.Row < 0) CP.Row = 0;
-    
+
     CheckBlock();
     V = View; /* check some window data */
     if (!V) {
@@ -263,53 +263,53 @@ void EBuffer::Redraw() {
     }
     if (View == 0 || View->MView == 0 || View->MView->Win == 0)
         return ;
-    
+
     for ( ; V; V = V->NextView) {
         //        printf("Checking\x7\n");
         if (V->Model != this)
             assert(1 == 0);
-        
+
         W = GetViewVPort(V);
-        
+
         if (W->Rows < 1 || W->Cols < 1)
             continue;
-        
+
         if (V == View) {
             int scrollJumpX = Min(ScrollJumpX, W->Cols / 2);
             int scrollJumpY = Min(ScrollJumpY, W->Rows / 2);
             int scrollBorderX = Min(ScrollBorderX, W->Cols / 2);
             int scrollBorderY = Min(ScrollBorderY, W->Rows / 2);
-            
+
             W->CP = CP;
             TP = W->TP;
-            
+
             if (W->ReCenter) {
                 W->TP.Row = CP.Row - W->Rows / 2;
                 W->TP.Col = CP.Col - W->Cols + 8;
                 W->ReCenter = 0;
             }
-            
+
             if (W->TP.Row + scrollBorderY > CP.Row) W->TP.Row = CP.Row - scrollJumpY + 1 - scrollBorderY;
             if (W->TP.Row + W->Rows - scrollBorderY <= CP.Row) W->TP.Row = CP.Row - W->Rows + 1 + scrollJumpY - 1 + scrollBorderY;
             if (!WeirdScroll)
                 if (W->TP.Row + W->Rows >= VCount) W->TP.Row = VCount - W->Rows;
             if (W->TP.Row < 0) W->TP.Row = 0;
-            
+
             if (W->TP.Col + scrollBorderX > CP.Col) W->TP.Col = CP.Col - scrollJumpX - scrollBorderX;
             if (W->TP.Col + W->Cols - scrollBorderX <= CP.Col) W->TP.Col = CP.Col - W->Cols + scrollJumpX + scrollBorderX;
             if (W->TP.Col < 0) W->TP.Col = 0;
-            
+
             if (W->OldTP.Row != -1 && W->OldTP.Col != -1 && RedrawToEos == 0) {
-                
+
                 if ((W->OldTP.Row != W->TP.Row) || (W->OldTP.Col != W->TP.Col)) {
                     int A, B;
                     int DeltaX, DeltaY;
                     int Rows = W->Rows;
                     int Delta1 = 0, Delta2 = 0;
-                    
+
                     DeltaY = W->TP.Row - W->OldTP.Row ;
                     DeltaX = W->TP.Col - W->OldTP.Col;
-                    
+
                     if ((DeltaX == 0) && (-Rows < DeltaY) && (DeltaY < Rows)) {
                         if (DeltaY < 0) {
                             W->ScrollY(DeltaY);
@@ -321,7 +321,7 @@ void EBuffer::Redraw() {
                             B = W->TP.Row + Rows;
                         }
                     } else {
-                        A = W->TP.Row; 
+                        A = W->TP.Row;
                         B = W->TP.Row + W->Rows;
                     }
                     if (A >= VCount) {
@@ -340,14 +340,14 @@ void EBuffer::Redraw() {
                 int A = W->TP.Row;
                 int B = A + W->Rows;
                 int Delta = 0;
-                
+
                 if (B > VCount) {
                     Delta += B - VCount;
                     B = VCount;
                 }
                 int LastV = VToR(VCount - 1);
                 int B1 = (B == VCount) ? RCount : VToR(B);
-                
+
                 if (B1 >= LastV) {
                     Delta += B1 - LastV;
                     B1 = LastV;
@@ -355,7 +355,7 @@ void EBuffer::Redraw() {
                 if (B1 < 0) B1 = 0;
                 Draw(VToR(A), B1 + Delta);
             }
-            
+
             W->OldTP = W->TP;
             TP = W->TP;
         }
@@ -363,18 +363,18 @@ void EBuffer::Redraw() {
         if (W->CP.Row < 0) W->CP.Row = 0;
         if (W->TP.Row > W->CP.Row) W->TP.Row = W->CP.Row;
         if (W->TP.Row < 0) W->TP.Row = 0;
-        
+
         if (V->MView->IsActive()) // hack
             SColor = hcStatus_Active;
         else
             SColor = hcStatus_Normal;
         MoveChar(B, 0, W->Cols, ' ', SColor, W->Cols);
-        
+
         if (V->MView->Win->GetViewContext() == V->MView) {
             V->MView->Win->SetSbVPos(W->TP.Row, W->Rows, VCount + (WeirdScroll ? W->Rows - 1 : 0));
             V->MView->Win->SetSbHPos(W->TP.Col, W->Cols, 1024 + (WeirdScroll ? W->Cols - 1 : 0));
         }
-        
+
         if (V->CurMsg == 0) {
             {
                 int CurLine = W->CP.Row;
@@ -388,9 +388,9 @@ void EBuffer::Redraw() {
                 unsigned char CurCh = 0xFF;
                 int lf = strlen(fName);
                 char CCharStr[20] = "";
-                
+
                 if (lf > 34) fName += lf - 34;
-                
+
                 if (CurPos < NumChars) {
                     CurCh = VLine(CurLine)->Chars[CurPos];
                     sprintf(CCharStr, "%3u,%02X", CurCh, CurCh);
@@ -399,7 +399,7 @@ void EBuffer::Redraw() {
                     else if (CurLine < NumLines - 1) strcpy(CCharStr, "   EOL");
                     else strcpy(CCharStr, "   EOF");
                 }
-                
+
                 sprintf(s, "%04d:%02d %c%c%c%c%c %.6s %c"
 #ifdef DOS
                         " %lu "
@@ -435,18 +435,18 @@ void EBuffer::Redraw() {
                         ,coreleft()
 #endif
                        );
-                
+
                 int l = strlen(s);
                 int fw = W->Cols - l;
                 int fl = strlen(FileName);
                 char num[10];
-                
+
                 MoveStr(B, 0, W->Cols, s, SColor, W->Cols);
                 sprintf(num, " %s %d", CCharStr, ModelNo);
                 MoveStr(B, W->Cols - strlen(num), W->Cols, num, SColor, W->Cols);
-                
+
                 fw -= strlen(num);
-                
+
                 if (fl > fw) {
                     MoveStr(B, l, W->Cols, FileName + fl - fw, SColor, W->Cols);
                 } else {
@@ -469,21 +469,21 @@ void EBuffer::Redraw() {
             }
         }
     }
-    
+
     Rehilit(VToR(CP.Row));
-    
+
     if (BFI(this, BFI_AutoHilitParen) == 1) {
         if (Match.Row == -1 && Match.Col == -1)
             HilitMatchBracket();
     }
-    
+
     //    if ((Window == WW) && (MinRedraw == -1))
     //        MaxRedraw = MinRedraw = VToR(CP.Row);
-    
+
     //printf("\n\nMinRedraw = %d, MaxRedraw = %d", MinRedraw, MaxRedraw);
     if (MinRedraw == -1)
         return;
-    
+
     //    printf("Will redraw: %d to %d, to eos = %d\n", MinRedraw, MaxRedraw, RedrawToEos);
     if (MinRedraw >= VCount) MinRedraw = VCount - 1;
     if (MinRedraw < 0) MinRedraw = 0;
@@ -494,7 +494,7 @@ void EBuffer::Redraw() {
     RowZ = MaxRedraw;
     if (MaxRedraw != -1) {
         int Delta = 0;
-        
+
         if (MaxRedraw >= RCount) {
             Delta = MaxRedraw - RCount + 1;
             MaxRedraw = RCount - 1;
@@ -505,14 +505,14 @@ void EBuffer::Redraw() {
     }
     //    puts("xxx\x7");
     //printf("\nRowA = %d, RowZ = %d", RowA, RowZ);
-    
+
     V = View;
     while (V) {
         if (V->Model != this)
             assert(1 == 0);
-        
+
         W = GetViewVPort(V);
-        
+
         for (int R = W->TP.Row; R < W->TP.Row + W->Rows; R++) {
             Row = R;
             if ((Row >= RowA) &&
@@ -528,17 +528,17 @@ void EBuffer::Redraw() {
     }
     MinRedraw = MaxRedraw = -1;
     RedrawToEos = 0;
-} 
+}
 
 int EBuffer::GetHilitWord(int len, char *str, ChColor &clr, int IgnCase) {
     char *p;
-    
+
     if (Mode == 0 || Mode->fColorize == 0)
         return 0;
-    
+
     if (len >= CK_MAXLEN)
         return 0;
-    
+
 #ifdef CONFIG_WORD_HILIT
     {
         char s[CK_MAXLEN + 1];
@@ -570,6 +570,17 @@ int EBuffer::GetHilitWord(int len, char *str, ChColor &clr, int IgnCase) {
             p += len + 1;
         }
     }
+    if (len < 128) {
+        char s[128];
+
+        memcpy(s, str, len);
+        s[len] = 0;
+        if (TagDefined(s)) {
+            clr = 0x0A;
+            return 1;
+        }
+    }
+    
     return 0;
 }
 
