@@ -31,10 +31,10 @@ static  RxNode *NewNode(int aWhat) {
     }
     return N;
 }
- 
+
 static RxNode *NewChar(char Ch) {
     RxNode *A = NewNode(RE_CHAR);
-    
+
     if (A) {
         A->fChar = (char *)malloc(1);
         A->fLen = 1;
@@ -43,7 +43,7 @@ static RxNode *NewChar(char Ch) {
     return A;
 }
 
-static RxNode *NewEscape(char **Regexp) {
+static RxNode *NewEscape(const char **const Regexp) {
     char Ch = **Regexp;
     ++*Regexp;
     switch (Ch) {
@@ -55,7 +55,7 @@ static RxNode *NewEscape(char **Regexp) {
       case 'r': Ch = '\r'; break;
       case 't': Ch = '\t'; break;
       case 'v': Ch = '\v'; break;
-      case 'e': Ch = '\e'; break;
+      case 'e': Ch = 27; break;
       case 's': return NewNode(RE_WSPACE);
       case 'S': return NewNode(RE_NWSPACE);
       case 'U': return NewNode(RE_UPPER);
@@ -68,11 +68,11 @@ static RxNode *NewEscape(char **Regexp) {
       case 'c': return NewNode(RE_NCASE);
       case 'N':
         {
-            int N = 0;
-            int A = 0;
+            unsigned int N = 0;
+            unsigned int A = 0;
             if (**Regexp == 0) return 0;
             N = toupper(**Regexp) - 48; if (N > 9) return 0;
-            (*Regexp)++; 
+            (*Regexp)++;
             A = N * 100;
             if (**Regexp == 0) return 0;
             N = toupper(**Regexp) - 48; if (N > 9) return 0;
@@ -87,11 +87,11 @@ static RxNode *NewEscape(char **Regexp) {
         break;
     case 'o':
         {
-            int N = 0;
-            int A = 0;
+            unsigned int N = 0;
+            unsigned int A = 0;
             if (**Regexp == 0) return 0;
             N = toupper(**Regexp) - 48; if (N > 7) return 0;
-            (*Regexp)++; 
+            (*Regexp)++;
             A = N * 64;
             if (**Regexp == 0) return 0;
             N = toupper(**Regexp) - 48; if (N > 7) return 0;
@@ -104,13 +104,13 @@ static RxNode *NewEscape(char **Regexp) {
             Ch = (char)A;
         }
         break;
-    case 'x': 
+    case 'x':
         {
-            int N = 0;
-            int A = 0;
+            unsigned int N = 0;
+            unsigned int A = 0;
             if (**Regexp == 0) return 0;
             N = toupper(**Regexp) - 48; if (N > 9) N = N + 48 - 65 + 10; if (N > 15) return 0;
-            (*Regexp)++; 
+            (*Regexp)++;
             A = N << 4;
             if (**Regexp == 0) return 0;
             N = toupper(**Regexp) - 48; if (N > 9) N = N + 48 - 65 + 10; if (N > 15) return 0;
@@ -128,17 +128,17 @@ static RxNode *NewEscape(char **Regexp) {
 
 #define SETOP(set,n) \
     do { \
-      set[n >> 3] |= (unsigned char)(1 << (n & 7)); \
+      set[(unsigned char)(n) >> 3] |= (unsigned char)(1 << ((unsigned char)(n) & 7)); \
     } while (0)
 
-static RxNode *NewSet(char **Regexp) {
+static RxNode *NewSet(const char ** const Regexp) {
     unsigned char set[NNN];
     int s = 0;
     int c = 0;
-    int i, xx;
+    unsigned int i, xx;
     unsigned char Ch, C1 = 0, C2 = 0;
     int doset = 0;
-    
+
     memset(set, 0, sizeof(set));
     s = 1;
     if (**Regexp == '^') {
@@ -146,7 +146,7 @@ static RxNode *NewSet(char **Regexp) {
         ++*Regexp;
     }
     c = 0;
-    
+
     while (**Regexp) {
         switch (Ch = *((*Regexp)++)) {
           case ']':
@@ -169,52 +169,52 @@ static RxNode *NewSet(char **Regexp) {
               case 'r': Ch = '\r'; break;
               case 't': Ch = '\t'; break;
               case 'v': Ch = '\v'; break;
-              case 'e': Ch = '\e'; break;
+              case 'e': Ch = 27; break;
               case 'N':
-		{
-		    int N = 0;
-		    int A = 0;
-		    if (**Regexp == 0) return 0;
-		    N = toupper(**Regexp) - 48; if (N > 9) return 0;
-		    (*Regexp)++; 
-		    A = N * 100;
-		    if (**Regexp == 0) return 0;
-		    N = toupper(**Regexp) - 48; if (N > 9) return 0;
-		    (*Regexp)++;
-		    A = A + N * 10;
-		    if (**Regexp == 0) return 0;
-		    N = toupper(**Regexp) - 48; if (N > 9) return 0;
-		    (*Regexp)++;
-		    A = A + N;
-		    Ch = (unsigned char)A;
-		}
-                break;
-              case 'o':
-		{
-		    int N = 0;
-		    int A = 0;
-		    if (**Regexp == 0) return 0;
-		    N = toupper(**Regexp) - 48; if (N > 7) return 0;
-		    (*Regexp)++; 
-		    A = N * 64;
-		    if (**Regexp == 0) return 0;
-		    N = toupper(**Regexp) - 48; if (N > 7) return 0;
-		    (*Regexp)++;
-		    A = A + N * 8;
-		    if (**Regexp == 0) return 0;
-		    N = toupper(**Regexp) - 48; if (N > 7) return 0;
-		    (*Regexp)++;
-		    A = A + N;
-		    Ch = (unsigned char)A;
-		}
-		break;
-              case 'x':
+                  {
+                      unsigned int N = 0;
+                      unsigned int A = 0;
+                      if (**Regexp == 0) return 0;
+                      N = toupper(**Regexp) - 48; if (N > 9) return 0;
+                      (*Regexp)++;
+                      A = N * 100;
+                      if (**Regexp == 0) return 0;
+                      N = toupper(**Regexp) - 48; if (N > 9) return 0;
+                      (*Regexp)++;
+                      A = A + N * 10;
+                      if (**Regexp == 0) return 0;
+                      N = toupper(**Regexp) - 48; if (N > 9) return 0;
+                      (*Regexp)++;
+                      A = A + N;
+                      Ch = (unsigned char)A;
+                  }
+                  break;
+            case 'o':
                 {
-                    int N = 0;
-                    int A = 0;
+                    unsigned int N = 0;
+                    unsigned int A = 0;
+                    if (**Regexp == 0) return 0;
+                    N = toupper(**Regexp) - 48; if (N > 7) return 0;
+                    (*Regexp)++;
+                    A = N * 64;
+                    if (**Regexp == 0) return 0;
+                    N = toupper(**Regexp) - 48; if (N > 7) return 0;
+                    (*Regexp)++;
+                    A = A + N * 8;
+                    if (**Regexp == 0) return 0;
+                    N = toupper(**Regexp) - 48; if (N > 7) return 0;
+                    (*Regexp)++;
+                    A = A + N;
+                    Ch = (unsigned char)A;
+                }
+                break;
+            case 'x':
+                {
+                    unsigned int N = 0;
+                    unsigned int A = 0;
                     if (**Regexp == 0) return 0;
                     N = toupper(**Regexp) - 48; if (N > 9) N = N + 48 - 65 + 10; if (N > 15) return 0;
-                    (*Regexp)++; 
+                    (*Regexp)++;
                     A = N << 4;
                     if (**Regexp == 0) return 0;
                     N = toupper(**Regexp) - 48; if (N > 9) N = N + 48 - 65 + 10; if (N > 15) return 0;
@@ -223,64 +223,64 @@ static RxNode *NewSet(char **Regexp) {
                     Ch = (unsigned char)A;
                 }
                 break;
-	    case 's':
-		c += 4;
-		SETOP(set, '\n');
-		SETOP(set, '\t');
-		SETOP(set, ' ');
+            case 's':
+                c += 4;
+                SETOP(set, '\n');
+                SETOP(set, '\t');
+                SETOP(set, ' ');
                 SETOP(set, '\r');
-		continue;
-	    case 'S':
-		for (xx = 0; xx <= 255; xx++) {
-		    if (xx != ' ' && xx != '\t' && xx != '\n' && xx != '\r') {
-			c++;
-			SETOP(set, xx);
-		    }
-		}
-		continue;
-	    case 'w':
-		for (xx = 0; xx <= 255; xx++) {
-		    if (isalnum(xx)) {
-			c++;
-			SETOP(set, xx);
-		    }
-		}
-		break;
-	    case 'W':
-		for (xx = 0; xx <= 255; xx++) {
-		    if (!isalnum(xx)) {
-			c++;
-			SETOP(set, xx);
-		    }
-		}
-		break;
-              case 'd':
+                continue;
+            case 'S':
+                for (xx = 0; xx <= 255; xx++) {
+                    if (xx != ' ' && xx != '\t' && xx != '\n' && xx != '\r') {
+                        c++;
+                        SETOP(set, xx);
+                    }
+                }
+                continue;
+            case 'w':
+                for (xx = 0; xx <= 255; xx++) {
+                    if (isalnum(xx)) {
+                        c++;
+                        SETOP(set, xx);
+                    }
+                }
+                break;
+            case 'W':
+                for (xx = 0; xx <= 255; xx++) {
+                    if (!isalnum(xx)) {
+                        c++;
+                        SETOP(set, xx);
+                    }
+                }
+                break;
+            case 'd':
                 for (xx = 0; xx <= 255; xx++) {
                     if (isdigit(xx)) {
-			c++;
-			SETOP(set, xx);
-		    }
-		}
-		break;
-              case 'D':
-		for (xx = 0; xx <= 255; xx++) {
-		    if (!isdigit(xx)) {
-			c++;
-			SETOP(set, xx);
-		    }
-		}
-		break;
-              case 'U':
-		for (xx = 'A'; xx <= 'Z'; xx++) {
-		    c++;
-		    SETOP(set, xx);
-		}
+                        c++;
+                        SETOP(set, xx);
+                    }
+                }
+                break;
+            case 'D':
+                for (xx = 0; xx <= 255; xx++) {
+                    if (!isdigit(xx)) {
+                        c++;
+                        SETOP(set, xx);
+                    }
+                }
+                break;
+            case 'U':
+                for (xx = 'A'; xx <= 'Z'; xx++) {
+                    c++;
+                    SETOP(set, xx);
+                }
                 continue;
-              case 'L':
-		for (xx = 'a'; xx <= 'z'; xx++) {
-		    c++;
-		    SETOP(set, xx);
-		}
+            case 'L':
+                for (xx = 'a'; xx <= 'z'; xx++) {
+                    c++;
+                    SETOP(set, xx);
+                }
                 continue;
             }
             break;
@@ -322,20 +322,20 @@ static int CountWidth(RxNode *N) {
     int w = 0;
 
     while (N) {
-	if (N->fWhat < 32) w += 0;
-	else if (N->fWhat >= 32 && N->fWhat < 64)
-	    w += 1;
-	N = N->fNext;
+        if (N->fWhat < 32) w += 0;
+        else if (N->fWhat >= 32 && N->fWhat < 64)
+            w += 1;
+        N = N->fNext;
     }
     return w;
 }
 
 static int MakeSub(RxNode **F, RxNode **N, char What) {
-//    printf("MakeSub: %c\n", What);
+    //printf("MakeSub: %c\n", What);
     if (*N) {
         RxNode *No;
         RxNode *New;
-	RxNode *Jump, *Skip;
+        RxNode *Jump, *Skip;
         RxNode *Last = (*N);
 
         if (Last->fWhat & RE_GROUP) {
@@ -343,8 +343,8 @@ static int MakeSub(RxNode **F, RxNode **N, char What) {
             int C = 1;
 
             while ((C > 0) && P) {
-//                puts("backtracking...-----");
-//		RxDump(0, P);
+                //puts("backtracking...-----");
+                //RxDump(0, P);
                 if (P->fWhat & RE_GROUP) {
                     if (P->fWhat & RE_CLOSE) C++;
                     else C--;
@@ -353,121 +353,121 @@ static int MakeSub(RxNode **F, RxNode **N, char What) {
                 if (C == 0) break;
                 P = P->fPrev;
             }
-//	    printf("P = %s, c = %d", P ? "ok":"null", C);
+            //printf("P = %s, c = %d", P ? "ok":"null", C);
             if (C != 0) return 0;
         }
-	assert(Last);
-	if (What != '?' && What != '|')
-	    if (CountWidth(Last) == 0) {
-//                puts("FAILED count");
-		return 0;
-	    }
-	switch (What) {
-	case '?':    /* BRANCH x NOTHING */
-	    New = NewNode(RE_BRANCH | RE_GREEDY | What);
-	    No = NewNode(RE_NOTHING);
-	    if (!New || !No) return 0;
-	    No->fPrev = *N;
-	    if (*N)
-		(*N)->fNext = No;
-	    New->fNext = Last;
-	    New->fPrev = Last->fPrev;
-	    Last->fPrev = New;
-	    if (New->fPrev) {
-		New->fPrev->fNext = New;
-	    } else {
-		*F = New;
-	    }
-	    New->fPtr = No;
-	    No->fPtr = New;
-	    *N = No;
-	    //puts("BRANCH ?");
-            break;
-	    
-	case '*':
-	case '@':
-	    New = NewNode(RE_BRANCH | What | ((What == '*') ? RE_GREEDY : 0));
-	    Jump = NewNode(RE_JUMP);
-	    No = NewNode(RE_NOTHING);
-	    
-	    if (!New || !No || !Jump) return 0;
-	    No->fPrev = Jump;
-	    Jump->fNext = No;
-	    Jump->fPrev = *N;
-	    if (*N)
-		(*N)->fNext = Jump;
-	    New->fNext = Last;
-	    New->fPrev = Last->fPrev;
-	    Last->fPrev = New;
-	    if (New->fPrev) {
-		New->fPrev->fNext = New;
-	    } else {
-		*F = New;
-	    }
-	    New->fPtr = No;
-	    No->fPtr = New;
-	    Jump->fPtr = New;
-	    *N = No;
-	    //puts("BRANCH *");
-            break;
-	    
-	case '#':
-	case '+':
-	    New = NewNode(RE_BRANCH | What | ((What == '+') ? RE_GREEDY : 0));
-            Skip = NewNode(RE_JUMP);
-	    Jump = NewNode(RE_JUMP);
-	    No = NewNode(RE_NOTHING);
-	    
-	    if (!New || !No || !Jump) return 0;
-	    No->fPrev = Jump;
-	    Jump->fPrev = *N;
-            Jump->fNext = No;
-	    
-	    Skip->fNext = New;
-	    New->fPrev = Skip;
-	    if (*N)
-		(*N)->fNext = Jump;
-	    New->fNext = Last;
-	    Skip->fPrev = Last->fPrev;
-	    Last->fPrev = New;
-	    if (Skip->fPrev) {
-		Skip->fPrev->fNext = Skip;
-	    } else {
-		*F = Skip;
-	    }
-	    New->fPtr = No;
-	    No->fPtr = New;
-	    Jump->fPtr = New;
-	    Skip->fPtr = Last;
-	    *N = No;
-	    //puts("BRANCH +");
-            break;
-	case '|':
-	    New = NewNode(RE_BRANCH | RE_GREEDY | What);
-	    Jump = NewNode(RE_BREAK);
-	    No = NewNode(RE_NOTHING);
-	    
-	    if (!New || !No || !Jump) return 0;
-	    No->fPrev = Jump;
-	    Jump->fNext = No;
-	    Jump->fPrev = *N;
-	    if (*N)
-		(*N)->fNext = Jump;
-	    New->fNext = Last;
-	    New->fPrev = Last->fPrev;
+        assert(Last);
+        if (What != '?' && What != '|')
+            if (CountWidth(Last) == 0) {
+                //                puts("FAILED count");
+                return 0;
+            }
+        switch (What) {
+        case '?':    /* BRANCH x NOTHING */
+            New = NewNode(RE_BRANCH | RE_GREEDY | What);
+            No = NewNode(RE_NOTHING);
+            if (!New || !No) return 0;
+            No->fPrev = *N;
+            if (*N)
+                (*N)->fNext = No;
+            New->fNext = Last;
+            New->fPrev = Last->fPrev;
             Last->fPrev = New;
-	    if (New->fPrev) {
-		New->fPrev->fNext = New;
-	    } else {
-		*F = New;
-	    }
-	    New->fPtr = No;
-	    No->fPtr = New;
-	    Jump->fPtr = New;
-	    *N = No;
-	    //puts("BRANCH |");
+            if (New->fPrev) {
+                New->fPrev->fNext = New;
+            } else {
+                *F = New;
+            }
+            New->fPtr = No;
+            No->fPtr = New;
+            *N = No;
+            //puts("BRANCH ?");
             break;
-	}
+
+        case '*':
+        case '@':
+            New = NewNode(RE_BRANCH | What | ((What == '*') ? RE_GREEDY : 0));
+            Jump = NewNode(RE_JUMP);
+            No = NewNode(RE_NOTHING);
+
+            if (!New || !No || !Jump) return 0;
+            No->fPrev = Jump;
+            Jump->fNext = No;
+            Jump->fPrev = *N;
+            if (*N)
+                (*N)->fNext = Jump;
+            New->fNext = Last;
+            New->fPrev = Last->fPrev;
+            Last->fPrev = New;
+            if (New->fPrev) {
+                New->fPrev->fNext = New;
+            } else {
+                *F = New;
+            }
+            New->fPtr = No;
+            No->fPtr = New;
+            Jump->fPtr = New;
+            *N = No;
+            //puts("BRANCH *");
+            break;
+
+        case '#':
+        case '+':
+            New = NewNode(RE_BRANCH | What | ((What == '+') ? RE_GREEDY : 0));
+            Skip = NewNode(RE_JUMP);
+            Jump = NewNode(RE_JUMP);
+            No = NewNode(RE_NOTHING);
+
+            if (!New || !No || !Jump) return 0;
+            No->fPrev = Jump;
+            Jump->fPrev = *N;
+            Jump->fNext = No;
+
+            Skip->fNext = New;
+            New->fPrev = Skip;
+            if (*N)
+                (*N)->fNext = Jump;
+            New->fNext = Last;
+            Skip->fPrev = Last->fPrev;
+            Last->fPrev = New;
+            if (Skip->fPrev) {
+                Skip->fPrev->fNext = Skip;
+            } else {
+                *F = Skip;
+            }
+            New->fPtr = No;
+            No->fPtr = New;
+            Jump->fPtr = New;
+            Skip->fPtr = Last;
+            *N = No;
+            //puts("BRANCH +");
+            break;
+        case '|':
+            New = NewNode(RE_BRANCH | RE_GREEDY | What);
+            Jump = NewNode(RE_BREAK);
+            No = NewNode(RE_NOTHING);
+
+            if (!New || !No || !Jump) return 0;
+            No->fPrev = Jump;
+            Jump->fNext = No;
+            Jump->fPrev = *N;
+            if (*N)
+                (*N)->fNext = Jump;
+            New->fNext = Last;
+            New->fPrev = Last->fPrev;
+            Last->fPrev = New;
+            if (New->fPrev) {
+                New->fPrev->fNext = New;
+            } else {
+                *F = New;
+            }
+            New->fPtr = No;
+            No->fPtr = New;
+            Jump->fPtr = New;
+            *N = No;
+            //puts("BRANCH |");
+            break;
+        }
         return 1;
     }
     return 0;
@@ -475,25 +475,25 @@ static int MakeSub(RxNode **F, RxNode **N, char What) {
 
 #define CHECK(n) do { if ((n) == 0) { return 0;} } while (0)
 
-static RxNode *RxComp(char **Regexp) {
+static RxNode *RxComp(const char **const Regexp) {
     RxNode *F = 0;
     RxNode *N = 0;
     int C;
     char Ch;
-    
+
     while (**Regexp) {
-//        puts(*Regexp);
+        //        puts(*Regexp);
         switch (Ch = (*(*Regexp)++)) {
-        case '?': 
-        case '*': 
-        case '+': 
-        case '@': 
-        case '#': 
-        case '|': 
+        case '?':
+        case '*':
+        case '+':
+        case '@':
+        case '#':
+        case '|':
             CHECK(MakeSub(&F, &N, Ch));
             break;
         case '}':
-        case ')': 
+        case ')':
             return F;
         case '{':
             CHECK(AddNode(&F, &N, NewNode(RE_GROUP | RE_OPEN)));
@@ -501,7 +501,7 @@ static RxNode *RxComp(char **Regexp) {
             while (N->fNext) N = N->fNext;
             CHECK(AddNode(&F, &N, NewNode(RE_GROUP | RE_CLOSE)));
             break;
-        case '(':        
+        case '(':
             C = ++RegCount;
             CHECK(AddNode(&F, &N, NewNode(RE_GROUP | RE_OPEN | RE_MEM | C)));
             CHECK(AddNode(&F, &N, RxComp(Regexp)));
@@ -515,7 +515,7 @@ static RxNode *RxComp(char **Regexp) {
         case '.': CHECK(AddNode(&F, &N, NewNode(RE_ANY)));       break;
         case '<': CHECK(AddNode(&F, &N, NewNode(RE_ATBOW)));     break;
         case '>': CHECK(AddNode(&F, &N, NewNode(RE_ATEOW)));     break;
-        default: 
+        default:
             --*Regexp;
             CHECK(AddNode(&F, &N, NewChar(**Regexp)));
             ++*Regexp;
@@ -529,7 +529,7 @@ RxNode *RxOptimize(RxNode *rx) {
     return rx;
 }
 
-RxNode *RxCompile(char *Regexp) {
+RxNode *RxCompile(const char *Regexp) {
     RxNode *n = 0, *x;
     if (Regexp == 0) return 0;
     RegCount = 0;
@@ -564,90 +564,90 @@ void RxFree(RxNode *n) {
 #define ChClass(x) (((((x) >= 'A') && ((x) <= 'Z')) || (((x) >= 'a') && ((x) <= 'z')) || (((x) >= '0') && ((x) <= '9')))?1:0)
 
 static RxMatchRes *match;
-static char *bop;
-static char *eop;
+static const char *bop;
+static const char *eop;
 static int flags = RX_CASE;
-static char *rex;
+static const char *rex;
 
 int RxMatch(RxNode *rx) {
     RxNode *n = rx;
 
-//    printf(">>");
+    //printf(">>");
     while (n) {
-//        printf("%-50.50s\n", rex);
-//	RxDump(1, n);
+        //printf("%-50.50s\n", rex);
+        //RxDump(1, n);
         switch (n->fWhat) {
-          case RE_NOTHING:
+        case RE_NOTHING:
             break;
-          case RE_CASE:
+        case RE_CASE:
             flags |= RX_CASE;
             break;
-          case RE_NCASE:
+        case RE_NCASE:
             flags &= ~RX_CASE;
             break;
-          case RE_ATBOL:
+        case RE_ATBOL:
             if (rex != bop) return 0;
             break;
-          case RE_ATEOL:
+        case RE_ATEOL:
             if (rex != eop) return 0;
             break;
-          case RE_ANY:
+        case RE_ANY:
             if (rex == eop) return 0;
             rex++;
             break;
-          case RE_WSPACE:
+        case RE_WSPACE:
             if (rex == eop) return 0;
             if (*rex != ' ' && *rex != '\n' && *rex != '\r' && *rex != '\t') return 0;
             rex++;
             break;
-          case RE_NWSPACE:
+        case RE_NWSPACE:
             if (rex == eop) return 0;
             if (*rex == ' ' || *rex == '\n' || *rex == '\r' || *rex == '\t') return 0;
             rex++;
             break;
-          case RE_WORD:
+        case RE_WORD:
             if (rex == eop) return 0;
             if (!isalnum(*rex)) return 0;
             rex++;
             break;
-          case RE_NWORD:
+        case RE_NWORD:
             if (rex == eop) return 0;
             if (isalnum(*rex)) return 0;
             rex++;
             break;
-          case RE_DIGIT:
+        case RE_DIGIT:
             if (rex == eop) return 0;
             if (!isdigit(*rex)) return 0;
             rex++;
             break;
-          case RE_NDIGIT:
+        case RE_NDIGIT:
             if (rex == eop) return 0;
             if (isdigit(*rex)) return 0;
             rex++;
             break;
-          case RE_UPPER:
+        case RE_UPPER:
             if (rex == eop) return 0;
             if (!isupper(*rex)) return 0;
             rex++;
             break;
-          case RE_LOWER:
+        case RE_LOWER:
             if (rex == eop) return 0;
             if (!islower(*rex)) return 0;
             rex++;
             break;
-          case RE_ATBOW:
+        case RE_ATBOW:
             if (rex >= eop) return 0;
             if (rex > bop) {
                 if ((ChClass(*rex) != 1) || (ChClass(*(rex-1)) != 0)) return 0;
             }
             break;
-          case RE_ATEOW:
+        case RE_ATEOW:
             if (rex <= bop) return 0;
             if (rex < eop) {
                 if ((ChClass(*rex) != 0) || (ChClass(*(rex-1)) != 1)) return 0;
             }
             break;
-          case RE_CHAR:
+        case RE_CHAR:
             if (rex == eop) return 0;
             if (flags & RX_CASE) {
                 if (*n->fChar != *rex) return 0;
@@ -659,22 +659,22 @@ int RxMatch(RxNode *rx) {
             }
             rex += n->fLen;
             break;
-          case RE_INSET:
+        case RE_INSET:
             if (rex == eop) return 0;
-            if ((n->fChar[(*rex) >> 3] & (1 << ((*rex) & 7))) == 0) return 0;
+            if ((n->fChar[(unsigned char)(*rex) >> 3] & (1 << ((unsigned char)(*rex) & 7))) == 0) return 0;
             rex++;
             break;
-          case RE_NOTINSET:
+        case RE_NOTINSET:
             if (rex == eop) return 0;
-            if (n->fChar[(*rex) >> 3] & (1 << ((*rex) & 7))) return 0;
+            if (n->fChar[(unsigned char)(*rex) >> 3] & (1 << ((unsigned char)(*rex) & 7))) return 0;
             rex++;
             break;
-          case RE_JUMP:
+        case RE_JUMP:
             n = n->fPtr;
             continue;
-          case RE_END:
+        case RE_END:
             return 1;
-          case RE_BREAK:
+        case RE_BREAK:
             n = n->fNext;
             if (n->fNext == 0) break;
             n = n->fNext;
@@ -695,13 +695,13 @@ int RxMatch(RxNode *rx) {
                 }
             }
             break;
-          default:
+        default:
             if (n->fWhat & RE_GROUP) {
                 if (n->fWhat & RE_MEM) {
-                    char *save = rex;
+                    const char *save = rex;
                     int b = n->fWhat & 0xFF;
                     int fl = flags;
-                    
+
                     if (RxMatch(n->fNext) == 0) {
                         flags = fl;
                         if (n->fWhat & RE_OPEN)
@@ -710,20 +710,20 @@ int RxMatch(RxNode *rx) {
                             match->Close[b] = -1;
                         return 0;
                     }
-                    
+
                     if (n->fWhat & RE_OPEN) {
-                        //                        if (match->Open[b] == -1) 
+                        //                        if (match->Open[b] == -1)
                         match->Open[b] = (int) (save - bop);
                     } else {
-                        //                        if (match->Close[b] == -1) 
+                        //                        if (match->Close[b] == -1)
                         match->Close[b] = (int) (save - bop);
                     }
                     return 1;
                 }
             } else if (n->fWhat & RE_BRANCH) {
-                char *save = rex;
+                const char *save = rex;
                 int fl = flags;
-                
+
                 if ((n->fWhat & RE_GREEDY) == 0) {
                     if (RxMatch(n->fPtr) == 1) return 1;
                     flags = fl;
@@ -745,10 +745,10 @@ int RxMatch(RxNode *rx) {
     return 0;
 }
 
-int RxTry(RxNode *rx, char *s) {
+int RxTry(RxNode *rx, const char *s) {
     int fl = flags;
     rex = s;
-    for (int i = 0; i < NSEXPS; i++) 
+    for (int i = 0; i < NSEXPS; i++)
         match->Open[i] = match->Close[i] = -1;
     if (RxMatch(rx)) {
         match->Open[0] = (int) (s - bop);
@@ -759,31 +759,48 @@ int RxTry(RxNode *rx, char *s) {
     return 0;
 }
 
-int RxExec(RxNode *Regexp, char *Data, int Len, char *Start, RxMatchRes *Match, unsigned int RxOpt) {
+int RxExec(RxNode *Regexp, const char *Data, int Len, const char *Start, RxMatchRes *Match, unsigned int RxOpt) {
     char Ch;
     if (Regexp == 0) return 0;
-    
+
     match = Match;
     bop = Data;
     eop = Data + Len;
-    
+
     flags = RxOpt;
-    
+
     for (int i = 0; i < NSEXPS; i++) Match->Open[i] = Match->Close[i] = -1;
-             
-    switch (Regexp->fWhat) {
+
+    switch (Regexp->fWhat) { // this should be more clever
     case RE_ATBOL:     // match is anchored
         return RxTry(Regexp, Start);
     case RE_CHAR:    // search for a character to match
         Ch = Regexp->fChar[0];
-        if (Start == eop) 
+        if (Start == eop)
             break;
-        while ((Start = (char *) memchr(Start, Ch, (int) (eop - Start))) != 0)
-        {
-            if (RxTry(Regexp, Start)) return 1;
-            Start++;
-            if (Start == eop) 
-                break;
+        if (flags & RX_CASE) {
+            while (1) {
+                while (Start < eop && *Start != Ch)
+                    Start++;
+                if (Start == eop)
+                    break;
+                if (RxTry(Regexp, Start))
+                    return 1;
+                if (++Start == eop)
+                    break;
+            }
+        } else {
+            Ch = toupper(Ch);
+            while (1) {
+                while (Start < eop && toupper(*Start) != Ch)
+                    Start++;
+                if (Start == eop)
+                    break;
+                if (RxTry(Regexp, Start))
+                    return 1;
+                if (++Start == eop)
+                    break;
+            }
         }
         break;
     default:         // (slow)
@@ -800,15 +817,15 @@ int RxExec(RxNode *Regexp, char *Data, int Len, char *Start, RxMatchRes *Match,
 #define FLAG_UP_NEXT     4
 #define FLAG_DOWN_NEXT   8
 
-static int add(int *len, char **s, char *a, int alen, int &flag) {
+static int add(int *len, char **s, const char *a, int alen, int &flag) {
     int NewLen = *len + alen;
     int i;
-    
+
     NewLen = NewLen * 2;
 
-    if (alen == 0) 
-	return 0;
-    
+    if (alen == 0)
+        return 0;
+
     if (*s) {
         *s = (char *) realloc(*s, NewLen);
         assert(*s);
@@ -817,61 +834,61 @@ static int add(int *len, char **s, char *a, int alen, int &flag) {
         *s = (char *) malloc(NewLen);
         assert(*s);
         memcpy(*s, a, alen);
-	*len = 0;
+        *len = 0;
     }
     if (flag & FLAG_UP_CASE) {
-	char *p = *s + *len;
+        char *p = *s + *len;
 
-	for (i = 0; i < alen; i++) {
+        for (i = 0; i < alen; i++) {
             *p = (char)toupper(*p);
-	    p++;
-	}
+            p++;
+        }
     } else if (flag & FLAG_DOWN_CASE) {
-	char *p = *s + *len;
+        char *p = *s + *len;
 
-	for (i = 0; i < alen; i++) {
+        for (i = 0; i < alen; i++) {
             *p = (char)tolower(*p);
-	    p++;
-	}
+            p++;
+        }
     }
     if (flag & FLAG_UP_NEXT) {
-	char *p = *s + *len;
-	
-	*p = (char)toupper(*p);
-	flag &= ~FLAG_UP_NEXT;
+        char *p = *s + *len;
+
+        *p = (char)toupper(*p);
+        flag &= ~FLAG_UP_NEXT;
     } else if (flag & FLAG_DOWN_NEXT) {
-	char *p = *s + *len;
-	
-	*p = (char)tolower(*p);
-	flag &= ~FLAG_DOWN_NEXT;
+        char *p = *s + *len;
+
+        *p = (char)tolower(*p);
+        flag &= ~FLAG_DOWN_NEXT;
     }
     *len += alen;
     return 0;
 }
 
-int RxReplace(char *rep, char *Src, int len, RxMatchRes match, char **Dest, int *Dlen) {
+int RxReplace(const char *rep, const char *Src, int /*len*/, RxMatchRes match, char **Dest, int *Dlen) {
     int dlen = 0;
     char *dest = 0;
     char Ch;
     int n;
     int flag = 0;
-    
+
     *Dest = 0;
     *Dlen = 0;
-//    add(&dlen, &dest, Src, match.Open[0]);
+    //    add(&dlen, &dest, Src, match.Open[0]);
     while (*rep) {
         switch (Ch = *rep++) {
-//        case '&':
-//            add(&dlen, &dest, Src + match.Open[0], match.Close[0] - match.Open[0], flag);
-//            break;
+            //        case '&':
+            //            add(&dlen, &dest, Src + match.Open[0], match.Close[0] - match.Open[0], flag);
+            //            break;
         case '\\':
             switch (Ch = *rep++) {
-	    case '0':
+            case '0':
             case '1': case '2': case '3':
             case '4': case '5': case '6':
             case '7': case '8': case '9':
                 n = Ch - 48;
-                
+
                 if (match.Open[n] != -1 && match.Close[n] != -1) {
                     add(&dlen, &dest, Src + match.Open[n], match.Close[n] - match.Open[n], flag);
                 } else return -1;
@@ -884,74 +901,74 @@ int RxReplace(char *rep, char *Src, int len, RxMatchRes match, char **Dest, int
             case 'b': Ch = '\b'; add(&dlen, &dest, &Ch, 1, flag); break;
             case 'a': Ch = '\a'; add(&dlen, &dest, &Ch, 1, flag); break;
             case 't': Ch = '\t'; add(&dlen, &dest, &Ch, 1, flag); break;
-	    case 'U': flag |= FLAG_UP_CASE; break;
-	    case 'u': flag |= FLAG_UP_NEXT; break;
-	    case 'L': flag |= FLAG_DOWN_CASE; break;
-	    case 'l': flag |= FLAG_DOWN_NEXT; break;
-	    case 'E':
-	    case 'e': flag &= ~(FLAG_UP_CASE | FLAG_DOWN_CASE); break;
-	    case 'x':
-		{
-		    int N = 0;
-		    int A = 0;
+            case 'U': flag |= FLAG_UP_CASE; break;
+            case 'u': flag |= FLAG_UP_NEXT; break;
+            case 'L': flag |= FLAG_DOWN_CASE; break;
+            case 'l': flag |= FLAG_DOWN_NEXT; break;
+            case 'E':
+            case 'e': flag &= ~(FLAG_UP_CASE | FLAG_DOWN_CASE); break;
+            case 'x':
+                {
+                    int N = 0;
+                    int A = 0;
 
-		    if (*rep == 0) return 0;
-		    N = toupper(*rep) - 48; if (N > 9) N = N + 48 - 65 + 10; if (N > 15) return 0;
-		    rep++;
-		    A = N << 4;
-		    if (*rep == 0) return 0;
-		    N = toupper(*rep) - 48; if (N > 9) N = N + 48 - 65 + 10; if (N > 15) return 0;
-		    rep++;
-		    A = A + N;
-		    Ch = (char)A;
-		}
-		add(&dlen, &dest, &Ch, 1, flag);
+                    if (*rep == 0) return 0;
+                    N = toupper(*rep) - 48; if (N > 9) N = N + 48 - 65 + 10; if (N > 15) return 0;
+                    rep++;
+                    A = N << 4;
+                    if (*rep == 0) return 0;
+                    N = toupper(*rep) - 48; if (N > 9) N = N + 48 - 65 + 10; if (N > 15) return 0;
+                    rep++;
+                    A = A + N;
+                    Ch = (char)A;
+                }
+                add(&dlen, &dest, &Ch, 1, flag);
                 break;
-	    case 'd':
-		{
-		    int N = 0;
-		    int A = 0;
+            case 'd':
+                {
+                    int N = 0;
+                    int A = 0;
 
-		    if (*rep == 0) return 0;
-		    N = toupper(*rep) - 48; if (N > 9) return 0;
-		    rep++;
-		    A = N * 100;
-		    if (*rep == 0) return 0;
-		    N = toupper(*rep) - 48; if (N > 9) return 0;
-		    rep++;
-		    A = N * 10;
-		    if (*rep == 0) return 0;
-		    N = toupper(*rep) - 48; if (N > 9) return 0;
-		    rep++;
-		    A = A + N;
-		    Ch = (char)A;
-		}
-		add(&dlen, &dest, &Ch, 1, flag);
+                    if (*rep == 0) return 0;
+                    N = toupper(*rep) - 48; if (N > 9) return 0;
+                    rep++;
+                    A = N * 100;
+                    if (*rep == 0) return 0;
+                    N = toupper(*rep) - 48; if (N > 9) return 0;
+                    rep++;
+                    A = N * 10;
+                    if (*rep == 0) return 0;
+                    N = toupper(*rep) - 48; if (N > 9) return 0;
+                    rep++;
+                    A = A + N;
+                    Ch = (char)A;
+                }
+                add(&dlen, &dest, &Ch, 1, flag);
                 break;
-	    case 'o':
-		{
-		    int N = 0;
-		    int A = 0;
+            case 'o':
+                {
+                    int N = 0;
+                    int A = 0;
 
-		    if (*rep == 0) return 0;
-		    N = toupper(*rep) - 48; if (N > 7) return 0;
-		    rep++;
-		    A = N * 64;
-		    if (*rep == 0) return 0;
-		    N = toupper(*rep) - 48; if (N > 7) return 0;
-		    rep++;
-		    A = N * 8;
-		    if (*rep == 0) return 0;
-		    N = toupper(*rep) - 48; if (N > 7) return 0;
-		    rep++;
-		    A = A + N;
-		    Ch = (char)A;
-		}
-		add(&dlen, &dest, &Ch, 1, flag);
+                    if (*rep == 0) return 0;
+                    N = toupper(*rep) - 48; if (N > 7) return 0;
+                    rep++;
+                    A = N * 64;
+                    if (*rep == 0) return 0;
+                    N = toupper(*rep) - 48; if (N > 7) return 0;
+                    rep++;
+                    A = N * 8;
+                    if (*rep == 0) return 0;
+                    N = toupper(*rep) - 48; if (N > 7) return 0;
+                    rep++;
+                    A = A + N;
+                    Ch = (char)A;
+                }
+                add(&dlen, &dest, &Ch, 1, flag);
+                break;
+            default:
+                add(&dlen, &dest, &Ch, 1, flag);
                 break;
-	    default:
-		add(&dlen, &dest, &Ch, 1, flag);
-		break;
             }
             break;
         default:
@@ -959,7 +976,7 @@ int RxReplace(char *rep, char *Src, int len, RxMatchRes match, char **Dest, int
             break;
         }
     }
-//    add(&dlen, &dest, Src + match.Close[0], len - match.Close[0]);
+    //    add(&dlen, &dest, Src + match.Close[0], len - match.Close[0]);
     *Dlen = dlen;
     *Dest = dest;
     return 0;
@@ -971,7 +988,7 @@ static void RxDump(int N, RxNode *n) {
     while (n) {
         for (int i = 0; i < N; i++) printf("    ");
         switch (n->fWhat) {
-	case RE_NOTHING:   printf("NOTHING\n"); break;
+        case RE_NOTHING:   printf("NOTHING\n"); break;
         case RE_CHAR:      printf("CHAR '%.1s'\n", n->fChar); break;
         case RE_ATBOL:     printf("^\n"); break;
         case RE_ATEOL:     printf("$\n"); break;
@@ -981,12 +998,12 @@ static void RxDump(int N, RxNode *n) {
         case RE_ATBOW:     printf("<\n"); break;
         case RE_ATEOW:     printf(">\n"); break;
         case RE_WSPACE:    printf("WSPACE\n"); break;
-	case RE_NWSPACE:   printf("NWSPACE\n"); break;
-	case RE_UPPER:     printf("UPPER\n"); break;
-	case RE_LOWER:     printf("LOWER\n"); break;
-	case RE_JUMP:      printf("JUMP\n"); break;
-	case RE_BREAK:     printf("BREAK\n"); break;
-	case RE_END:       printf("END\n"); break;
+        case RE_NWSPACE:   printf("NWSPACE\n"); break;
+        case RE_UPPER:     printf("UPPER\n"); break;
+        case RE_LOWER:     printf("LOWER\n"); break;
+        case RE_JUMP:      printf("JUMP\n"); break;
+        case RE_BREAK:     printf("BREAK\n"); break;
+        case RE_END:       printf("END\n"); break;
         default:
             if (n->fWhat & RE_GROUP) {
                 if (n->fWhat & RE_MEM) {
@@ -998,7 +1015,7 @@ static void RxDump(int N, RxNode *n) {
                 }
             } else if (n->fWhat & RE_BRANCH) {
                 if (n->fWhat & RE_GREEDY) {
-		    printf("%c\n", n->fWhat & 0xFF);
+                    printf("%c\n", n->fWhat & 0xFF);
                 } else {
                     printf("%c\n", n->fWhat & 0xFF);
                 }
@@ -1017,7 +1034,7 @@ static void RxDump(int N, RxNode *n) {
     puts("\n--- " rx " -- " st " -- "); \
     RxDump(0,a);\
     assert(rc == RxExec(a, line, strlen(line), line, &b)); \
-    RxFree(a); 
+    RxFree(a);
 
 int main() {
     RxNode *a;
@@ -1042,7 +1059,7 @@ int main() {
     TEST(0, "^aa", "baa");
     TEST(1, "^aa$" ,"aa");
     TEST(0, "^aa$", "baab");
-    TEST(1, "a*b", "aaab"); 
+    TEST(1, "a*b", "aaab");
     TEST(0, "a*b", "aaaa");
     TEST(1, "{aa}*b", "aaab");
     TEST(0, "{aa}*b", "aaaa");
@@ -1081,17 +1098,17 @@ int main() {
     TEST(1, "{a{a{a{a|a}|{a|a}a}a}a|a}", "aaaaaaaaaaaaaaaaa");
 
     while (1) {
-	printf("Regexp: "); fflush(stdout); gets(line);
-	if (!*line) break;
-	a = RxCompile(line); RxDump(0, a);
-	printf("String: "); fflush(stdout); gets(line);
-	printf("rc = %d\n", RxExec(a, line, strlen(line), line, &b));
-	for (int i = 0; i < NSEXPS; i++) {
-	    if (b.Open[i] != -1) {
-		printf("%d: %d %d\n", i, b.Open[i], b.Close[i]);
-	    }
-	}
-	RxFree(a);
+        printf("Regexp: "); fflush(stdout); gets(line);
+        if (!*line) break;
+        a = RxCompile(line); RxDump(0, a);
+        printf("String: "); fflush(stdout); gets(line);
+        printf("rc = %d\n", RxExec(a, line, strlen(line), line, &b));
+        for (int i = 0; i < NSEXPS; i++) {
+            if (b.Open[i] != -1) {
+                printf("%d: %d %d\n", i, b.Open[i], b.Close[i]);
+            }
+        }
+        RxFree(a);
     }
     return 0;
 }
