@@ -22,12 +22,12 @@ EFrame::~EFrame() {
 
 void EFrame::Update() {
     GxView *V = (GxView *)Active;
-    
+
     if (V) {
         if (CModel != ActiveModel && ActiveModel) {
             char Title[256] = ""; //fte: ";
             char STitle[256] = ""; //"fte: ";
-            
+
             ActiveModel->GetTitle((char *)(Title + 0), sizeof(Title) - 0,
                                   (char *)(STitle + 0), sizeof(STitle) - 0);
             ConSetTitle(Title, STitle);
@@ -40,15 +40,15 @@ void EFrame::Update() {
 void EFrame::UpdateMenu() {
     GxView *V = (GxView *)Active;
     EEventMap *Map = 0;
-    
+
     if (V)
         Map = V->GetEventMap();
 
     if (Map != CMap || CMap == 0) {
-        char *Menu = 0;
-        char *OMenu = 0;
+        const char *Menu = 0;
+        const char *OMenu = 0;
         // set menu
-        
+
         if (CMap)
             OMenu = CMap->GetMenu(EM_MainMenu);
         if (Map)
@@ -56,7 +56,7 @@ void EFrame::UpdateMenu() {
         if (Menu == 0)
             Menu = "Main";
         CMap = Map;
-        
+
         if (OMenu && strcmp(OMenu, Menu) == 0) {
             // ok
         } else {
@@ -65,7 +65,7 @@ void EFrame::UpdateMenu() {
     } /*else if (CMap == 0 && Map == 0) {
         SetMenu("Main");
     }*/
-    
+
     GFrame::UpdateMenu();
 }
 
@@ -73,7 +73,6 @@ EGUI::EGUI(int &argc, char **argv, int XSize, int YSize): GUI(argc, argv, XSize,
     ActiveMap = 0;
     OverrideMap = 0;
     strcpy(CharMap, "");
-    gui = this;
 }
 
 EGUI::~EGUI() {
@@ -82,14 +81,14 @@ EGUI::~EGUI() {
 int EGUI::ExecCommand(GxView *view, int Command, ExState &State) {
     if (Command & CMD_EXT)
         return ExecMacro(view, Command & ~CMD_EXT);
-    
+
     if (Command == ExFail)
         return ErFAIL;
-    
+
     if (view->IsModelView()) {
         ExModelView *V = (ExModelView *)view->Top;
         EView *View = V->View;
-        
+
         switch (Command) {
         case ExFileClose:               return FileClose(View, State);
         case ExFileCloseAll:            return FileCloseAll(View, State);
@@ -108,13 +107,14 @@ int EGUI::ExecCommand(GxView *view, int Command, ExState &State) {
     case ExWinPrev:                 return WinPrev(view);
     case ExShowEntryScreen:         return ShowEntryScreen();
     case ExRunProgram:              return RunProgram(State, view);
+    case ExRunProgramAsync:         return RunProgramAsync(State, view);
     case ExMainMenu:                return MainMenu(State, view);
     case ExShowMenu:                return ShowMenu(State, view);
     case ExLocalMenu:               return LocalMenu(view);
     case ExFrameNew:                return FrameNew();
     case ExFrameNext:               return FrameNext(view);
     case ExFramePrev:               return FramePrev(view);
-    
+
     case ExWinHSplit:               return WinHSplit(view);
     case ExWinClose:                return WinClose(view);
     case ExWinZoom:                 return WinZoom(view);
@@ -129,7 +129,7 @@ int EGUI::ExecCommand(GxView *view, int Command, ExState &State) {
             char kmaps[64] = "";
             EEventMap *m;
 
-            if (State.GetStrParam(kmaps, sizeof(kmaps)) == 0) {
+            if (State.GetStrParam(0, kmaps, sizeof(kmaps)) == 0) {
                 SetOverrideMap(0, 0);
                 return 0;
             }
@@ -151,13 +151,14 @@ int EGUI::BeginMacro(GxView *view) {
 int EGUI::ExecMacro(GxView *view, int Macro) {
     int i, j;
     ExMacro *m;
-    
+    ExState State;
+
     if (Macro == -1)
         return ErFAIL;
-    
+
     if (BeginMacro(view) == -1)
         return ErFAIL;
-    
+
     State.Macro = Macro;
     State.Pos = 0;
     m = &Macros[State.Macro];
@@ -166,11 +167,13 @@ int EGUI::ExecMacro(GxView *view, int Macro) {
         if (m->cmds[i].type != CT_COMMAND ||
             m->cmds[i].u.num == ExNop)
             continue;
-        
+
         for (j = 0; j < m->cmds[i].repeat; j++) {
             State.Pos = i + 1;
             if (ExecCommand(view, m->cmds[i].u.num, State) == 0 && !m->cmds[i].ign)
+            {
                 return ErFAIL;
+            }
         }
         State.Pos = i;
     }
@@ -188,7 +191,7 @@ void EGUI::SetMsg(char *Msg) {
         strcat(CharMap, "]");
     }
     if (ActiveModel)
-        ActiveModel->Msg(INFO, CharMap);
+        ActiveModel->Msg(S_INFO, CharMap);
 }
 
 void EGUI::SetOverrideMap(EKeyMap *aMap, char *ModeName) {
@@ -201,7 +204,7 @@ void EGUI::SetOverrideMap(EKeyMap *aMap, char *ModeName) {
 
 void EGUI::SetMap(EKeyMap *aMap, KeySel *ks) {
     char key[32] = "";
-    
+
     ActiveMap = aMap;
     if (ActiveMap == 0) {
         SetMsg(0);
@@ -218,39 +221,39 @@ void EGUI::DispatchKey(GxView *view, TEvent &Event) {
     EKeyMap *map;
     EKey *key = 0;
     char Ch;
-    
+
     if (Event.Key.Code & kfModifier)
         return;
-    
+
     LastEventChar = -1;
     if (GetCharFromEvent(Event, &Ch))
         LastEventChar = Ch;
-    
+
     if ((EventMap = view->GetEventMap()) == 0)
         return;
-    
+
     map = EventMap->KeyMap;
-    
+
     if (ActiveMap || OverrideMap) {
         map = ActiveMap;
         if (OverrideMap)
             map = OverrideMap;
-	while (map) {
-	    if ((key = map->FindKey(Event.Key.Code)) != 0) {
-		if (key->fKeyMap) {
-		    SetMap(key->fKeyMap, &key->fKey);
+        while (map) {
+            if ((key = map->FindKey(Event.Key.Code)) != 0) {
+                if (key->fKeyMap) {
+                    SetMap(key->fKeyMap, &key->fKey);
                     Event.What = evNone;
-		    return ;
-		} else {
-		    SetMap(0, &key->fKey);
+                    return ;
+                } else {
+                    SetMap(0, &key->fKey);
                     ExecMacro(view, key->Cmd);
                     Event.What = evNone;
-		    return ;
-		}
+                    return ;
+                }
             }
-//            printf("Going up\n");
+            //            printf("Going up\n");
             map = map->fParent;
-	}
+        }
         if (!OverrideMap) {
             SetMap(0, 0);
             Event.What = evNone;
@@ -261,7 +264,7 @@ void EGUI::DispatchKey(GxView *view, TEvent &Event) {
         if (map) {
             if ((key = map->FindKey(Event.Key.Code)) != 0) {
                 if (key->fKeyMap) {
-		    SetMap(key->fKeyMap, &key->fKey);
+                    SetMap(key->fKeyMap, &key->fKey);
                     Event.What = evNone;
                     return ;
                 } else {
@@ -285,6 +288,7 @@ void EGUI::DispatchCommand(GxView *view, TEvent &Event) {
     { // hack for PM toolbar
         Event.Msg.Command -= 65536 + 16384;
         BeginMacro(view);
+        ExState State;
         State.Macro = 0;
         State.Pos = 0;
         ExecCommand(view, Event.Msg.Command, State);
@@ -302,7 +306,7 @@ void EGUI::DispatchEvent(GFrame *frame, GView *view, TEvent &Event) {
     if (Event.What == evNone ||
         (Event.What == evMouseMove && Event.Mouse.Buttons == 0))
         return ;
-    
+
     if (Event.What == evNotify && Event.Msg.Command == cmPipeRead) {
         Event.Msg.Model->NotifyPipe(Event.Msg.Param1);
         return;
@@ -317,7 +321,7 @@ void EGUI::DispatchEvent(GFrame *frame, GView *view, TEvent &Event) {
                 DispatchCommand(xview, Event);
             } else {
                 switch (Event.Msg.Command) {
-                case cmClose: 
+                case cmClose:
                     {
                         assert(ActiveView != 0);
                         FrameClose(ActiveView->MView->Win);
@@ -328,6 +332,10 @@ void EGUI::DispatchEvent(GFrame *frame, GView *view, TEvent &Event) {
         }
     }
     GUI::DispatchEvent(frame, view, Event);
+#if defined(OS2) && !defined(DBMALLOC) && defined(CHECKHEAP)
+    if (_heapchk() != _HEAPOK)
+        DieError(0, "Heap memory is corrupt.");
+#endif
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -349,7 +357,7 @@ int EGUI::FileCloseX(EView *View, int CreateNew, int XClose) {
     // this should never fail!
     if (GetDefaultDirectory(View->Model, Path, sizeof(Path)) == 0)
         return 0;
-    
+
     if (View->Model->ConfQuit(View->MView->Win)) {
 
         View->Model->DeleteRelated();
@@ -365,7 +373,8 @@ int EGUI::FileCloseX(EView *View, int CreateNew, int XClose) {
 #ifdef CONFIG_OBJ_DIRECTORY
         if (ActiveModel == 0 && CreateNew) {
             EView *V = ActiveView;
-            new EDirectory(&ActiveModel, Path);
+            EModel *m = new EDirectory(0, &ActiveModel, Path);
+            assert(m != 0);
 
             do {
                 V = V->Next;
@@ -374,7 +383,7 @@ int EGUI::FileCloseX(EView *View, int CreateNew, int XClose) {
             return 0;
         }
 #endif
-        
+
         if (ActiveModel == 0) {
             StopLoop();
         }
@@ -386,8 +395,8 @@ int EGUI::FileCloseX(EView *View, int CreateNew, int XClose) {
 
 int EGUI::FileClose(EView *View, ExState &State) {
     int x = 0;
-    
-    if (State.GetIntParam(&x) == 0)
+
+    if (State.GetIntParam(View, &x) == 0)
         x = OpenAfterClose;
 
     return FileCloseX(View, x);
@@ -395,10 +404,10 @@ int EGUI::FileClose(EView *View, ExState &State) {
 
 int EGUI::FileCloseAll(EView *View, ExState &State) {
     int x = 0;
-    
-    if (State.GetIntParam(&x) == 0)
+
+    if (State.GetIntParam(View, &x) == 0)
         x = OpenAfterClose;
-    
+
     while (ActiveModel)
         if (FileCloseX(View, x, 1) == 0) return 0;
     return 1;
@@ -409,9 +418,9 @@ int EGUI::WinHSplit(GxView *View) {
     ExModelView *edit;
     EView *win;
     int W, H;
-    
+
     View->ConQuerySize(&W, &H);
-    
+
     if (H < 8)
         return 0;
     view = new GxView(View->Parent);
@@ -430,7 +439,7 @@ int EGUI::WinHSplit(GxView *View) {
 
 int EGUI::WinClose(GxView *V) {
     EView *View = ActiveView;
-    
+
     if (View->Next == View) {
         // when closing last window, close all files
         if (ExitEditor(View) == 0)
@@ -445,7 +454,7 @@ int EGUI::WinClose(GxView *V) {
 int EGUI::WinZoom(GxView *View) {
     GView *V = View->Next;
     GView *V1;
-    
+
     while (V) {
         V1 = V;
         if (V == View)
@@ -458,8 +467,8 @@ int EGUI::WinZoom(GxView *View) {
 
 int EGUI::WinResize(ExState &State, GxView *View) {
     int Delta = 1;
-    
-    if (State.GetIntParam(&Delta)) {
+
+    if (State.GetIntParam(0, &Delta)) {
         if (View->ExpandHeight(Delta) == 0)
             return 1;
     }
@@ -474,10 +483,15 @@ int EGUI::ExitEditor(EView *View) {
         if (ActiveModel->CanQuit()) ;
         else {
             View->SelectModel(ActiveModel);
-            if (ActiveModel->ConfQuit(View->MView->Win) == 0)
+            int rc = ActiveModel->ConfQuit(View->MView->Win, 1);
+            if (rc == -2) {
+                View->FileSaveAll();
+                break;
+            }
+            if (rc == 0)
                 return 0;
         }
-        
+
         ActiveModel = ActiveModel->Next;
         if (ActiveModel == B)
             break;
@@ -488,7 +502,7 @@ int EGUI::ExitEditor(EView *View) {
         SaveDesktop(DesktopFileName);
     else if (LoadDesktopMode == 2) {       // Ask about saving?
         GxView* gx = View->MView->Win;
-        
+
         if (gx->GetStr("Save desktop As",
                        sizeof(DesktopFileName), DesktopFileName,
                        HIST_DEFAULT) != 0)
@@ -504,7 +518,7 @@ int EGUI::ExitEditor(EView *View) {
         while (View->Model->Next != View->Model &&
                View->Model->Next->CanQuit())
             delete View->Model->Next;
-        
+
         View->DeleteModel(View->Model);
     }
 
@@ -521,50 +535,62 @@ int EGUI::RunProgram(ExState &State, GxView *view) {
 
     if (ActiveModel)
         SetDefaultDirectory(ActiveModel);
-    
-    if (State.GetStrParam(Cmd, sizeof(Cmd)) == 0)
+
+    if (State.GetStrParam(ActiveView, Cmd, sizeof(Cmd)) == 0)
         if (view->GetStr("Run", sizeof(Cmd), Cmd, HIST_COMPILE) == 0) return 0;
-    gui->RunProgram(Cmd);
+    gui->RunProgram(RUN_WAIT, Cmd);
+    return 1;
+}
+
+int EGUI::RunProgramAsync(ExState &State, GxView *view) {
+    static char Cmd[512] = "";
+
+    if (ActiveModel)
+        SetDefaultDirectory(ActiveModel);
+
+    if (State.GetStrParam(ActiveView, Cmd, sizeof(Cmd)) == 0)
+        if (view->GetStr("Run", sizeof(Cmd), Cmd, HIST_COMPILE) == 0) return 0;
+    gui->RunProgram(RUN_ASYNC, Cmd);
     return 1;
 }
 
 int EGUI::MainMenu(ExState &State, GxView *View) {
     char s[3];
-    
-    if (State.GetStrParam(s, sizeof(s)) == 0)
+
+    if (State.GetStrParam(0, s, sizeof(s)) == 0)
         s[0] = 0;
-    
+
     View->Parent->ExecMainMenu(s[0]);
     return 1;
 }
 
 int EGUI::ShowMenu(ExState &State, GxView *View) {
     char MName[32] = "";
-    
-    if (State.GetStrParam(MName, sizeof(MName)) == 0)
+
+    if (State.GetStrParam(0, MName, sizeof(MName)) == 0)
         return 0;
-    
+
     View->Parent->PopupMenu(MName);
     return 0;
 }
 
 int EGUI::LocalMenu(GxView *View) {
     EEventMap *Map = View->GetEventMap();
-    char *MName = 0;
-    
+    const char *MName = 0;
+
     if (Map)
         MName = Map->GetMenu(EM_LocalMenu);
-    if (MName == 0) 
+    if (MName == 0)
         MName = "Local";
     View->Parent->PopupMenu(MName);
     return 0;
 }
 
 int EGUI::DesktopSaveAs(ExState &State, GxView *view) {
-    if (State.GetStrParam(DesktopFileName, sizeof(DesktopFileName)) == 0)
+    if (State.GetStrParam(0, DesktopFileName, sizeof(DesktopFileName)) == 0)
         if (view->GetFile("Save Desktop", sizeof(DesktopFileName), DesktopFileName, HIST_PATH, GF_SAVEAS) == 0)
             return 0;
-    
+
     if (DesktopFileName[0] != 0)
         return SaveDesktop(DesktopFileName);
     return 0;
@@ -576,18 +602,18 @@ int EGUI::FrameNew() {
 
     if (!multiFrame() && frames)
         return 0;
-    
+
     new EFrame(ScreenSizeX, ScreenSizeY);
     assert(frames != 0);
-    
+
     //frames->SetMenu("Main"); //??
-    
+
     view = new GxView(frames);
     assert(view != 0);
-    
+
     new EView(ActiveModel);
     assert(ActiveView != 0);
-    
+
     edit = new ExModelView(ActiveView);
     assert(edit != 0);
     view->PushView(edit);
@@ -598,7 +624,7 @@ int EGUI::FrameNew() {
 int EGUI::FrameClose(GxView *View) {
     assert(frames != 0);
     assert(View != 0);
-    
+
     if (!frames->isLastFrame()) {
         deleteFrame(frames);
     } else {
@@ -640,7 +666,7 @@ int EGUI::findDesktop(char *argv[]) {
     switch (LoadDesktopMode) {
     default:
         //** 0: try curdir then "homedir"..
-        //	    fprintf(stderr, "ld: Mode 0\n");
+        //         fprintf(stderr, "ld: Mode 0\n");
         if (FileExists(DESKTOP_NAME))
             ExpandPath(DESKTOP_NAME, DesktopFileName);
         else {
@@ -653,40 +679,37 @@ int EGUI::findDesktop(char *argv[]) {
 #endif
         }
         return FileExists(DesktopFileName);
-        
+
     case 1:
     case 2:
         //** Try curdir, then it's owner(s)..
         ExpandPath(".", DesktopFileName);
         //fprintf(stderr, "ld: Mode 1 (start at %s)\n", DesktopFileName);
-        
+
         for (;;) {
             //** Try current location,
             char *pe = DesktopFileName + strlen(DesktopFileName);
             Slash(DesktopFileName, 1);      // Add appropriate slash
             strcat(DesktopFileName, DESKTOP_NAME);
-            
+
             //fprintf(stderr, "ld: Mode 1 (trying %s)\n", DesktopFileName);
             if (FileExists(DesktopFileName)) {
                 //fprintf(stderr, "ld: Mode 1 (using %s)\n", DesktopFileName);
                 return 1;
             }
-            
+
             //** Not found. Remove added stuff, then go level UP,
             *pe = 0;
-            
+
             // Remove the current part,
-            char *p, *p2;
-            p  = strrchr(DesktopFileName, '/');
-            p2 = strrchr(DesktopFileName, '\\');
-            if (p2 > p)
-                p = p2;              // Get LAST slash there,
+            char *p = SepRChr(DesktopFileName);
+
             if (p == NULL) {
                 //** No desktop! Set default name in current directory,
                 ExpandPath(".", DesktopFileName);
                 Slash(DesktopFileName, 1);
                 strcat(DesktopFileName, DESKTOP_NAME);
-                
+
                 SaveDesktopOnExit = 0;      // Don't save,
                 return 0;                   // NOT found!!
             }
@@ -695,10 +718,16 @@ int EGUI::findDesktop(char *argv[]) {
     }
 }
 
-void EGUI::DoLoadDesktopOnEntry(int &argc, char **argv) {
-    if (DesktopFileName[0] == 0) {
+void EGUI::DoLoadDesktopOnEntry(int &/*argc*/, char **argv) {
+    if (DesktopFileName[0] == 0)
         findDesktop(argv);
 
+    if (DesktopFileName[0] != 0) {
+        if (IsDirectory(DesktopFileName)) {
+            Slash(DesktopFileName, 1);
+            strcat(DesktopFileName, DESKTOP_NAME);
+        }
+
         if (LoadDesktopOnEntry && FileExists(DesktopFileName))
             LoadDesktop(DesktopFileName);
     }
@@ -706,7 +735,8 @@ void EGUI::DoLoadDesktopOnEntry(int &argc, char **argv) {
 #endif
 
 void EGUI::EditorInit() {
-    SS = new EBuffer((EModel **)&SS, "Scrap");
+    SS = new EBuffer(0, (EModel **)&SS, "Scrap");
+    assert(SS != 0);
     BFI(SS, BFI_Undo) = 0; // disable undo for clipboard
     ActiveModel = 0;
 }
@@ -728,19 +758,15 @@ void EGUI::DoLoadHistoryOnEntry(int &argc, char **argv) {
 #endif
     } else {
         char p[256];
-        
+
         ExpandPath(HistoryFileName, p);
         if (IsDirectory(p)) {
             Slash(p, 1);
-#ifdef UNIX
-            strcat(p, ".fte-history");
-#else
-            strcat(p, "fte.his");
-#endif
+            strcat(p, HISTORY_NAME);
         }
         strcpy(HistoryFileName, p);
     }
-    
+
     if (KeepHistory && FileExists(HistoryFileName))
         LoadHistory(HistoryFileName);
 }
@@ -761,18 +787,18 @@ int EGUI::CmdLoadFiles(int &argc, char **argv) {
     char Mode[32];
     int LCount = 0;
     int ReadOnly = 0;
-    
+
     for (int Arg = 1; Arg < argc; Arg++) {
         if (!QuoteAll && !QuoteNext && (argv[Arg][0] == '-')) {
             if (argv[Arg][1] == '-') {
                 QuoteAll = 1;
-	    } else if (argv[Arg][1] == '!') {
+            } else if (argv[Arg][1] == '!') {
                 // handled before
-            } else if (argv[Arg][1] == 'c') {
+            } else if (argv[Arg][1] == 'c' || argv[Arg][1] == 'C') {
                 // ^
-            } else if (argv[Arg][1] == 'd') {
+            } else if (argv[Arg][1] == 'D' || argv[Arg][1] == 'd') {
                 // ^
-            } else if (argv[Arg][1] == 'h') {
+            } else if (argv[Arg][1] == 'H') {
                 // ^
             } else if (argv[Arg][1] == '+') {
                 QuoteNext = 1;
@@ -788,16 +814,16 @@ int EGUI::CmdLoadFiles(int &argc, char **argv) {
             } else if (argv[Arg][1] == 'r') {
                 ReadOnly = 1;
             } else if (argv[Arg][1] == 'm') {
-               if (argv[Arg][2] == 0) {
-                   ModeOverride = 0;
-               } else {
-                   ModeOverride = 1;
-                   strcpy(Mode, argv[Arg] + 2);
-               }
+                if (argv[Arg][2] == 0) {
+                    ModeOverride = 0;
+                } else {
+                    ModeOverride = 1;
+                    strcpy(Mode, argv[Arg] + 2);
+                }
             } else if (argv[Arg][1] == 'T') {
                 TagsAdd(argv[Arg] + 2);
             } else if (argv[Arg][1] == 't') {
-                TagGoto(0, argv[Arg] + 2);
+                TagGoto(ActiveView, argv[Arg] + 2);
             } else {
                 DieError(2, "Invalid command line option %s", argv[Arg]);
                 return 0;
@@ -805,40 +831,50 @@ int EGUI::CmdLoadFiles(int &argc, char **argv) {
         } else {
             char Path[MAXPATH];
 
+            QuoteNext = 0;
             if (ExpandPath(argv[Arg], Path) == 0 && IsDirectory(Path)) {
-                new EDirectory(&ActiveModel, Path);
-                assert(ActiveModel != 0);
-                ActiveView->SwitchToModel(ActiveModel);
+                EModel *m = new EDirectory(cfAppend, &ActiveModel, Path);
+                assert(ActiveModel != 0 && m != 0);
             } else {
-                if (ModeOverride) {
-                    if (MultiFileLoad(argv[Arg], Mode, ActiveView) == 0) return 0;
-                } else {
-                    if (MultiFileLoad(argv[Arg], 0, ActiveView) == 0) return 0;
+                if (LCount != 0)
+                    suspendLoads = 1;
+                if (MultiFileLoad(cfAppend, argv[Arg],
+                                  ModeOverride ? Mode : 0,
+                                  ActiveView) == 0) {
+                    suspendLoads = 0;
+                    return 0;
                 }
-                if (((EBuffer *)ActiveModel)->Loaded == 0)
-                    ((EBuffer *)ActiveModel)->Load();
+                suspendLoads = 0;
+
                 if (GotoLine) {
-                    GotoLine = 0;
-                    ((EBuffer *)ActiveModel)->SetNearPosR(ColNum - 1, LineNum - 1);
-                } else {
-                    int r, c;
-                    
-                    if (RetrieveFPos(((EBuffer *)ActiveModel)->FileName, r, c) == 1)
-                        ((EBuffer *)ActiveModel)->SetNearPosR(c, r);
+                    if (((EBuffer *)ActiveModel)->Loaded == 0)
+                        ((EBuffer *)ActiveModel)->Load();
+                    if (GotoLine) {
+                        GotoLine = 0;
+                        ((EBuffer *)ActiveModel)->SetNearPosR(ColNum - 1, LineNum - 1);
+                    } else {
+                        int r, c;
+
+                        if (RetrieveFPos(((EBuffer *)ActiveModel)->FileName, r, c) == 1)
+                            ((EBuffer *)ActiveModel)->SetNearPosR(c, r);
+                    }
+                    //ActiveView->SelectModel(ActiveModel);
                 }
-                
                 if (ReadOnly) {
                     ReadOnly = 0;
                     BFI(((EBuffer *)ActiveModel), BFI_ReadOnly) = 1;
                 }
             }
-            QuoteNext = 0;
-            ActiveModel = ActiveModel->Next;
+            suspendLoads = 1;
+            ActiveView->SelectModel(ActiveModel->Next);
+            suspendLoads = 0;
             LCount++;
         }
     }
+    EModel *P = ActiveModel;
     while (LCount-- > 0)
-        ActiveModel = ActiveModel->Prev;
+        P = P->Prev;
+    ActiveView->SelectModel(P);
     return 1;
 }
 
@@ -851,7 +887,7 @@ int EGUI::Start(int &argc, char **argv) {
         if (rc)
             return rc;
     }
-    
+
     if (InterfaceInit(argc, argv) != 0)
         return 2;
 
@@ -860,21 +896,21 @@ int EGUI::Start(int &argc, char **argv) {
 #ifdef CONFIG_HISTORY
     DoLoadHistoryOnEntry(argc, argv);
 #endif
-    
+
 #ifdef CONFIG_DESKTOP
     DoLoadDesktopOnEntry(argc, argv);
 #endif
-    
+
     if (CmdLoadFiles(argc, argv) == 0)
         return 3;
-    
+
     if (ActiveModel == 0) {
 #ifdef CONFIG_OBJ_DIRECTORY
         char Path[MAXPATH];
-        
+
         GetDefaultDirectory(0, Path, sizeof(Path));
-        new EDirectory(&ActiveModel, Path);
-        assert(ActiveModel != 0);
+        EModel *m = new EDirectory(0, &ActiveModel, Path);
+        assert(ActiveModel != 0 && m != 0);
         ActiveView->SwitchToModel(ActiveModel);
 #else
         Usage();
@@ -887,7 +923,7 @@ int EGUI::Start(int &argc, char **argv) {
 void EGUI::EditorCleanup() {
     if (ActiveModel) {
         EModel *B, *N, *A;
-        
+
         B = A = ActiveModel;
         while (B != A) {
             N = B->Next;
@@ -896,13 +932,13 @@ void EGUI::EditorCleanup() {
         }
     }
     ActiveModel = 0;
-    
+
     delete SS;
     SS = 0;
-    
+
     if (ActiveView) {
         EView *BW, *NW, *AW;
-    
+
         BW = AW = ActiveView;
         while (BW != AW) {
             NW = BW->Next;
