@@ -9,12 +9,12 @@
 
 #include "console.h"
 
-#ifdef  NT
+#ifdef  NTCONSOLE
 #   define  WIN32_LEAN_AND_MEAN 1
 #   include <windows.h>
 #endif
 
-int CStrLen(char *p) {
+int CStrLen(const char *p) {
     int len = 0, was = 0;
     while (*p) {
         len++;
@@ -28,8 +28,7 @@ int CStrLen(char *p) {
     return len;
 }
 
-
-#ifndef NT
+#ifndef NTCONSOLE
 
 void MoveCh(PCell B, char CCh, TAttr Attr, int Count) {
     unsigned char *p = (unsigned char *) B;
@@ -55,7 +54,7 @@ void MoveChar(PCell B, int Pos, int Width, char CCh, TAttr Attr, int Count) {
     }
 }
 
-void MoveMem(PCell B, int Pos, int Width, char* Ch, TAttr Attr, int Count) {
+void MoveMem(PCell B, int Pos, int Width, const char* Ch, TAttr Attr, int Count) {
     unsigned char *p = (unsigned char *) B;
     
     if (Pos < 0) {
@@ -72,7 +71,7 @@ void MoveMem(PCell B, int Pos, int Width, char* Ch, TAttr Attr, int Count) {
     }
 }
 
-void MoveStr(PCell B, int Pos, int Width, char* Ch, TAttr Attr, int MaxCount) {
+void MoveStr(PCell B, int Pos, int Width, const char* Ch, TAttr Attr, int MaxCount) {
     unsigned char *p = (unsigned char *) B;
     
     if (Pos < 0) {
@@ -89,7 +88,7 @@ void MoveStr(PCell B, int Pos, int Width, char* Ch, TAttr Attr, int MaxCount) {
     }
 }
 
-void MoveCStr(PCell B, int Pos, int Width, char* Ch, TAttr A0, TAttr A1, int MaxCount) {
+void MoveCStr(PCell B, int Pos, int Width, const char* Ch, TAttr A0, TAttr A1, int MaxCount) {
     unsigned char *p = (unsigned char *) B;
     
     char was = 0;
@@ -162,7 +161,7 @@ void MoveCh(PCell B, char Ch, TAttr Attr, int Count) {
     }
 }
 
-void MoveChar(PCell B, int Pos, int Width, char Ch, TAttr Attr, int Count) {
+void MoveChar(PCell B, int Pos, int Width, const char Ch, TAttr Attr, int Count) {
     PCHAR_INFO p = (PCHAR_INFO) B;
     if (Pos < 0) {
         Count += Pos;
@@ -178,7 +177,7 @@ void MoveChar(PCell B, int Pos, int Width, char Ch, TAttr Attr, int Count) {
     }
 }
 
-void MoveMem(PCell B, int Pos, int Width, char* Ch, TAttr Attr, int Count) {
+void MoveMem(PCell B, int Pos, int Width, const char* Ch, TAttr Attr, int Count) {
     PCHAR_INFO p = (PCHAR_INFO) B;
     
     if (Pos < 0) {
@@ -196,7 +195,7 @@ void MoveMem(PCell B, int Pos, int Width, char* Ch, TAttr Attr, int Count) {
     }
 }
 
-void MoveStr(PCell B, int Pos, int Width, char* Ch, TAttr Attr, int MaxCount) {
+void MoveStr(PCell B, int Pos, int Width, const char* Ch, TAttr Attr, int MaxCount) {
     PCHAR_INFO p = (PCHAR_INFO) B;
     
     if (Pos < 0) {
@@ -214,10 +213,10 @@ void MoveStr(PCell B, int Pos, int Width, char* Ch, TAttr Attr, int MaxCount) {
     }
 }
 
-void MoveCStr(PCell B, int Pos, int Width, char* Ch, TAttr A0, TAttr A1, int MaxCount) {
+void MoveCStr(PCell B, int Pos, int Width, const char* Ch, TAttr A0, TAttr A1, int MaxCount) {
     PCHAR_INFO p = (PCHAR_INFO) B;
     char was;
-    TAttr A;
+    //TAttr A;
     
     if (Pos < 0) {
         MaxCount += Pos;
@@ -272,7 +271,8 @@ void MoveBgAttr(PCell B, int Pos, int Width, TAttr Attr, int Count) {
     for (p += Pos; Count > 0; Count--) {
         p->Attributes =
             ((unsigned char)(p->Attributes & 0xf)) |
-            ((unsigned char) Attr); +	  p++;
+            ((unsigned char) Attr);
+        p++;
     }
 }
 
