@@ -5,12 +5,12 @@
 #include "console.h"
 #include "gui.h"
 
-int DLGGetFile(GView *v, char *Prompt, unsigned int BufLen, char *FileName, int Flags) {
+int DLGGetFile(GView *v, const char *Prompt, unsigned int BufLen, char *FileName, int Flags) {
     assert(1==0);
     return 0;
 }
 
-int DLGPickChoice(GView *v, char *ATitle, int NSel, va_list ap, int Flags) {
+int DLGPickChoice(GView *v, const char *ATitle, int NSel, va_list ap, int Flags) {
     assert(1==0);
     return 0;
 }
@@ -25,7 +25,7 @@ int DLGGetFindReplace(GView *View, SearchReplaceOptions &sr) {
     return 0;
 }
 
-int DLGGetStr(GView *View, char *Prompt, unsigned int BufLen, char *Str, int HistId, int Flags) {
+int DLGGetStr(GView *View, const char *Prompt, unsigned int BufLen, char *Str, int HistId, int Flags) {
     assert(1 == 0);
     return 0;
 }
