@@ -20,8 +20,6 @@
 #include "console.h"
 #include "gui.h"
 
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
 
 #include <qwidget.h>
 #include <qframe.h>
@@ -34,6 +32,12 @@
 #include <qclipbrd.h>
 //#include <qfiledlg.h> // conflicts with <Xlib.h> above
 
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+
 #define DEBUGX(x) //printf x
 
 #define EDIT_BORDER 2
@@ -1795,7 +1799,7 @@ QMenuBar *QEFrame::CreateMenuBar(QWidget *parent, int Id) {
     return menu;
 }
 
-int GFrame::SetMenu(char *Name) {
+int GFrame::SetMenu(const char *Name) {
     int id = GetMenuId(Name);
 
     if (Menu)
@@ -1819,7 +1823,7 @@ int GFrame::ExecMainMenu(char Sub) {
     return 0;
 }
 
-int GFrame::PopupMenu(char *Name) {
+int GFrame::PopupMenu(const char *Name) {
     int id = GetMenuId(Name);
 
     QPopupMenu *popup = Peer->qFrame->CreatePopup(0, id, 1);
@@ -1940,11 +1944,11 @@ void GUI::ProcessEvent() {
 int GUI::Run() {
     if (Start(fArgc, fArgv) == 0) {
         doLoop = 1;
-	frames->Show();
-    
+        frames->Show();
+
         while (doLoop)
             ProcessEvent();
-        
+
         Stop();
         return 0;
     }
@@ -1960,12 +1964,13 @@ int GUI::RunProgram(char *Command) {
     int rc;
     
     if (*Command == 0)  // empty string = shell
-        strcpy(Cmd, "xterm");
+        strcpy(Cmd, "xterm &");
     else {
         strcpy(Cmd, "xterm -e ");
         strcat(Cmd, Command);
+        if (mode == RUN_ASYNC)
+            strcat(Cmd, " &");
     }
-    strcat(Cmd, " &");
     
     rc = system(Cmd);
     
@@ -2110,7 +2115,7 @@ int SetXSelection(int len, char *data) {
     return 0;
 }
 
-void DieError(int rc, char *msg, ...) {
+void DieError(int rc, const char *msg, ...) {
     va_list ap;
     
     va_start(ap, msg);
