@@ -472,10 +472,10 @@ void GView::Update() {
 void GView::Repaint() {
 }
 
-void GView::HandleEvent(TEvent &Event) {
+void GView::HandleEvent(TEvent &/*Event*/) {
 }
 
-void GView::Resize(int width, int height) {
+void GView::Resize(int /*width*/, int /*height*/) {
     Repaint();
 }
 
@@ -627,11 +627,11 @@ int GFrame::ConSplitView(GView *view, GView *newview) {
     return 0;
 }
 
-int GFrame::ConCloseView(GView *view) {
+int GFrame::ConCloseView(GView * /*view*/) {
     return 0;
 }
 
-int GFrame::ConResizeView(GView *view, int DeltaY) {
+int GFrame::ConResizeView(GView * /*view*/, int /*DeltaY*/) {
     return 0;
 }
 
@@ -845,7 +845,7 @@ int GFrame::ExecMainMenu(char Sub) {
     return 0;
 }
 
-int GFrame::SetMenu(char *Name) {
+int GFrame::SetMenu(const char *Name) {
     if (Menu) free(Menu);
     Menu = strdup(Name);
     return 0;
@@ -858,11 +858,11 @@ void GFrame::Activate() {
     frames = this;
 }
 
-int GUI::ConGrabEvents(TEventMask EventMask) {
+int GUI::ConGrabEvents(TEventMask /*EventMask*/) {
     return 0;
 }
 
-void GUI::DispatchEvent(GFrame *frame, GView *view, TEvent &Event) {
+void GUI::DispatchEvent(GFrame * /*frame*/, GView *view, TEvent &Event) {
     if (Event.What != evNone) {
         if (view)
             view->HandleEvent(Event);
@@ -883,6 +883,16 @@ int GUI::ConFlush(void) {
     return 0;
 }
 
+static inline int scrollBreak(TEvent &E)
+{
+#if defined(DOS) || defined(DOSP32)
+    // workaround until mouse code in con_dosx.cpp is finished
+    return (E.What != evMouseDown);
+#else
+    return (E.What == evMouseUp);
+#endif
+}
+
 void HandleVScroll(GView *view, TEvent &E) {
     int y; //, x
     int wY, wH;
@@ -904,7 +914,7 @@ void HandleVScroll(GView *view, TEvent &E) {
                 if (E.What & evNotify)
                     gui->DispatchEvent(frames, view, E);
             } while (E.What & evNotify);
-            if (E.What == evMouseUp) break;
+            if (scrollBreak(E)) break;
         } while (1);
     } else if (y == wY + wH - 1) {
         do {
@@ -918,7 +928,7 @@ void HandleVScroll(GView *view, TEvent &E) {
                 if (E.What & evNotify)
                     gui->DispatchEvent(frames, view, E);
             } while (E.What & evNotify);
-            if (E.What == evMouseUp) break;
+            if (scrollBreak(E)) break;
         } while (1);
     } else if (y < wY + view->Peer->SbVBegin + 1) {
         do {
@@ -932,7 +942,7 @@ void HandleVScroll(GView *view, TEvent &E) {
                 if (E.What & evNotify)
                     gui->DispatchEvent(frames, view, E);
             } while (E.What & evNotify);
-            if (E.What == evMouseUp) break;
+            if (scrollBreak(E)) break;
         } while (1);
     } else if (y > wY + view->Peer->SbVEnd + 1) {
         do {
@@ -946,7 +956,7 @@ void HandleVScroll(GView *view, TEvent &E) {
                 if (E.What & evNotify)
                     gui->DispatchEvent(frames, view, E);
             } while (E.What & evNotify);
-            if (E.What == evMouseUp) break;
+            if (scrollBreak(E)) break;
         } while (1);
     } else {
         int delta = y - 1 - view->Peer->SbVBegin - wY;
@@ -964,7 +974,7 @@ void HandleVScroll(GView *view, TEvent &E) {
                 if (E.What & evNotify)
                     gui->DispatchEvent(frames, view, E);
             } while (E.What & evNotify);
-            if (E.What == evMouseUp) break;
+            if (scrollBreak(E)) break;
         } while (1);
     }
     E.What = evNone;
@@ -991,7 +1001,7 @@ void HandleHScroll(GView *view, TEvent &E) {
                 if (E.What & evNotify)
                     gui->DispatchEvent(frames, view, E);
             } while (E.What & evNotify);
-            if (E.What == evMouseUp) break;
+            if (scrollBreak(E)) break;
         } while (1);
     } else if (x == wX + wW - 1) {
         do {
@@ -1005,7 +1015,7 @@ void HandleHScroll(GView *view, TEvent &E) {
                 if (E.What & evNotify)
                     gui->DispatchEvent(frames, view, E);
             } while (E.What & evNotify);
-            if (E.What == evMouseUp) break;
+            if (scrollBreak(E)) break;
         } while (1);
     } else if (x < wX + view->Peer->SbHBegin + 1) {
         do {
@@ -1019,7 +1029,7 @@ void HandleHScroll(GView *view, TEvent &E) {
                 if (E.What & evNotify)
                     gui->DispatchEvent(frames, view, E);
             } while (E.What & evNotify);
-            if (E.What == evMouseUp) break;
+            if (scrollBreak(E)) break;
         } while (1);
     } else if (x > wX + view->Peer->SbHEnd + 1) {
         do {
@@ -1033,7 +1043,7 @@ void HandleHScroll(GView *view, TEvent &E) {
                 if (E.What & evNotify)
                     gui->DispatchEvent(frames, view, E);
             } while (E.What & evNotify);
-            if (E.What == evMouseUp) break;
+            if (scrollBreak(E)) break;
         } while (1);
     } else {
         int delta = x - 1 - view->Peer->SbHBegin - wX;
@@ -1051,7 +1061,7 @@ void HandleHScroll(GView *view, TEvent &E) {
                 if (E.What & evNotify)
                     gui->DispatchEvent(frames, view, E);
             } while (E.What & evNotify);
-            if (E.What == evMouseUp) break;
+            if (scrollBreak(E)) break;
         } while (1);
     }
     E.What = evNone;
@@ -1064,19 +1074,14 @@ void GUI::ProcessEvent() {
     if (E.What != evNone) {
         NextEvent.What = evNone;
     }
-    if (E.What == evNone) {
-        if (ConGetEvent(evMouse | evCommand | evKeyboard, &E, 0, 1, 0) == -1)
-            //            return -1
-            ;
-    }
-    if (E.What == evNone) {
+    if (E.What == evNone &&
+       ( ConGetEvent(evMouse | evCommand | evKeyboard, &E, 0, 1, 0) == -1 ||
+          E.What == evNone )
+       )
+    {
         frames->Update();
-    again:
-        if (ConGetEvent(evMouse | evCommand | evKeyboard, &E, -1, 1, 0) == -1)
-            //    return -1
-            ;
-        if (E.What == evMouseMove && E.Mouse.Buttons == 0)
-            goto again;
+        while(ConGetEvent(evMouse | evCommand | evKeyboard, &E, -1, 1, 0) == -1 ||
+           (E.What == evMouseMove && E.Mouse.Buttons == 0));
     }
     if (E.What != evNone) {
         GView *view = frames->Active;
@@ -1207,7 +1212,7 @@ int GUI::multiFrame() {
     return 0;
 }
 
-void DieError(int rc, char *msg, ...) {
+void DieError(int rc, const char *msg, ...) {
     va_list ap;
     
     va_start(ap, msg);
