@@ -27,7 +27,7 @@
 #define hsC_CPP_String2 15
 #define hsC_CPP_ABrace  16
 
-int Hilit_C(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlState &State, hsState *StateMap, int *ECol) {
+int Hilit_C(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, hlState &State, hsState *StateMap, int *ECol) {
     ChColor *Colors = BF->Mode->fColorize->Colors;
     int j = 0;
     int firstnw = 0;
@@ -42,34 +42,34 @@ int Hilit_C(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlSta
             switch(State) {
             default:
             case hsC_Normal:
-		if (isalpha(*p) || *p == '_') {
-		    j = 0;
-		    while (((i + j) < Line->Count) &&
-			   (isalnum(Line->Chars[i+j]) ||
-			    (Line->Chars[i + j] == '_'))
-			   ) j++;
-		    if (BF->GetHilitWord(j, &Line->Chars[i], Color)) {
-		    //                        Color = hcC_Keyword;
-			State = hsC_Keyword;
-		    } else {
-			int x;
-			x = i + j;
-			while ((x < Line->Count) &&
-			       ((Line->Chars[x] == ' ') || (Line->Chars[x] == 9))) x++;
-			if ((x < Line->Count) && (Line->Chars[x] == '(')) {
-			    Color = Colors[CLR_Function];
+                if (isalpha(*p) || *p == '_') {
+                    j = 0;
+                    while (((i + j) < Line->Count) &&
+                           (isalnum(Line->Chars[i+j]) ||
+                            (Line->Chars[i + j] == '_'))
+                          ) j++;
+                    if (BF->GetHilitWord(j, &Line->Chars[i], Color)) {
+                        //                        Color = hcC_Keyword;
+                        State = hsC_Keyword;
+                    } else {
+                        int x;
+                        x = i + j;
+                        while ((x < Line->Count) &&
+                               ((Line->Chars[x] == ' ') || (Line->Chars[x] == 9))) x++;
+                        if ((x < Line->Count) && (Line->Chars[x] == '(')) {
+                            Color = Colors[CLR_Function];
                         } else if ((x < Line->Count) && (Line->Chars[x] == ':' && (x == Line->Count - 1 || Line->Chars[x + 1] != ':')) && firstnw == 1) {
                             Color = Colors[CLR_Label];
                         } else {
                             Color = Colors[CLR_Normal];
                         }
-			State = hsC_Normal;
+                        State = hsC_Normal;
                     }
-		    if (StateMap)
-			memset(StateMap + i, State, j);
-                    if (B) 
-			MoveMem(B, C - Pos, Width, Line->Chars + i, Color, j);
-		    i += j;
+                    if (StateMap)
+                        memset(StateMap + i, State, j);
+                    if (B)
+                        MoveMem(B, C - Pos, Width, Line->Chars + i, Color, j);
+                    i += j;
                     len -= j;
                     p += j;
                     C += j;
@@ -77,10 +77,10 @@ int Hilit_C(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlSta
                     continue;
                 } else if ((len >= 2) && (*p == '/') && (*(p+1) == '*')) {
                     State = hsC_Comment;
-		    Color = Colors[CLR_Comment];
+                    Color = Colors[CLR_Comment];
                     goto hilit2;
-		} else if ((len >= 2) && (*p == '/') && (*(p+1) == '/')) {
-		    State = hsC_CommentL;
+                } else if ((len >= 2) && (*p == '/') && (*(p+1) == '/')) {
+                    State = hsC_CommentL;
                     Color = Colors[CLR_Comment];
                     goto hilit2;
                 } else if ((len >= 2) && (*p == '0') && (*(p+1) == 'x')) {
@@ -191,12 +191,12 @@ int Hilit_C(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlSta
                     while (((i + j) < Line->Count) &&
                            (isalnum(Line->Chars[i+j]) ||
                             (Line->Chars[i + j] == '_'))
-                           ) j++;
+                          ) j++;
                     if (j == 7 && memcmp(Line->Chars + i, "include", 7) == 0)
                         was_include = 1;
                     if (StateMap)
                         memset(StateMap + i, State, j);
-                    if (B) 
+                    if (B)
                         MoveMem(B, C - Pos, Width, Line->Chars + i, Color, j);
                     i += j;
                     len -= j;
@@ -214,7 +214,7 @@ int Hilit_C(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlSta
                     ColorNext();
                 hilit:
                     ColorNext();
-                    continue;                              
+                    continue;
                 } else if ((len >= 2) && (*p == '0') && (*(p+1) == 'x')) {
                     Color = Colors[CLR_HexNumber];
                     ColorNext();
@@ -268,19 +268,19 @@ int Hilit_C(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlSta
 
 int IsState(hsState *Buf, hsState State, int Len) {
     int I;
-    
+
     for(I = 0; I < Len; I++)
         if (Buf[I] != State) return 0;
     return 1;
 }
 
-int LookAt(EBuffer *B, int Row, int Pos, char *What, hsState State, int NoWord) {
+int LookAt(EBuffer *B, int Row, unsigned int Pos, const char *What, hsState State, int NoWord) {
     char *P;
-    int L;
-    int StateLen;
-    hsState *StateMap;
+    unsigned int L;
+    //int StateLen;
+    //hsState *StateMap;
     int Len = strlen(What);
-    
+
     if (Row < 0 || Row >= B->RCount) return 0;
     P = B->RLine(Row)->Chars;
     L = B->RLine(Row)->Count;
@@ -298,16 +298,52 @@ int LookAt(EBuffer *B, int Row, int Pos, char *What, hsState State, int NoWord)
 
 int C_Indent = 4;
 int C_BraceOfs = 0;
+int C_ParenDelta = -1;
 int C_CaseOfs = 0;
 int C_CaseDelta = 4;
 int C_ClassOfs = 0;
 int C_ClassDelta = 4;
-int C_ColonOfs = -4;
+int C_ColonOfs = 0;//-4;
 int C_CommentOfs = 0;
 int C_CommentDelta = 1;
+int C_FirstLevelWidth = -1;
+int C_FirstLevelIndent = 4;
+int C_Continuation = 4;
+
+// this is global, unfortunately -- FIX !!!
+int EBuffer::SetCIndentStyle(ExState &State) {
+    if (State.GetIntParam(View, &C_Indent) == 0)
+        return 0;
+    if (State.GetIntParam(View, &C_BraceOfs) == 0)
+        return 0;
+    if (State.GetIntParam(View, &C_ParenDelta) == 0)
+        return 0;
+    if (State.GetIntParam(View, &C_CaseOfs) == 0)
+        return 0;
+    if (State.GetIntParam(View, &C_CaseDelta) == 0)
+        return 0;
+    if (State.GetIntParam(View, &C_ClassOfs) == 0)
+        return 0;
+    if (State.GetIntParam(View, &C_ClassDelta) == 0)
+        return 0;
+    if (State.GetIntParam(View, &C_ColonOfs) == 0)
+        return 0;
+    if (State.GetIntParam(View, &C_CommentOfs) == 0)
+        return 0;
+    if (State.GetIntParam(View, &C_CommentDelta) == 0)
+        return 0;
+    if (State.GetIntParam(View, &C_FirstLevelWidth) == 0)
+        return 0;
+    if (State.GetIntParam(View, &C_FirstLevelIndent) == 0)
+        return 0;
+    if (State.GetIntParam(View, &C_Continuation) == 0)
+        return 0;
+    return 1;
+}
 
 #define C_INDENT            C_Indent
 #define C_BRACE_OFS         C_BraceOfs
+#define C_PAREN_DELTA       C_ParenDelta
 #define C_CASE_OFS          C_CaseOfs
 #define C_CASE_DELTA        C_CaseDelta
 #define C_CLASS_OFS         C_ClassOfs
@@ -315,7 +351,9 @@ int C_CommentDelta = 1;
 #define C_COLON_OFS         C_ColonOfs
 #define C_COMMENT_OFS       C_CommentOfs
 #define C_COMMENT_DELTA     C_CommentDelta
-#define C_CONTINUATION      C_Indent
+#define C_CONTINUATION      C_Continuation
+#define C_FIRST_INDENT      C_FirstLevelIndent
+#define C_FIRST_WIDTH       C_FirstLevelWidth
 
 #define FIND_IF         0x0001
 #define FIND_SEMICOLON  0x0002
@@ -328,28 +366,30 @@ int C_CommentDelta = 1;
 //#define FIND_BEGINBLOCK 0x0100
 #define FIND_CLASS      0x0200
 #define FIND_CASE       0x0400
+#define FIND_SWITCH     0x0800
+#define FIND_QUESTION   0x1000
 
 static int CheckLabel(EBuffer *B, int Line) {
     PELine L = B->RLine(Line);
     int P = B->CharOffset(L, B->LineIndented(Line));
     int Cnt = 0;
-    
+
     if (Line > 0 && B->RLine(Line - 1)->StateE != hsC_Normal)
-	return 0;
-    
+        return 0;
+
     while ((P < L->Count) &&
-	   (L->Chars[P] == ' ' || L->Chars[P] == 9)) P++;
+           (L->Chars[P] == ' ' || L->Chars[P] == 9)) P++;
     while (P < L->Count) {
-	if (Cnt > 0)
+        if (Cnt > 0)
             if (L->Chars[P] == ':' && (Cnt == 1 || L->Chars[P + 1] != ':')) return 1;
-	if (!isalnum(L->Chars[P]) && L->Chars[P] != '_') return 0;
-	Cnt++;
-	P++;
+        if (!isalnum(L->Chars[P]) && L->Chars[P] != '_') return 0;
+        Cnt++;
+        P++;
     }
     return 0;
 }
 
-static int SearchBackMatch(int Count, EBuffer *B, int Row, hsState State, char *Open, char *Close, int *OPos, int *OLine, int matchparens = 0, int bolOnly = 0) {
+static int SearchBackMatch(int Count, EBuffer *B, int Row, hsState State, const char *Open, const char *Close, int *OPos, int *OLine, int matchparens = 0, int bolOnly = 0) {
     char *P;
     int L;
     int Pos;
@@ -359,7 +399,7 @@ static int SearchBackMatch(int Count, EBuffer *B, int Row, hsState State, char *
     hsState *StateMap;
     int CountX[3] = { 0, 0, 0 };
     int didMatch = 0;
-    
+
     *OLine = Row;
     *OPos = 0;
     while (Row >= 0) {
@@ -372,12 +412,12 @@ static int SearchBackMatch(int Count, EBuffer *B, int Row, hsState State, char *
             if (P[Pos] != ' ' && P[Pos] != 9) {
                 if (StateMap[Pos] == hsC_Normal) {
                     switch (P[Pos]) {
-                      case '{': CountX[0]--; break;
-                      case '}': CountX[0]++; break;
-                      case '(': CountX[1]--; break;
-                      case ')': CountX[1]++; break;
-                      case '[': CountX[2]--; break;
-                      case ']': CountX[2]++; break;
+                    case '{': CountX[0]--; break;
+                    case '}': CountX[0]++; break;
+                    case '(': CountX[1]--; break;
+                    case ')': CountX[1]++; break;
+                    case '[': CountX[2]--; break;
+                    case ']': CountX[2]++; break;
                     }
                 }
                 if (!matchparens ||
@@ -438,9 +478,11 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
         ColP = L - 1;
     assert(ColP >= -1 && ColP <= L);
 
+    char BolChar = ' ';
+    int BolCol = -1;
+    int BolRow = -1;
+
     while (RowP >= 0) {
-        char BolChar = ' ';
-        int BolCol = -1;
 
         P = B->RLine(RowP)->Chars;
         L = B->RLine(RowP)->Count;
@@ -448,10 +490,17 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
         if (B->GetMap(RowP, &StateLen, &StateMap) == 0)
             return 0;
         if (L > 0) while (ColP >= 0) {
-            
+
             if (StateMap[ColP] == hsC_Normal) {
                 switch (CharP = P[ColP]) {
                 case '{':
+                    if (BolChar == ':' || BolChar == ',') {
+                        CharP = BolChar;
+                        ColP = BolCol;
+                        RowP = BolRow;
+                        free(StateMap);
+                        return 1;
+                    }
                     if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0 && Count[3] == 0)
                     {
                         free(StateMap);
@@ -460,6 +509,13 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                     Count[0]--;
                     break;
                 case '}':
+                    if (BolChar == ':' || BolChar == ',') {
+                        CharP = BolChar;
+                        ColP = BolCol;
+                        RowP = BolRow;
+                        free(StateMap);
+                        return 1;
+                    }
                     if (ColP == 0) { /* speed optimization */
                         free(StateMap);
                         return 1;
@@ -512,13 +568,36 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                         {
                             BolChar = CharP;
                             BolCol = ColP;
-//                            free(StateMap);
-//                            return 1;
+                            BolRow = RowP;
+                            //                            free(StateMap);
+                            //                            return 1;
                         }
                     }
+                    if (BolChar == ',' && CharP == ':') {
+                        BolChar = ' ';
+                        BolCol = -1;
+                        BolRow = -1;
+                        break;
+                    }
+                    if ((BolChar == ':' || BolChar == ',') && (CharP == ';'/* || CharP == ','*/)) {
+                        CharP = ':';
+                        ColP = BolCol;
+                        RowP = BolRow;
+                        free(StateMap);
+                        return 1;
+                    }
+                    break;
+                case '?':
+                    //if ((Flags & FIND_QUESTION)) {
+                    if (BolChar == ':' || BolChar == ',') {
+                        BolChar = ' ';
+                        BolCol = -1;
+                        BolRow = -1;
+                    }
+                    //}
                     break;
                 }
-            } else if (BolChar == ' ' && StateMap[ColP] == hsC_Keyword) {
+            } else if (StateMap[ColP] == hsC_Keyword && (BolChar == ' ' || BolChar == ':')) {
                 if (L - ColP >= 2 &&
                     IsState(StateMap + ColP, hsC_Keyword, 2) &&
                     memcmp(P + ColP, "if", 2) == 0)
@@ -549,7 +628,7 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                     Count[3]++;
                 }
                 if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0 && Count[3] == 0) {
-                    
+
                     if ((Flags & FIND_FOR) &&
                         L - ColP >= 3 &&
                         IsState(StateMap + ColP, hsC_Keyword, 3) &&
@@ -568,7 +647,16 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                         free(StateMap);
                         return 1;
                     }
-                    if ((Flags & FIND_CASE) &&
+                    if ((Flags & FIND_SWITCH) &&
+                        L - ColP >= 6 &&
+                        IsState(StateMap + ColP, hsC_Keyword, 6) &&
+                        memcmp(P + ColP, "switch", 6) == 0)
+                    {
+                        CharP = 's';
+                        free(StateMap);
+                        return 1;
+                    }
+                    if (((Flags & FIND_CASE) || (BolChar == ':')) &&
                         (L - ColP >= 4 &&
                          IsState(StateMap + ColP, hsC_Keyword, 4) &&
                          memcmp(P + ColP, "case", 4) == 0) ||
@@ -577,30 +665,45 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                          memcmp(P + ColP, "default", 7) == 0))
                     {
                         CharP = 'c';
+                        if (BolChar == ':') {
+                            CharP = BolChar;
+                            ColP = BolCol;
+                            RowP = BolRow;
+                        }
                         free(StateMap);
                         return 1;
                     }
-                    if ((Flags & FIND_CLASS) &&
+                    if (((Flags & FIND_CLASS) || (BolChar == ':')) &&
                         (L - ColP >= 5 &&
                          IsState(StateMap + ColP, hsC_Keyword, 5) &&
                          memcmp(P + ColP, "class", 5) == 0))
                     {
                         CharP = 'l';
+                        if (BolChar == ':') {
+                            CharP = BolChar;
+                            ColP = BolCol;
+                            RowP = BolRow;
+                        }
                         free(StateMap);
                         return 1;
                     }
-                    if ((Flags & FIND_CLASS) &&
-                        (L - ColP >= 6 &&
-                         IsState(StateMap + ColP, hsC_Keyword, 6) &&
-                         memcmp(P + ColP, "public", 6) == 0) ||
-                        ((L - ColP >= 7) &&
-                         IsState(StateMap + ColP, hsC_Keyword, 7) &&
-                         memcmp(P + ColP, "private", 7) == 0) ||
-                        ((L - ColP >= 9) &&
-                         IsState(StateMap + ColP, hsC_Keyword, 9) &&
-                         memcmp(P + ColP, "protected", 9) == 0))
+                    if (((Flags & FIND_CLASS) || (BolChar == ':')) &&
+                        ((L - ColP >= 6 &&
+                          IsState(StateMap + ColP, hsC_Keyword, 6) &&
+                          memcmp(P + ColP, "public", 6) == 0) ||
+                         ((L - ColP >= 7) &&
+                          IsState(StateMap + ColP, hsC_Keyword, 7) &&
+                          memcmp(P + ColP, "private", 7) == 0) ||
+                         ((L - ColP >= 9) &&
+                          IsState(StateMap + ColP, hsC_Keyword, 9) &&
+                          memcmp(P + ColP, "protected", 9) == 0)))
                     {
                         CharP = 'p';
+                        if (BolChar == ':') {
+                            CharP = BolChar;
+                            ColP = BolCol;
+                            RowP = BolRow;
+                        }
                         free(StateMap);
                         return 1;
                     }
@@ -609,7 +712,7 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
             ColP--;
         }
         free(StateMap);
-        if (BolChar != ' ') {
+        if (BolChar != ' ' && BolChar != ':' && BolChar != ',') {
             CharP = BolChar;
             ColP = BolCol;
             return 1;
@@ -650,8 +753,8 @@ static int SkipWhite(EBuffer *B, int Bottom, int &Row, int &Col, int Flags) {
             return 0;
 
         if (L > 0) for ( ;
-                        Col >= 0 && Col < L;
-                        Col += ((Flags & SKIP_BACK) ? -1 : +1))
+                         Col >= 0 && Col < L;
+                         Col += ((Flags & SKIP_BACK) ? -1 : +1))
         {
             if (P[Col] == ' ' || P[Col] == '\t')
                 continue;
@@ -699,12 +802,12 @@ static int SkipWhite(EBuffer *B, int Bottom, int &Row, int &Col, int Flags) {
 }
 
 
-static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
+static int IndentNormal(EBuffer *B, int Line, int /*StateLen*/, hsState * /*StateMap*/) {
     int I = 0;
     int Pos, L;
 
     if (LookAt(B, Line, 0, "case", hsC_Keyword) ||
-        LookAt(B, Line, 0, "default", hsC_Keyword)) 
+        LookAt(B, Line, 0, "default", hsC_Keyword))
     {
         I = SearchBackMatch(-1, B, Line - 1, hsC_Normal, "{", "}", &Pos, &L) + C_CASE_OFS;
         return I;
@@ -717,17 +820,23 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
     } else if (LookAt(B, Line, 0, "else", hsC_Keyword)) {
         I = SearchBackMatch(-1, B, Line - 1, hsC_Keyword, "if", "else", &Pos, &L, 1);
         return I;
-    } else if (LookAt(B, Line, 0, "}", hsC_Normal)) {
+    } else if (LookAt(B, Line, 0, "}", hsC_Normal, 0)) {
         I = SearchBackMatch(-1, B, Line - 1, hsC_Normal, "{", "}", &Pos, &L, 0, 1);
         return I;
-    } else if (LookAt(B, Line, 0, ")", hsC_Normal)) {
+    } else if (LookAt(B, Line, 0, ")", hsC_Normal, 0)) {
         I = SearchBackMatch(-1, B, Line - 1, hsC_Normal, "(", ")", &Pos, &L);
-        return Pos;
-    } else if (LookAt(B, Line, 0, "]", hsC_Normal)) {
+        if (C_PAREN_DELTA >= 0)
+            return I + C_PAREN_DELTA;
+        else
+            return Pos;
+    } else if (LookAt(B, Line, 0, "]", hsC_Normal, 0)) {
         I = SearchBackMatch(-1, B, Line - 1, hsC_Normal, "[", "]", &Pos, &L);
-        return Pos;
+        if (C_PAREN_DELTA >= 0)
+            return I + C_PAREN_DELTA;
+        else
+            return Pos;
     } else {
-        char CharP = ' ';
+        char CharP = ' ', FirstCharP = ' ';
         int RowP = Line;
         int ColP = -1;
         int PrevRowP = RowP;
@@ -738,7 +847,7 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
 
         if (SkipWhite(B, Line, PrevRowP, PrevColP, SKIP_BACK) != 1)
             return 0;
-        
+
         PrevColP++;
         PRINTF(("\nPrevRowP=%d, PrevColP=%d\n", PrevRowP, PrevColP));
 
@@ -747,6 +856,9 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
                            FIND_ELSE |
                            FIND_FOR |
                            FIND_WHILE |
+                           FIND_SWITCH |
+                           FIND_CASE |
+                           //FIND_CLASS |
                            FIND_COLON |
                            FIND_SEMICOLON |
                            FIND_COMMA |
@@ -764,19 +876,27 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
 
         FirstRowP = RowP;
         FirstColP = ColP;
+        FirstCharP = CharP;
 
-        PRINTF(("Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
+        PRINTF(("\nFirstRowP=%d, FirstColP=%d, CharP=%c\n", FirstRowP, FirstColP, CharP));
 
         switch (CharP) {
+        case 'c':
+            I = B->LineIndented(RowP) + C_CONTINUATION;
+            return I;
+
         case '(':
         case '[':
-
-            ColP++;
-            if (SkipWhite(B, Line, RowP, ColP, SKIP_FORWARD | SKIP_LINE) != 1)
-                return 0;
-            I = B->ScreenPos(B->RLine(RowP), ColP);
+            if (C_PAREN_DELTA >= 0) {
+                I = B->LineIndented(FirstRowP) + C_PAREN_DELTA;
+            } else {
+                ColP++;
+                if (SkipWhite(B, Line, RowP, ColP, SKIP_FORWARD | SKIP_LINE) != 1)
+                    return 0;
+                I = B->ScreenPos(B->RLine(RowP), ColP);
+            }
             return I;
-            
+
         case '{':
             ColP++;
             if (((PrevRowP != RowP) ||
@@ -786,28 +906,35 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
             ColP--; ColP--;
             if (SkipWhite(B, Line, RowP, ColP, SKIP_BACK | SKIP_TOBOL | SKIP_MATCH) != 1)
                 return 0;
-            I = B->LineIndented(RowP) + C_INDENT;
+            I = B->LineIndented(RowP);
+            if (B->LineIndented(FirstRowP) <= C_FIRST_WIDTH)
+                I += C_FIRST_INDENT;
+            else
+                I += C_INDENT;
             PRINTF(("'{' indent : Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
 
-            if (!LookAt(B, Line, 0, "{", hsC_Normal, 0))
+            if (LookAt(B, Line, 0, "{", hsC_Normal, 0))
                 I -= C_BRACE_OFS;
+            else
+                I += ContinuationIndent;
 
-            return I + ContinuationIndent;
-
+            return I;
+        case ',':
+            I = B->LineIndented(FirstRowP);
+            return I;
         case '}':
             ColP++;
             ColP++;
             /*---nobreak---*/
         case ';':
-        case ',':
             ColP--;
             if (FindPrevIndent(B, RowP, ColP, CharP,
                                ((CharP == ',') ? FIND_COMMA | FIND_COLON :
-//                                (CharP == ';') ? FIND_SEMICOLON | FIND_COLON :
+                                //(CharP == ';') ? FIND_SEMICOLON | FIND_COLON :
                                 FIND_SEMICOLON | FIND_COLON)) != 1)
             {
                 if (FirstRowP != PrevRowP)
-                        ContinuationIndent = C_CONTINUATION;
+                    ContinuationIndent = C_CONTINUATION;
                 I = 0;
                 if (LookAt(B, Line, 0, "{", hsC_Normal, 0)) {
                     I += C_BRACE_OFS;
@@ -815,6 +942,8 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
                 }
                 return I + ContinuationIndent;
             }
+            PRINTF(("';' Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
+
             switch (CharP) {
             case ',':
             case ';':
@@ -825,7 +954,9 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
                     return 0;
                 //ColP--;
                 //if (SkipWhite(B, RowP, ColP, SKIP_BACK) != 1)
-                //    return 0;
+                //if (CharP == ':') {
+                //    I -= C_COLON_OFS;
+                //}
                 PRINTF(("';' indent : Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
                 I = B->LineIndented(RowP);
                 if (((PrevRowP != RowP) ||
@@ -834,7 +965,7 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
                     ContinuationIndent = C_CONTINUATION;
 
                 if (LookAt(B, Line, 0, "{", hsC_Normal, 0)) {
-                    //I += C_BRACE_OFS;
+                    //I -= C_BRACE_OFS;
                     ContinuationIndent = 0;
                 }
                 if (LookAt(B, Line, 0, "{", hsC_Normal, 0)
@@ -859,62 +990,65 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
             }
             PRINTF(("';' -- indent : Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
 
-//            else
-//            if (LookAt(B, Line, 0, "{", hsC_Normal, 0))
-//                I += C_INDENT - C_BRACE_OFS;
+            //            else
+            //            if (LookAt(B, Line, 0, "{", hsC_Normal, 0))
+            //                I += C_INDENT - C_BRACE_OFS;
 
             return I + ContinuationIndent;
 
         case ':':
             ColP--;
-            if (FindPrevIndent(B, RowP, ColP, CharP, FIND_SEMICOLON | FIND_COLON | FIND_CLASS | FIND_CASE) != 1) {
+            if (FindPrevIndent(B, RowP, ColP, CharP, FIND_SEMICOLON | FIND_COLON | FIND_QUESTION | FIND_CLASS | FIND_CASE) != 1) {
                 if (FirstRowP != PrevRowP)
                     ContinuationIndent = C_CONTINUATION;
                 return 0 + ContinuationIndent;
             }
+
+            PRINTF(("':' Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
+
             switch (CharP) {
             case ':':
-                ColP++;
-                //if (SkipWhite(B, Line, RowP, ColP, SKIP_FORWARD) != 1)
-                //    return 0;
-                I = B->LineIndented(RowP) - C_COLON_OFS;
-                PRINTF(("':' 0 indent : Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
-                break;
+                //ColP++;
+                /*if (SkipWhite(B, Line, RowP, ColP, SKIP_FORWARD) != 1)
+                 return 0;
+                 I = B->LineIndented(RowP);// - C_COLON_OFS;
+                 PRINTF(("':' 0 indent : Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
+                 break;*/
             case '{':
             case ';':
                 ColP++;
                 if (SkipWhite(B, Line, RowP, ColP, SKIP_FORWARD) != 1)
                     return 0;
                 I = B->LineIndented(RowP);
-                if (FirstRowP == RowP) {
+                PRINTF(("!!! FirstRowP=%d, PrevRowP=%d, RowP=%d, I=%d\n", FirstRowP, PrevRowP, RowP, I));
+                PRINTF(("!!! FirstColP=%d, PrevColP=%d, ColP=%d\n", FirstColP, PrevColP, ColP));
+                if (CheckLabel(B, RowP))
                     I -= C_COLON_OFS;
-                }
+                else if (PrevRowP == RowP && FirstRowP == PrevRowP && FirstColP + 1 == PrevColP)
+                    I += C_CONTINUATION;
                 if (LookAt(B, Line, 0, "{", hsC_Normal, 0)
                     && LookAt(B, RowP, ColP, "{", hsC_Normal, 0))
-                    I -= 0; //C_BRACE_OFS;
+                    I -= 0;//C_BRACE_OFS;
                 else if (LookAt(B, Line, 0, "{", hsC_Normal, 0)
                          && !LookAt(B, RowP, ColP, "{", hsC_Normal, 0))
                     I += C_BRACE_OFS;
                 else if (!LookAt(B, Line, 0, "{", hsC_Normal, 0)
                          && LookAt(B, RowP, ColP, "{", hsC_Normal, 0))
                     I -= C_BRACE_OFS;
-///                if (LookAt(B, Line, 0, "{", hsC_Normal, 0)) {
-///                    I += C_INDENT - C_BRACE_OFS;
-///                }
                 PRINTF(("':' 1 indent : Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
                 break;
             case 'p':
                 ColP++;
                 //if (SkipWhite(B, Line, RowP, ColP, SKIP_FORWARD) != 1)
-//                    return 0;
+                //                    return 0;
                 I = B->LineIndented(RowP) + C_CLASS_DELTA;
-//                if (FirstRowP == RowP) {
-//                    I += C_CLASS_DELTA;
-///                if (LookAt(B, Line, 0, "{", hsC_Normal, 0)) {
-///                    I += C_INDENT - C_BRACE_OFS;
-///                }
-//                }
-               PRINTF(("':' 2 indent : Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
+                //                if (FirstRowP == RowP) {
+                //                    I += C_CLASS_DELTA;
+                ///                if (LookAt(B, Line, 0, "{", hsC_Normal, 0)) {
+                ///                    I += C_INDENT - C_BRACE_OFS;
+                ///                }
+                //                }
+                PRINTF(("':' 2 indent : Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
                 break;
             case 'l':
                 ColP++;
@@ -923,15 +1057,15 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
                 break;
             case 'c':
                 ColP++;
-//                if (SkipWhite(B, Line, RowP, ColP, SKIP_FORWARD) != 1)
-//                    return 0;
+                //                if (SkipWhite(B, Line, RowP, ColP, SKIP_FORWARD) != 1)
+                //                    return 0;
                 I = B->LineIndented(RowP) + C_CASE_DELTA;
-//                if (FirstRowP == RowP) {
-//                    I += C_CASE_DELTA;
-///                if (LookAt(B, Line, 0, "{", hsC_Normal, 0)) {
-///                        I += C_INDENT - C_BRACE_OFS;
-///                }
-//                }
+                //                if (FirstRowP == RowP) {
+                //                    I += C_CASE_DELTA;
+                ///                if (LookAt(B, Line, 0, "{", hsC_Normal, 0)) {
+                ///                        I += C_INDENT - C_BRACE_OFS;
+                ///                }
+                //                }
 
                 PRINTF(("':' 3 indent : Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
                 break;
@@ -944,13 +1078,14 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
                 && FirstRowP != PrevRowP)
                 ContinuationIndent = C_CONTINUATION;
             if (LookAt(B, Line, 0, "{", hsC_Normal, 0)) {
-///                I -= C_INDENT - C_BRACE_OFS;
+                //I -= C_INDENT - C_BRACE_OFS;
                 ContinuationIndent = 0;
             }
             PRINTF(("':' -- indent : Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
             return I + ContinuationIndent;
-            
+
         case 'i':
+        case 's':
         case 'f':
         case 'e':
         case 'w':
@@ -960,8 +1095,9 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
             case 'f': ColP += 3; break; // for
             case 'e': ColP += 4; break; // else
             case 'w': ColP += 5; break; // while
+            case 's': ColP += 6; break;
             }
-            PRINTF(("'ifew' -- indent 1: Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
+            PRINTF(("'ifews' -- indent 1: Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
             if (SkipWhite(B, Line, RowP, ColP,
                           SKIP_FORWARD | (CharP != 'e' ? SKIP_MATCH : 0)) != 1)
                 return 0;
@@ -973,7 +1109,7 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
             if (SkipWhite(B, Line, RowP, ColP, SKIP_BACK) != 1)
                 return 0;
             ColP++;
-            PRINTF(("'ifew' -- indent 2: Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
+            PRINTF(("'ifews' -- indent 2: Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
 
             if (((PrevRowP != RowP) ||
                  ((PrevRowP == RowP) && (PrevColP != ColP)))
@@ -988,7 +1124,7 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
             }
 
             return I + ContinuationIndent;
-            
+
         default:
             return 0;
         }
@@ -996,20 +1132,20 @@ static int IndentNormal(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
     return 0;
 }
 
-static int IndentComment(EBuffer *B, int Row, int StateLen, hsState *StateMap) {
+static int IndentComment(EBuffer *B, int Row, int /*StateLen*/, hsState * /*StateMap*/) {
     int I = 0, R;
-    
+
     //puts("Comment");
 
     if (Row > 0) {
-	R = Row - 1;
-	while (R >= 0) {
-	    if (B->RLine(R)->Count == 0) R--;
-	    else {
-		I = B->LineIndented(R);
-		break;
-	    }
-	}
+        R = Row - 1;
+        while (R >= 0) {
+            if (B->RLine(R)->Count == 0) R--;
+            else {
+                I = B->LineIndented(R);
+                break;
+            }
+        }
         if (B->RLine(Row - 1)->StateE == hsC_Comment)
             if (LookAt(B, Row - 1, I, "/*", hsC_Comment, 0)) I += C_COMMENT_DELTA;
         if (B->RLine(Row - 1)->StateE == hsC_CPP_Comm)
@@ -1018,11 +1154,11 @@ static int IndentComment(EBuffer *B, int Row, int StateLen, hsState *StateMap) {
     return I;
 }
 
-static int IndentCPP(EBuffer *B, int Line, int StateLen, hsState *StateMap) {
+static int IndentCPP(EBuffer *B, int Line, int /*StateLen*/, hsState * /*StateMap*/) {
     if (LookAt(B, Line, 0, "#", hsC_CPP, 0))
-	return 0;
+        return 0;
     else
-	return C_INDENT;
+        return C_INDENT;
 }
 
 int Indent_C(EBuffer *B, int Line, int PosCursor) {
@@ -1037,14 +1173,14 @@ int Indent_C(EBuffer *B, int Line, int PosCursor) {
     } else {
         if (I != 0) B->IndentLine(Line, 0);
         if (B->GetMap(Line, &StateLen, &StateMap) == 0) return 0;
-        
+
         switch (B->RLine(Line - 1)->StateE) {
         case hsC_Comment:
         case hsC_CPP_Comm:
             I = IndentComment(B, Line, StateLen, StateMap);
             break;
         case hsC_CPP:
-        /*case hsC_CPP_Comm:*/
+            /*case hsC_CPP_Comm:*/
         case hsC_CPP_String1:
         case hsC_CPP_String2:
         case hsC_CPP_ABrace:
@@ -1086,23 +1222,23 @@ int Indent_C(EBuffer *B, int Line, int PosCursor) {
     }
     if (StateMap)
         free(StateMap);
-    if (I >= 0) 
-	B->IndentLine(Line, I);
-    else 
-	I = 0;
+    if (I >= 0)
+        B->IndentLine(Line, I);
+    else
+        I = 0;
     if (PosCursor == 1) {
-	int X = B->CP.Col;
+        int X = B->CP.Col;
 
         X = X - OI + I;
         if (X < I) X = I;
         if (X < 0) X = 0;
-	if (X > B->LineLen(Line)) {
+        if (X > B->LineLen(Line)) {
             X = B->LineLen(Line);
-	    if (X < I) X = I;
-	}
+            if (X < I) X = I;
+        }
         if (B->SetPosR(X, Line) == 0) return 0;
     } else if (PosCursor == 2) {
-	if (B->SetPosR(I, Line) == 0) return 0;
+        if (B->SetPosR(I, Line) == 0) return 0;
     }
     return 1;
 }
