@@ -62,8 +62,8 @@
 #define hsPerl_Comment      1
 #define hsPerl_Normal      30
 #define hsPerl_Keyword      4
-#define hsPerl_String1     20
-#define hsPerl_String2     21
+#define hsPerl_String1     10
+#define hsPerl_String2     11
 #define hsPerl_StringBk    22
 #define hsPerl_Variable    23
 #define hsPerl_Number      24
@@ -87,6 +87,8 @@
 #define hsPerl_Regexp2     42
 #define hsPerl_Regexp2M    43
 
+#define hsPerl_HereDoc     44 // hack (eod not detected properly)
+
 #define opQ  1
 #define opQQ 2
 #define opQW 3
@@ -95,19 +97,20 @@
 #define opS  6
 #define opTR 7
 
-int Hilit_PERL(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlState &State, hsState *StateMap, int *ECol) {
+int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, hlState &State, hsState *StateMap, int *ECol) {
     ChColor *Colors = BF->Mode->fColorize->Colors;
     int j;
     HILIT_VARS(Colors[CLR_Normal], Line);
     int firstnw = 0;
     int op;
+    int setHereDoc = 0;
 
     C = 0;
     NC = 0;
     for(i = 0; i < Line->Count;) {
         if (*p != ' ' && *p != 9) firstnw++;
         IF_TAB() else {
-	//	    printf("State = %d pos = %d", State, i); fflush(stdout);
+            //         printf("State = %d pos = %d", State, i); fflush(stdout);
             switch (State & X_MASK) {
             default:
             case hsPerl_Normal:
@@ -124,9 +127,9 @@ int Hilit_PERL(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hl
                     Color = Colors[CLR_Comment];
                 hilit5:
                     ColorNext();
-                hilit4:
+                //hilit4:
                     ColorNext();
-                hilit3:
+                //hilit3:
                     ColorNext();
                 hilit2:
                     ColorNext();
@@ -215,21 +218,21 @@ int Hilit_PERL(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hl
                         continue;
                     }
                 } else if (len >= 2 && ((*p == '-' && p[1] == '-') || (*p == '+' && p[1] == '+'))) {
-                    hsState s = State;
+                    hlState s = State;
                     State = hsPerl_Punct;
                     Color = Colors[CLR_Punctuation];
                     ColorNext();
                     ColorNext();
                     State = s;
                     continue;
-		} else if (len >= 2 && *p == '&' && (p[1] == '&' || isspace(p[1]))) {
-		    State = hsPerl_Punct;
-		    Color = Colors[CLR_Punctuation];
-		    ColorNext();
-		    ColorNext();
-		    State = hsPerl_Normal;
-		    continue;
-		} else if (*p == '&' && (len < 2 || p[1] != '&') && X_NOT(State)) {
+                } else if (len >= 2 && *p == '&' && (p[1] == '&' || isspace(p[1]))) {
+                    State = hsPerl_Punct;
+                    Color = Colors[CLR_Punctuation];
+                    ColorNext();
+                    ColorNext();
+                    State = hsPerl_Normal;
+                    continue;
+                } else if (*p == '&' && (len < 2 || p[1] != '&') && X_NOT(State)) {
                     State = hsPerl_Function;
                     Color = Colors[CLR_Function];
                     ColorNext();
@@ -237,41 +240,41 @@ int Hilit_PERL(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hl
                                          *p == '@' ||
                                          *p == '*' ||
                                          *p == '%' ||
-					 *p == '\\'))
-			ColorNext();
+                                         *p == '\\'))
+                        ColorNext();
                     while ((len > 0) && (isalnum(*p) || 
                                          *p == '_' ||
-					 *p == '\''))
+                                         *p == '\''))
                         ColorNext();
                     State = hsPerl_Normal | X_BIT;
                     continue;
                 } else if ((*p == '$') && (len > 1) &&
                            ((p[1] == '$') || p[1] == '"')) {
                     State = hsPerl_Variable;
-		    Color = Colors[CLR_Variable];
-		    ColorNext();
-		    ColorNext();
+                    Color = Colors[CLR_Variable];
+                    ColorNext();
+                    ColorNext();
                     State = hsPerl_Normal | X_BIT;
                     continue;
                 } else if (*p == '$' || *p == '@' || *p == '\\' || (len > 2 && (*p == '%' || *p == '*') && X_NOT(State))) {
                     State = hsPerl_Variable;
-		    Color = Colors[CLR_Variable];
-		    ColorNext();
+                    Color = Colors[CLR_Variable];
+                    ColorNext();
                     while ((len > 0) && ((*p == ' ') || (*p == '\t'))) {
                         IF_TAB() else
                             ColorNext();
                     }
-		    while ((len > 0) && (*p == '$' ||
+                    while ((len > 0) && (*p == '$' ||
                                          *p == '@' ||
                                          *p == '*' ||
-					 *p == '%' ||
-					 *p == '\\'))
-			ColorNext();
-		    if (len > 0 && *p != '{' && *p != ' ' && *p != '\t' && *p != '"' && *p != '\'')
+                                         *p == '%' ||
+                                         *p == '\\'))
+                        ColorNext();
+                    if (len > 0 && *p != '{' && *p != ' ' && *p != '\t' && *p != '"' && *p != '\'')
                         ColorNext();
                     while ((len > 0) && (isalnum(*p) || 
                                          *p == '_' || 
-					 *p == '\''))
+                                         *p == '\''))
                         ColorNext();
                     State = hsPerl_Normal | X_BIT;
                     continue;
@@ -281,8 +284,8 @@ int Hilit_PERL(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hl
                     ColorNext();
                     ColorNext();
                     while (len && (isxdigit(*p) || *p == '_')) ColorNext();
-//                    if (len && (toupper(*p) == 'U')) ColorNext();
-//                    if (len && (toupper(*p) == 'L')) ColorNext();
+                    //                    if (len && (toupper(*p) == 'U')) ColorNext();
+                    //                    if (len && (toupper(*p) == 'L')) ColorNext();
                     State = hsPerl_Normal | X_BIT;
                     continue;
                 } else if (isdigit(*p)) {
@@ -290,8 +293,8 @@ int Hilit_PERL(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hl
                     Color = Colors[CLR_Number];
                     ColorNext();
                     while (len && (isdigit(*p) || (*p == 'e' || *p == 'E' || *p == '_'))) ColorNext();
-//                    if (len && (toupper(*p) == 'U')) ColorNext();
-//                    if (len && (toupper(*p) == 'L')) ColorNext();
+                    //                    if (len && (toupper(*p) == 'U')) ColorNext();
+                    //                    if (len && (toupper(*p) == 'L')) ColorNext();
                     State = hsPerl_Normal | X_BIT;
                     continue;
                 } else if (*p == '\'') {
@@ -302,6 +305,15 @@ int Hilit_PERL(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hl
                     State = QSET(hsPerl_String2, '"');
                     Color = Colors[CLR_String];
                     goto hilit;
+                } else if (*p == '<' && len > 2 && p[1] == '<' &&
+                           (p[2] == '"' || p[2] == '\'' || p[2] == '_' || (p[2] >= 'A' && p[2] <= 'Z')))
+                {
+                    setHereDoc++;
+                    State = hsPerl_Punct;
+                    Color = Colors[CLR_Punctuation];
+                    ColorNext();
+                    State = hsPerl_Normal;
+                    continue;
                 } else if (*p == '`') {
                     State = QSET(hsPerl_StringBk, '`');
                     Color = Colors[CLR_String];
@@ -309,29 +321,29 @@ int Hilit_PERL(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hl
                 } else if (*p == '#') {
                     State = hsPerl_Comment | (State & X_BIT);
                     continue;
-		} else if (X_NOT(State) && *p == '/') {
-		    State = QSET(hsPerl_Regexp1, '/');
+                } else if (X_NOT(State) && *p == '/') {
+                    State = QSET(hsPerl_Regexp1, '/');
                     Color = Colors[CLR_RegexpDelim];
                     goto hilit;
-		} else if (X_NOT(State) && *p == '-' && (len >= 2 && isalpha(p[1]))) {
+                } else if (X_NOT(State) && *p == '-' && (len >= 2 && isalpha(p[1]))) {
                     Color = Colors[CLR_Punctuation];
-		    ColorNext();
-		    ColorNext();
-		    State = hsPerl_Normal;
-		    continue;
-		} else if (*p == ')' || *p == ']') {
-		    State = hsPerl_Punct;
-		    Color = Colors[CLR_Punctuation];
-		    ColorNext();
-                    State = hsPerl_Normal | X_BIT;
-		    continue;
-		} else if (ispunct(*p)) {
-		    State = hsPerl_Punct;
-		    Color = Colors[CLR_Punctuation];
-		    ColorNext();
+                    ColorNext();
+                    ColorNext();
                     State = hsPerl_Normal;
-		    continue;
-		}
+                    continue;
+                } else if (*p == ')' || *p == ']') {
+                    State = hsPerl_Punct;
+                    Color = Colors[CLR_Punctuation];
+                    ColorNext();
+                    State = hsPerl_Normal | X_BIT;
+                    continue;
+                } else if (ispunct(*p)) {
+                    State = hsPerl_Punct;
+                    Color = Colors[CLR_Punctuation];
+                    ColorNext();
+                    State = hsPerl_Normal;
+                    continue;
+                }
                 Color = Colors[CLR_Normal];
                 goto hilit;
             case hsPerl_Quote1Op:
@@ -477,6 +489,21 @@ int Hilit_PERL(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hl
             case hsPerl_Data:
                 Color = Colors[CLR_Comment];
                 goto hilit;
+            case hsPerl_HereDoc:
+                if (i == 0) {
+                    for (int xx = 0; xx < len; xx++)
+                        if (p[xx] != '_' && (p[xx] < 'A' || p[xx] > 'Z'))
+                            goto not_end;
+                    Color = Colors[CLR_Punctuation];
+                    setHereDoc = QCHAR(State);
+                    while (len > 0)
+                        ColorNext();
+                    State = hsPerl_Normal | (State & X_BIT);
+                    continue;
+                }
+            not_end:
+                Color = Colors[CLR_String];
+                goto hilit;
             case hsPerl_Docs:
                 Color = Colors[CLR_Comment];
                 if (i == 0 && *p == '=' && len > 3 &&
@@ -491,7 +518,7 @@ int Hilit_PERL(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hl
                     continue;
                 }
                 goto hilit;
-	    case hsPerl_Comment:
+            case hsPerl_Comment:
                 Color = Colors[CLR_Comment];
                 goto hilit;
             case hsPerl_String1:
@@ -529,6 +556,8 @@ int Hilit_PERL(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hl
     }
     if ((State & X_MASK) == hsPerl_Comment)
         State = hsPerl_Normal | (State & X_BIT);
+    if (setHereDoc)
+        State = QSET(hsPerl_HereDoc | (State & X_BIT), setHereDoc - 1);
     *ECol = C;
     return 0;
 }
