@@ -9,7 +9,7 @@
 
 #include "fte.h"
 
-ExInput::ExInput(char *APrompt, char *ALine, unsigned int ABufLen, Completer AComp, int Select, int AHistId): ExView() {
+ExInput::ExInput(const char *APrompt, char *ALine, unsigned int ABufLen, Completer AComp, int Select, int AHistId): ExView() {
     assert(ABufLen > 0);
     MaxLen = ABufLen - 1;
     Comp = AComp;
@@ -123,6 +123,7 @@ void ExInput::HandleEvent(TEvent &Event) {
             Event.What = evNone;
             break;
         case kbBackSp:
+        case kbBackSp | kfShift:
             if (SelStart < SelEnd) {
                 memmove(Line + SelStart, Line + SelEnd, strlen(Line + SelEnd) + 1);
                 Pos = SelStart;
@@ -160,6 +161,32 @@ void ExInput::HandleEvent(TEvent &Event) {
             TabCount = 0;
             Event.What = evNone;
             break;
+        case kbIns | kfShift:
+            {
+                int len;
+
+                if (SystemClipboard)
+                    GetPMClip();
+                
+                if (SS == 0) break;
+                if (SS->RCount == 0) break;
+
+                if (SelStart < SelEnd) {
+                    memmove(Line + SelStart, Line + SelEnd, strlen(Line + SelEnd) + 1);
+                    Pos = SelStart;
+                    SelStart = SelEnd = 0;
+                }
+
+                len = SS->LineChars(0);
+                if (strlen(Line) + len < MaxLen) {
+                    memmove(Line + Pos + len, Line + Pos, strlen(Line + Pos));
+                    memcpy(Line + Pos, SS->RLine(0)->Chars, len);
+                    TabCount = 0;
+                    Event.What = evNone;
+                    Pos += len;
+                }
+            }
+            break;
 #ifdef CONFIG_HISTORY
         case kbUp:
             SelStart = SelEnd = 0;
@@ -209,12 +236,9 @@ void ExInput::HandleEvent(TEvent &Event) {
             SelStart = SelEnd = 0;
             Event.What = evNone;
             break;
-        case ('Q' - '@') | kfCtrl:
-        case ('Q') | kfCtrl:
+        case 'Q' | kfCtrl:
             Event.What = evKeyDown;
             Event.Key.Code = Win->GetChar(0);
-            if (keyType(Event.Key.Code) == kfCtrl)
-                Event.Key.Code &= ~kfCtrl;
         default:
             {
                 char Ch;
@@ -266,7 +290,7 @@ void ExInput::RepaintStatus() {
     if (Pos > strlen(Line)) 
         Pos = strlen(Line);
     //if (Pos < 0) Pos = 0;
-    if (LPos + FLen <= Pos) LPos = Pos - FLen;
+    if (LPos + FLen <= Pos) LPos = Pos - FLen + 1;
     if (Pos < LPos) LPos = Pos;
     
     MoveChar(B, 0, W, ' ', hcEntry_Field, W);
