@@ -63,6 +63,36 @@ void ExISearch::HandleEvent(TEvent &Event) {
             }
             break;
         case kbUp:
+            Buffer->ScrollDown(1);
+            break;
+        case kbDown:
+            Buffer->ScrollUp(1);
+            break;
+        case kbLeft:
+            Buffer->ScrollRight(8);
+            break;
+        case kbRight:
+            Buffer->ScrollLeft(8);
+            break;
+        case kbPgDn:
+            Buffer->MovePageDown();
+            break;
+        case kbPgUp:
+            Buffer->MovePageUp();
+            break;
+        case kbPgUp | kfCtrl:
+            Buffer->MoveFileStart();
+            break;
+        case kbPgDn | kfCtrl:
+            Buffer->MoveFileEnd();
+            break;
+        case kbHome:
+            Buffer->MoveLineStart();
+            break;
+        case kbEnd:
+            Buffer->MoveLineEnd();
+            break;
+        case kbTab | kfShift:
             Direction = SEARCH_BACK;
             if (len == 0) {
                 strcpy(ISearchStr, PrevISearch);
@@ -75,7 +105,7 @@ void ExISearch::HandleEvent(TEvent &Event) {
                 SetState(INoPrev);
             }
             break;
-        case kbDown:
+        case kbTab:
             Direction = 0;
             if (len == 0) {
                 strcpy(ISearchStr, PrevISearch);
@@ -88,6 +118,9 @@ void ExISearch::HandleEvent(TEvent &Event) {
                 SetState(INoNext);
             }
             break;
+        case 'Q' | kfCtrl:
+            Event.What = evKeyDown;
+            Event.Key.Code = Win->GetChar(0);
         default:
             if (isAscii(Event.Key.Code) && (len < MAXISEARCH)) {
                 char Ch = (char) Event.Key.Code;
@@ -128,7 +161,7 @@ void ExISearch::UpdateStatus() {
 void ExISearch::RepaintStatus() {
     TDrawBuffer B;
     char s[MAXISEARCH + 1];
-    char *p;
+    const char *p;
     int W, H;
     
     ConQuerySize(&W, &H);
