@@ -10,12 +10,13 @@
 #include "fte.h"
 
 SearchReplaceOptions LSearch = { 0 };
+int suspendLoads = 0;
 
 EViewPort *EBuffer::CreateViewPort(EView *V) {
     V->Port = new EEditPort(this, V);
     AddView(V);
-    
-    if (Loaded == 0) {
+
+    if (Loaded == 0 && !suspendLoads) {
         Load();
 
 #ifdef CONFIG_OBJ_MESSAGES
@@ -23,11 +24,15 @@ EViewPort *EBuffer::CreateViewPort(EView *V) {
             CompilerMsgs->FindFileErrors(this);
 #endif
 
+        markIndex.retrieveForBuffer(this);
+
 #ifdef CONFIG_HISTORY
         int r, c;
-        
+
         if (RetrieveFPos(FileName, r, c) == 1)
             SetNearPosR(c, r);
+        //printf("setting to c:%d r:%d f:%s", c, r, FileName);
+        V->Port->GetPos();
         V->Port->ReCenter = 1;
 #endif
     }
@@ -60,16 +65,16 @@ void EEditPort::Resize(int Width, int Height) {
 
 int EEditPort::SetTop(int Col, int Line) {
     int A, B;
-    
+
     if (Line >= Buffer->VCount) Line = Buffer->VCount - 1;
     if (Line < 0) Line = 0;
-    
+
     A = Line;
     B = Line + Rows;
-    
+
     TP.Row = Line;
     TP.Col = Col;
-    
+
     if (A >= Buffer->VCount) A = Buffer->VCount - 1;
     if (B >= Buffer->VCount) {
         B = Buffer->VCount - 1;
@@ -117,11 +122,11 @@ void EEditPort::RedrawAll() {
     ///    Redraw(0, 0, Cols, Rows);
 }
 
-int EBuffer::GetContext() { 
+int EBuffer::GetContext() {
     return CONTEXT_FILE;
 }
 
-void EEditPort::HandleEvent(TEvent &Event) { 
+void EEditPort::HandleEvent(TEvent &Event) {
     EViewPort::HandleEvent(Event);
     switch (Event.What) {
     case evKeyDown:
@@ -132,7 +137,7 @@ void EEditPort::HandleEvent(TEvent &Event) {
                     return ;
                 Buffer->TypeChar(Ch);
                 Event.What = evNone;
-	    }
+            }
         }
         break;
     case evCommand:
@@ -156,7 +161,7 @@ void EEditPort::HandleEvent(TEvent &Event) {
         case cmVScrollMove:
             {
                 int ypos;
-                
+
 //                fprintf(stderr, "Pos = %d\n\x7", Event.Msg.Param1);
                 ypos = Buffer->CP.Row - TP.Row;
                 Buffer->SetNearPos(Buffer->CP.Col, Event.Msg.Param1 + ypos);
@@ -184,7 +189,7 @@ void EEditPort::HandleEvent(TEvent &Event) {
         case cmHScrollMove:
             {
                 int xpos;
-                
+
                 xpos = Buffer->CP.Col - TP.Col;
                 Buffer->SetNearPos(Event.Msg.Param1 + xpos, Buffer->CP.Row);
                 SetTop(Event.Msg.Param1, TP.Row);
@@ -204,19 +209,19 @@ void EEditPort::HandleEvent(TEvent &Event) {
 }
 void EEditPort::HandleMouse(TEvent &Event) {
     int x, y, xx, yy, W, H;
-    
+
     View->MView->ConQuerySize(&W, &H);
-    
+
     x = Event.Mouse.X;
     y = Event.Mouse.Y;
-    
+
     if (Event.What != evMouseDown || y < H - 1) {
         xx = x + TP.Col;
         yy = y + TP.Row;
         if (yy >= Buffer->VCount) yy = Buffer->VCount - 1;
         if (yy < 0) yy = 0;
         if (xx < 0) xx = 0;
-        
+
         switch (Event.What) {
         case evMouseDown:
             if (Event.Mouse.Y == H - 1)
@@ -225,9 +230,9 @@ void EEditPort::HandleMouse(TEvent &Event) {
                 View->MView->MouseCaptured = 1;
             else
                 break;
-            
+
             View->MView->MouseMoved = 0;
-            
+
             if (Event.Mouse.Buttons == 1) {
                 Buffer->SetNearPos(xx, yy);
                 switch (Event.Mouse.Count % 5) {
@@ -299,8 +304,8 @@ void EEditPort::HandleMouse(TEvent &Event) {
             if (Event.Mouse.Buttons == 2) {
                 if (!View->MView->MouseMoved) {
                     EEventMap *Map = View->MView->Win->GetEventMap();
-                    char *MName = 0;
-                    
+                    const char *MName = 0;
+
                     if (Map)
                         MName = Map->GetMenu(EM_LocalMenu);
                     if (MName == 0)
@@ -311,7 +316,7 @@ void EEditPort::HandleMouse(TEvent &Event) {
             if (Event.Mouse.Buttons == 4) {
                 if (SystemClipboard) {
                     Buffer->NextCommand();
-                    if (Event.Mouse.KeyMask == 0) 
+                    if (Event.Mouse.KeyMask == 0)
                         Buffer->BlockPasteStream();
                     else if (Event.Mouse.KeyMask == kfCtrl)
                         Buffer->BlockPasteColumn();
@@ -322,7 +327,7 @@ void EEditPort::HandleMouse(TEvent &Event) {
             Event.What = evNone;
             break;
         }
-    } 
+    }
 }
 
 void EEditPort::UpdateView() {
@@ -364,6 +369,14 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
     case ExMoveWordEndRight:      return MoveWordEndRight();
     case ExMoveWordEndPrev:       return MoveWordEndPrev();
     case ExMoveWordEndNext:       return MoveWordEndNext();
+    case ExMoveWordOrCapLeft:     return MoveWordOrCapLeft();
+    case ExMoveWordOrCapRight:    return MoveWordOrCapRight();
+    case ExMoveWordOrCapPrev:     return MoveWordOrCapPrev();
+    case ExMoveWordOrCapNext:     return MoveWordOrCapNext();
+    case ExMoveWordOrCapEndLeft:  return MoveWordOrCapEndLeft();
+    case ExMoveWordOrCapEndRight: return MoveWordOrCapEndRight();
+    case ExMoveWordOrCapEndPrev:  return MoveWordOrCapEndPrev();
+    case ExMoveWordOrCapEndNext:  return MoveWordOrCapEndNext();
     case ExMoveLineStart:         return MoveLineStart();
     case ExMoveLineEnd:           return MoveLineEnd();
     case ExMovePageStart:         return MovePageStart();
@@ -387,6 +400,9 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
     case ExMoveLineTop:           return MoveLineTop();
     case ExMoveLineCenter:        return MoveLineCenter();
     case ExMoveLineBottom:        return MoveLineBottom();
+    case ExMoveBeginOrNonWhite:   return MoveBeginOrNonWhite();
+    case ExMoveBeginLinePageFile: return MoveBeginLinePageFile();
+    case ExMoveEndLinePageFile:   return MoveEndLinePageFile();
     case ExScrollLeft:            return ScrollLeft(State);
     case ExScrollRight:           return ScrollRight(State);
     case ExScrollDown:            return ScrollDown(State);
@@ -396,6 +412,8 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
     case ExKillCharPrev:          return KillCharPrev();
     case ExKillWord:              return KillWord();
     case ExKillWordPrev:          return KillWordPrev();
+    case ExKillWordOrCap:         return KillWordOrCap();
+    case ExKillWordOrCapPrev:     return KillWordOrCapPrev();
     case ExKillToLineStart:       return KillToLineStart();
     case ExKillToLineEnd:         return KillToLineEnd();
     case ExKillBlock:             return KillBlock();
@@ -414,10 +432,18 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
     case ExLineNew:               return LineNew();
     case ExLineIndent:            return LineIndent();
     case ExLineTrim:              return LineTrim();
-    case ExInsertSpacesToTab:     return InsertSpacesToTab(0);
+    case ExLineCenter:            return LineCenter();
+    case ExInsertSpacesToTab:
+        {
+            int no;
+
+            if(State.GetIntParam(View, &no) == 0)
+                no = 0;
+            return InsertSpacesToTab(no);
+        }
     case ExInsertTab:             return InsertTab();
     case ExInsertSpace:           return InsertSpace();
-    case ExWrapPara:              
+    case ExWrapPara:
 #ifdef CONFIG_WORDWRAP
         return WrapPara();
 #else
@@ -482,12 +508,15 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
     case ExFoldOpenAll:           return FoldOpenAll();
     case ExFoldCloseAll:          return FoldCloseAll();
     case ExFoldToggleOpenClose:   return FoldToggleOpenClose();
+    case ExFoldCreateAtRoutines:  return FoldCreateAtRoutines();
     case ExMoveFoldTop:           return MoveFoldTop();
     case ExMoveFoldPrev:          return MoveFoldPrev();
     case ExMoveFoldNext:          return MoveFoldNext();
     case ExFileSave:              return Save();
     case ExFilePrint:             return FilePrint();
     case ExBlockPrint:            return BlockPrint();
+    case ExBlockTrim:             return BlockTrim();
+    case ExFileTrim:              return FileTrim();
     case ExHilitWord:
 #ifdef CONFIG_WORD_HILIT
         return HilitWord();
@@ -515,7 +544,8 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
     case ExToggleShowMarkers:     return ToggleShowMarkers();
     case ExSetLeftMargin:         return SetLeftMargin();
     case ExSetRightMargin:        return SetRightMargin();
-        
+    case ExSetIndentWithTabs:     return SetIndentWithTabs(State);
+
         // stuff with UI
     case ExMoveToLine:          return MoveToLine(State);
     case ExMoveToColumn:        return MoveToColumn(State);
@@ -529,6 +559,8 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
     case ExRemoveBookmark:      return ErFAIL;
     case ExGotoBookmark:        return ErFAIL;
 #endif
+    case ExPlaceGlobalBookmark: return PlaceGlobalBookmark(State);
+    case ExPushGlobalBookmark:  return PushGlobalBookmark();
     case ExInsertString:        return InsertString(State);
     case ExSelfInsert:          return SelfInsert(State);
     case ExFileReload:          return FileReload(State);
@@ -562,7 +594,7 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
     case ExChangeTabSize:       return ChangeTabSize(State);
     case ExChangeLeftMargin:    return ChangeLeftMargin(State);
     case ExChangeRightMargin:   return ChangeRightMargin(State);
-    case ExASCIITable:          
+    case ExASCIITable:
 #ifdef CONFIG_I_ASCII
         return ASCIITable(State);
 #else
@@ -574,15 +606,18 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
 
 #ifdef CONFIG_TAGS
     case ExTagFind:             return FindTag(State);
-    case ExTagFindWord:         return FindTagWord();
+    case ExTagFindWord:         return FindTagWord(State);
 #endif
 
+    case ExSetCIndentStyle:     return SetCIndentStyle(State);
+
     case ExBlockMarkFunction:   return BlockMarkFunction();
     case ExIndentFunction:      return IndentFunction();
     case ExMoveFunctionPrev:    return MoveFunctionPrev();
     case ExMoveFunctionNext:    return MoveFunctionNext();
-    case ExInsertDate:      	return InsertDate(State);
+    case ExInsertDate:          return InsertDate(State);
     case ExInsertUid:           return InsertUid();
+    case ExShowHelpWord:        return ShowHelpWord(State);
     }
     return EModel::ExecCommand(Command, State);
 }
@@ -593,8 +628,8 @@ void EBuffer::HandleEvent(TEvent &Event) {
 
 int EBuffer::MoveToLine(ExState &State) {
     int No = 0;
-    
-    if (State.GetIntParam(&No) == 0) {
+
+    if (State.GetIntParam(View, &No) == 0) {
         char Num[10];
 
         sprintf(Num, "%d", VToR(CP.Row) + 1);
@@ -607,10 +642,10 @@ int EBuffer::MoveToLine(ExState &State) {
 
 int EBuffer::MoveToColumn(ExState &State) {
     int No = 0;
-    
-    if (State.GetIntParam(&No) == 0) {
+
+    if (State.GetIntParam(View, &No) == 0) {
         char Num[10];
-        
+
         sprintf(Num, "%d", CP.Col + 1);
         if (View->MView->Win->GetStr("Goto Column", 8, Num, HIST_POSITION) == 0) return 0;
         No = atol(Num);
@@ -620,8 +655,8 @@ int EBuffer::MoveToColumn(ExState &State) {
 
 int EBuffer::FoldCreateByRegexp(ExState &State) {
     char strbuf[1024] = "";
-    
-    if (State.GetStrParam(strbuf, sizeof(strbuf)) == 0) {
+
+    if (State.GetStrParam(View, strbuf, sizeof(strbuf)) == 0) {
         if (View->MView->Win->GetStr("Create Fold Regexp", sizeof(strbuf), strbuf, HIST_REGEXP) == 0) return 0;
     }
     return FoldCreateByRegexp(strbuf);
@@ -631,39 +666,63 @@ int EBuffer::FoldCreateByRegexp(ExState &State) {
 int EBuffer::PlaceBookmark(ExState &State) {
     char name[256] = "";
     EPoint P = CP;
-    
+
     P.Row = VToR(P.Row);
-    
-    if (State.GetStrParam(name, sizeof(name)) == 0)
+
+    if (State.GetStrParam(View, name, sizeof(name)) == 0)
         if (View->MView->Win->GetStr("Place Bookmark", sizeof(name), name, HIST_BOOKMARK) == 0) return 0;
     return PlaceBookmark(name, P);
 }
 
 int EBuffer::RemoveBookmark(ExState &State) {
     char name[256] = "";
-    
-    if (State.GetStrParam(name, sizeof(name)) == 0)
+
+    if (State.GetStrParam(View, name, sizeof(name)) == 0)
         if (View->MView->Win->GetStr("Remove Bookmark", sizeof(name), name, HIST_BOOKMARK) == 0) return 0;
     return RemoveBookmark(name);
 }
 
 int EBuffer::GotoBookmark(ExState &State) {
     char name[256] = "";
-    
-    if (State.GetStrParam(name, sizeof(name)) == 0)
+
+    if (State.GetStrParam(View, name, sizeof(name)) == 0)
         if (View->MView->Win->GetStr("Goto Bookmark", sizeof(name), name, HIST_BOOKMARK) == 0) return 0;
     return GotoBookmark(name);
 }
 #endif
 
+int EBuffer::PlaceGlobalBookmark(ExState &State) {
+    char name[256] = "";
+    EPoint P = CP;
+
+    P.Row = VToR(P.Row);
+
+    if (State.GetStrParam(View, name, sizeof(name)) == 0)
+        if (View->MView->Win->GetStr("Place Global Bookmark", sizeof(name), name, HIST_BOOKMARK) == 0) return 0;
+    if (markIndex.insert(name, this, P) == 0) {
+        Msg(S_ERROR, "Error placing global bookmark %s.", name);
+    }
+    return 1;
+}
+
+int EBuffer::PushGlobalBookmark() {
+    EPoint P = CP;
+
+    P.Row = VToR(P.Row);
+    EMark *m = markIndex.pushMark(this, P);
+    if (m)
+         Msg(S_INFO, "Placed bookmark %s", m->getName());
+    return m ? 1 : 0;
+}
+
 int EBuffer::InsertChar(ExState &State) {
     char Ch;
     int No;
 
-    if (State.GetIntParam(&No) == 0) {
+    if (State.GetIntParam(View, &No) == 0) {
         TEvent E;
         E.What = evKeyDown;
-        E.Key.Code = View->MView->Win->GetChar(0);
+        E.Key.Code = View->MView->Win->GetChar("Quote Char:");
         if (!GetCharFromEvent(E, &Ch)) return 0;
         No = Ch;
     }
@@ -676,7 +735,7 @@ int EBuffer::TypeChar(ExState &State) {
     char Ch;
     int No;
 
-    if (State.GetIntParam(&No) == 0) {
+    if (State.GetIntParam(View, &No) == 0) {
         TEvent E;
         E.What = evKeyDown;
         E.Key.Code = View->MView->Win->GetChar(0);
@@ -690,8 +749,8 @@ int EBuffer::TypeChar(ExState &State) {
 
 int EBuffer::InsertString(ExState &State) {
     char strbuf[1024] = "";
-    
-    if (State.GetStrParam(strbuf, sizeof(strbuf)) == 0) {
+
+    if (State.GetStrParam(View, strbuf, sizeof(strbuf)) == 0) {
         if (View->MView->Win->GetStr("Insert String", sizeof(strbuf), strbuf, HIST_DEFAULT) == 0)
             return 0;
     }
@@ -700,13 +759,13 @@ int EBuffer::InsertString(ExState &State) {
 
 extern int LastEventChar;
 
-int EBuffer::SelfInsert(ExState &State) {
+int EBuffer::SelfInsert(ExState &/*State*/) {
     if (LastEventChar != -1)
         return TypeChar(char(LastEventChar));
     return 0;
 }
 
-int EBuffer::FileReload(ExState &State) {
+int EBuffer::FileReload(ExState &/*State*/) {
     if (Modified) {
         switch (View->MView->Win->Choice(GPC_ERROR, "File Modified",
                        2,
@@ -728,13 +787,13 @@ int EBuffer::FileReload(ExState &State) {
 
 int EBuffer::FileSaveAs(char *FName) {
     char Name[MAXPATH];
-    
+
     if (ExpandPath(FName, Name) == -1) {
         View->MView->Win->Choice(GPC_ERROR, "Error", 1, "O&K", "Invalid path: %s.", FName);
         return 0;
     }
     if (FindFile(Name) == 0) {
-	if (FileExists(Name)) {
+        if (FileExists(Name)) {
             switch (View->MView->Win->Choice(GPC_ERROR, "File Exists",
                            2,
                            "&Overwrite",
@@ -747,9 +806,9 @@ int EBuffer::FileSaveAs(char *FName) {
             case -1:
             default:
                 return 0;
-                
+
             }
-	}
+        }
         free(FileName);
         FileName = strdup(Name);
         UpdateTitle();
@@ -762,9 +821,9 @@ int EBuffer::FileSaveAs(char *FName) {
 
 int EBuffer::FileSaveAs(ExState &State) {
     char FName[MAXPATH];
-    
+
     strcpy(FName, FileName);
-    if (State.GetStrParam(FName, sizeof(FName)) == 0)
+    if (State.GetStrParam(View, FName, sizeof(FName)) == 0)
         if (View->MView->Win->GetFile("Save As", sizeof(FName), FName, HIST_PATH, GF_SAVEAS) == 0)
             return 0;
     return FileSaveAs(FName);
@@ -772,13 +831,13 @@ int EBuffer::FileSaveAs(ExState &State) {
 
 int EBuffer::FileWriteTo(char *FName) {
     char Name[MAXPATH];
-    
+
     if (ExpandPath(FName, Name) == -1) {
         View->MView->Win->Choice(GPC_ERROR, "Error", 1, "O&K", "Invalid path: %s.", FName);
         return 0;
     }
     if (FindFile(Name) == 0) {
-	if (FileExists(Name)) {
+        if (FileExists(Name)) {
             switch (View->MView->Win->Choice(GPC_ERROR, "File Exists",
                            2,
                            "&Overwrite",
@@ -791,9 +850,8 @@ int EBuffer::FileWriteTo(char *FName) {
             case -1:
             default:
                 return 0;
-                
             }
-	}
+        }
         return SaveTo(Name);
     } else {
         View->MView->Win->Choice(GPC_ERROR, "Error", 1, "O&K", "Already editing '%s.'", Name);
@@ -803,9 +861,9 @@ int EBuffer::FileWriteTo(char *FName) {
 
 int EBuffer::FileWriteTo(ExState &State) {
     char FName[MAXPATH];
-    
+
     strcpy(FName, FileName);
-    if (State.GetStrParam(FName, sizeof(FName)) == 0)
+    if (State.GetStrParam(View, FName, sizeof(FName)) == 0)
         if (View->MView->Win->GetFile("Write To", sizeof(FName), FName, HIST_PATH, GF_SAVEAS) == 0) return 0;
     return FileWriteTo(FName);
 }
@@ -816,7 +874,7 @@ int EBuffer::BlockReadX(ExState &State, int blockMode) {
 
     if (JustDirectory(FileName, FName) == -1) return 0;
     SlashDir(FName);
-    if (State.GetStrParam(FName, sizeof(FName)) == 0)
+    if (State.GetStrParam(View, FName, sizeof(FName)) == 0)
         if (View->MView->Win->GetFile("Read block", sizeof(FName), FName, HIST_PATH, GF_OPEN) == 0) return 0;
 
     if (ExpandPath(FName, Name) == -1) {
@@ -849,10 +907,10 @@ int EBuffer::BlockWrite(ExState &State) {
 
     if (JustDirectory(FileName, FName) == -1) return 0;
     SlashDir(FName);
-    if (State.GetStrParam(FName, sizeof(FName)) == 0)
+    if (State.GetStrParam(View, FName, sizeof(FName)) == 0)
         if (View->MView->Win->GetFile("Write block", sizeof(FName), FName, HIST_PATH, GF_SAVEAS) == 0)
             return 0;
-    
+
     if (ExpandPath(FName, Name) == -1) {
         View->MView->Win->Choice(GPC_ERROR, "Error", 1, "O&K", "Invalid path: %s.", FName);
         return 0;
@@ -875,7 +933,7 @@ int EBuffer::BlockWrite(ExState &State) {
             case -1:
             default:
                 return 0;
-                
+
             }
         }
     } else {
@@ -888,9 +946,9 @@ int EBuffer::BlockWrite(ExState &State) {
 int EBuffer::Find(ExState &State) {
     char find[MAXSEARCH+1] = "";
     char options[32] = "";
-    
-    if (State.GetStrParam(find, sizeof(find)) != 0) {
-        if (State.GetStrParam(options, sizeof(options)) == 0)
+
+    if (State.GetStrParam(View, find, sizeof(find)) != 0) {
+        if (State.GetStrParam(View, options, sizeof(options)) == 0)
             strcpy(options, BFS(this, BFS_DefFindOpt));
 
         LSearch.ok = 0;
@@ -908,15 +966,15 @@ int EBuffer::Find(ExState &State) {
             strcpy(options, BFS(this, BFS_DefFindOpt));
         if (ParseSearchOptions(0, options, LSearch.Options) == 0)
             LSearch.Options = 0;
-        
+
         if (DLGGetFind(View->MView->Win, LSearch) == 0)
             return 0;
     } else {
         if (BFS(this, BFS_DefFindOpt))
             strcpy(options, BFS(this, BFS_DefFindOpt));
         if (View->MView->Win->GetStr("Find", sizeof(find), find, HIST_SEARCH) == 0) return 0;
-        if (View->MView->Win->GetStr("Options (abcdgijrx)", sizeof(options), options, HIST_SEARCHOPT) == 0) return 0;
-        
+        if (View->MView->Win->GetStr("Options (All/Block/Cur/Delln/Glob/Igncase/Joinln/Rev/Word/regX)", sizeof(options), options, HIST_SEARCHOPT) == 0) return 0;
+
         LSearch.ok = 0;
         strcpy(LSearch.strSearch, find);
         LSearch.strReplace[0] = 0;
@@ -934,11 +992,11 @@ int EBuffer::FindReplace(ExState &State) {
     char find[MAXSEARCH+1] = "";
     char replace[MAXSEARCH+1] = "";
     char options[32] = "";
-    
-    if (State.GetStrParam(find, sizeof(find)) != 0) {
-        if (State.GetStrParam(replace, sizeof(replace)) == 0)
+
+    if (State.GetStrParam(View, find, sizeof(find)) != 0) {
+        if (State.GetStrParam(View, replace, sizeof(replace)) == 0)
             return 0;
-        if (State.GetStrParam(options, sizeof(options)) == 0)
+        if (State.GetStrParam(View, options, sizeof(options)) == 0)
             return 0;
 
         LSearch.ok = 0;
@@ -962,13 +1020,13 @@ int EBuffer::FindReplace(ExState &State) {
     } else {
         if (BFS(this, BFS_DefFindReplaceOpt))
             strcpy(options, BFS(this, BFS_DefFindReplaceOpt));
-        if (State.GetStrParam(find, sizeof(find)) == 0)
+        if (State.GetStrParam(View, find, sizeof(find)) == 0)
             if (View->MView->Win->GetStr("Find", sizeof(find), find, HIST_SEARCH) == 0) return 0;
-        if (State.GetStrParam(replace, sizeof(replace)) == 0)
+        if (State.GetStrParam(View, replace, sizeof(replace)) == 0)
             if (View->MView->Win->GetStr("Replace", sizeof(replace), replace, HIST_SEARCH) == 0) return 0;
-        if (State.GetStrParam(options, sizeof(options)) == 0)
-            if (View->MView->Win->GetStr("Options (abcdgijrnx)", sizeof(options), options, HIST_SEARCHOPT) == 0) return 0;
-        
+        if (State.GetStrParam(View, options, sizeof(options)) == 0)
+            if (View->MView->Win->GetStr("Options (All/Block/Cur/Delln/Glob/Igncase/Joinln/Rev/Noask/Word/regX)", sizeof(options), options, HIST_SEARCHOPT) == 0) return 0;
+
         LSearch.ok = 0;
         strcpy(LSearch.strSearch, find);
         strcpy(LSearch.strReplace, replace);
@@ -993,11 +1051,15 @@ int EBuffer::FindRepeat(ExState &State) {
 
 int EBuffer::FindRepeatReverse(ExState &State) {
     int rc;
-    
-    if (LSearch.ok == 0) return Find(State);
+
+    if (LSearch.ok == 0) { // ???
+        LSearch.Options ^= SEARCH_BACK;
+        rc = Find(State);
+        LSearch.Options ^= SEARCH_BACK;
+        return rc;
+    }
     LSearch.Options |= SEARCH_NEXT;
     LSearch.Options &= ~SEARCH_GLOBAL;
-    LSearch.Options ^= SEARCH_BACK;
     rc = Find(LSearch);
     LSearch.Options ^= SEARCH_BACK;
     return rc;
@@ -1016,7 +1078,7 @@ int EBuffer::ChangeMode(ExState &State) {
     char Mode[32] = "";
     int rc;
 
-    if (State.GetStrParam(Mode, sizeof(Mode)) == 0)
+    if (State.GetStrParam(View, Mode, sizeof(Mode)) == 0)
         if (View->MView->Win->GetStr("Mode", sizeof(Mode), Mode, HIST_SETUP) == 0) return 0;
 
     rc = ChangeMode(Mode);
@@ -1028,7 +1090,7 @@ int EBuffer::ChangeKeys(ExState &State) {
     int rc;
     char Mode[32] = "";
 
-    if (State.GetStrParam(Mode, sizeof(Mode)) == 0)
+    if (State.GetStrParam(View, Mode, sizeof(Mode)) == 0)
         if (View->MView->Win->GetStr("Mode", sizeof(Mode), Mode, HIST_SETUP) == 0) return 0;
 
     rc = ChangeKeys(Mode);
@@ -1040,7 +1102,7 @@ int EBuffer::ChangeFlags(ExState &State) {
     int rc;
     char Mode[32] = "";
 
-    if (State.GetStrParam(Mode, sizeof(Mode)) == 0)
+    if (State.GetStrParam(View, Mode, sizeof(Mode)) == 0)
         if (View->MView->Win->GetStr("Mode", sizeof(Mode), Mode, HIST_SETUP) == 0) return 0;
 
     rc = ChangeFlags(Mode);
@@ -1050,10 +1112,10 @@ int EBuffer::ChangeFlags(ExState &State) {
 
 int EBuffer::ChangeTabSize(ExState &State) {
     int No;
-    
-    if (State.GetIntParam(&No) == 0) {
+
+    if (State.GetIntParam(View, &No) == 0) {
         char Num[10];
-        
+
         sprintf(Num, "%d", BFI(this, BFI_TabSize));
         if (View->MView->Win->GetStr("TabSize", sizeof(Num), Num, HIST_SETUP) == 0) return 0;
         No = atol(Num);
@@ -1065,33 +1127,41 @@ int EBuffer::ChangeTabSize(ExState &State) {
     return 1;
 }
 
+int EBuffer::SetIndentWithTabs(ExState &State) {
+    int No;
+
+    if (State.GetIntParam(View, &No) == 0) return 0;
+    Flags.num[BFI_IndentWithTabs] = No ? 1 : 0;
+    return 1;
+}
+
 int EBuffer::ChangeRightMargin(ExState &State) {
     char Num[10];
     int No;
-    
-    if (State.GetIntParam(&No) == 0) {
+
+    if (State.GetIntParam(View, &No) == 0) {
         sprintf(Num, "%d", BFI(this, BFI_RightMargin) + 1);
         if (View->MView->Win->GetStr("RightMargin", sizeof(Num), Num, HIST_SETUP) == 0) return 0;
         No = atol(Num) - 1;
     }
     if (No <= 1) return 0;
     BFI(this, BFI_RightMargin) = No;
-    Msg(INFO, "RightMargin set to %d.", No + 1);
+    Msg(S_INFO, "RightMargin set to %d.", No + 1);
     return 1;
 }
 
 int EBuffer::ChangeLeftMargin(ExState &State) {
     char Num[10];
     int No;
-    
-    if (State.GetIntParam(&No) == 0) {
+
+    if (State.GetIntParam(View, &No) == 0) {
         sprintf(Num, "%d", BFI(this, BFI_LeftMargin) + 1);
         if (View->MView->Win->GetStr("LeftMargin", sizeof(Num), Num, HIST_SETUP) == 0) return 0;
         No = atol(Num) - 1;
     }
     if (No < 0) return 0;
     BFI(this, BFI_LeftMargin) = No;
-    Msg(INFO, "LeftMargin set to %d.", No + 1);
+    Msg(S_INFO, "LeftMargin set to %d.", No + 1);
     return 1;
 }
 
@@ -1102,41 +1172,74 @@ int EBuffer::CanQuit() {
     else
         return 1;
 }
-    
-int EBuffer::ConfQuit(GxView *V) {
+
+int EBuffer::ConfQuit(GxView *V, int multiFile) {
     if (Modified) {
-        switch (V->Choice(GPC_ERROR,
-                          "File Modified",
-                          4,
-                          "&Save",
-                          "&As",
-                          "&Discard",
-                          "&Cancel",
-                          "%s", FileName))
-        {
-        case 0: /* Save */
-            if (Save() == 0) return 0;
-            break;
-        case 1: /* As */
+        if (multiFile) {
+            switch (V->Choice(GPC_ERROR,
+                              "File Modified",
+                              5,
+                              "&Save",
+                              "&As",
+                              "A&ll",
+                              "&Discard",
+                              "&Cancel",
+                              "%s", FileName))
             {
-                char FName[MAXPATH];
-                strcpy(FName, FileName);
-                if (V->GetFile("Save As", sizeof(FName), FName, HIST_PATH, GF_SAVEAS) == 0) return 0;
-                if (FileSaveAs(FName) == 0) return 0;
+            case 0: /* Save */
+                if (Save() == 0) return 0;
+                break;
+            case 1: /* As */
+                {
+                    char FName[MAXPATH];
+                    strcpy(FName, FileName);
+                    if (V->GetFile("Save As", sizeof(FName), FName, HIST_PATH, GF_SAVEAS) == 0) return 0;
+                    if (FileSaveAs(FName) == 0) return 0;
+                }
+                break;
+            case 2: /* Save all */
+                return -2;
+            case 3: /* Discard */
+                break;
+            case 4: /* Cancel */
+            case -1:
+            default:
+                return 0;
+            }
+        }else {
+            switch (V->Choice(GPC_ERROR,
+                              "File Modified",
+                              4,
+                              "&Save",
+                              "&As",
+                              "&Discard",
+                              "&Cancel",
+                              "%s", FileName))
+            {
+            case 0: /* Save */
+                if (Save() == 0) return 0;
+                break;
+            case 1: /* As */
+                {
+                    char FName[MAXPATH];
+                    strcpy(FName, FileName);
+                    if (V->GetFile("Save As", sizeof(FName), FName, HIST_PATH, GF_SAVEAS) == 0) return 0;
+                    if (FileSaveAs(FName) == 0) return 0;
+                }
+                break;
+            case 2: /* Discard */
+                break;
+            case 3: /* Cancel */
+            case -1:
+            default:
+                return 0;
             }
-            break;
-        case 2: /* Discard */
-            break;
-        case 3: /* Cancel */
-        case -1:
-        default:
-            return 0;
         }
     }
     return 1;
 }
 
-void EBuffer::GetName(char *AName, int MaxLen) { 
+void EBuffer::GetName(char *AName, int MaxLen) {
     strncpy(AName, FileName, MaxLen);
     AName[MaxLen - 1] = 0;
 }
@@ -1146,20 +1249,20 @@ void EBuffer::GetPath(char *APath, int MaxLen) {
 }
 
 void EBuffer::GetInfo(char *AInfo, int MaxLen) {
-    sprintf(AInfo, 
+    sprintf(AInfo,
             "%2d %04d:%03d%c%-150s ",
-            ModelNo, 
-            1 + CP.Row, 1 + CP.Col, 
-            Modified ? '*': ' ', 
+            ModelNo,
+            1 + CP.Row, 1 + CP.Col,
+            Modified ? '*': ' ',
             FileName);
 }
 
 void EBuffer::GetTitle(char *ATitle, int MaxLen, char *ASTitle, int SMaxLen) {
     char *p;
-    
+
     strncpy(ATitle, FileName, MaxLen - 1);
     ATitle[MaxLen - 1] = 0;
-    p = strrchr(FileName, SLASH);
+    p = SepRChr(FileName);
     if (p) {
         strncpy(ASTitle, p + 1, SMaxLen - 1);
         ASTitle[SMaxLen - 1] = 0;
@@ -1170,45 +1273,45 @@ void EBuffer::GetTitle(char *ATitle, int MaxLen, char *ASTitle, int SMaxLen) {
 }
 
 #ifdef CONFIG_I_ASCII
-int EBuffer::ASCIITable(ExState &State) {
+int EBuffer::ASCIITable(ExState &/*State*/) {
     int rc;
-    
+
     rc = View->MView->Win->PickASCII();
-    if (rc != -1) 
+    if (rc != -1)
         return InsertChar(char(rc));
-    
+
     return 0;
 }
 #endif
 
 int EBuffer::ScrollLeft(ExState &State) {
     int Cols;
-    
-    if (State.GetIntParam(&Cols) == 0)
+
+    if (State.GetIntParam(View, &Cols) == 0)
         Cols = 8;
     return ScrollLeft(Cols);
 }
 
 int EBuffer::ScrollRight(ExState &State) {
     int Cols;
-    
-    if (State.GetIntParam(&Cols) == 0)
+
+    if (State.GetIntParam(View, &Cols) == 0)
         Cols = 8;
     return ScrollRight(Cols);
 }
 
 int EBuffer::ScrollDown(ExState &State) {
     int Rows;
-    
-    if (State.GetIntParam(&Rows) == 0)
+
+    if (State.GetIntParam(View, &Rows) == 0)
         Rows = 1;
     return ScrollDown(Rows);
 }
 
 int EBuffer::ScrollUp(ExState &State) {
     int Rows;
-    
-    if (State.GetIntParam(&Rows) == 0)
+
+    if (State.GetIntParam(View, &Rows) == 0)
         Rows = 1;
     return ScrollUp(Rows);
 }
@@ -1216,29 +1319,52 @@ int EBuffer::ScrollUp(ExState &State) {
 #ifdef CONFIG_TAGS
 int EBuffer::FindTag(ExState &State) {
     char Tag[MAXSEARCH] = "";
-    
-    if (State.GetStrParam(Tag, sizeof(Tag)) == 0)
+
+    if (State.GetStrParam(View, Tag, sizeof(Tag)) == 0)
         if (View->MView->Win->GetStr("Find tag", sizeof(Tag), Tag, HIST_SEARCH) == 0) return 0;
-    
-    if (TagFind(this, View, Tag) == 0) {
-        Msg(INFO, "Tag '%s' not found.", Tag);
-        return 0;
+
+    int j = 2;
+    while (j--) {
+        int i;
+
+        i = TagFind(this, View, Tag);
+        if (i > 0)
+            return 1;
+        else if (j && (i < 0)) {
+            /* Try autoload tags */
+            if (View->ExecCommand(ExTagLoad, State) == 0)
+                break;
+        } else {
+            Msg(S_INFO, "Tag '%s' not found.", Tag);
+            break;
+        }
     }
-    return 1;
+    return 0;
 
 }
 #endif
 
 // these two will probably be replaced in the future
-int EBuffer::InsertDate(ExState& state) {
-    char buf[50], *p;
+int EBuffer::InsertDate(ExState &State) {
+    char strArg[128] = "";
+    char buf[128], *p;
+
     time_t t;
-    
+
     time(&t);
-    p = ctime(&t);
-    //** 012345678901234567890123
-    //** Wed Jan 02 02:23:54 1991
-    sprintf(buf, "%.10s %.4s", p, p + 20);
+
+    if (State.GetStrParam(View, strArg, sizeof(strArg))) {
+        struct tm *tt = localtime(&t);
+        strftime(buf, sizeof(buf), strArg, tt);
+        buf[sizeof(buf) - 1] = 0;
+    } else {
+        //** 012345678901234567890123
+        //** Wed Jan 02 02:23:54 1991
+        p = ctime(&t);
+        sprintf(buf, "%.10s %.4s", p, p + 20);
+    }
+    puts(buf);
+
     return InsertString(buf, strlen(buf));
 }
 
@@ -1248,8 +1374,122 @@ int EBuffer::InsertUid() {
     if (p == 0) p = getenv("NAME");
     if (p == 0) p = getenv("ID");
     if (p == 0) {
-        Msg(INFO, "User ID not set ($USER).");
+        Msg(S_INFO, "User ID not set ($USER).");
         return 0;
     }
     return InsertString(p, strlen(p));
 }
+
+int EBuffer::ShowHelpWord(ExState &State) {
+    //** Code for BlockSelectWord to find the word under the cursor,
+    const char *achr = "+-_."; // these are accepted characters
+    char    buf[128];
+    int     Y = VToR(CP.Row);
+    PELine  L = RLine(Y);
+    int     P;
+
+    P = CharOffset(L, CP.Col);
+
+    // fix \b for the case of CATBS
+    for(int i = 0; i < P; i++) {
+        //printf("%d - %d  %d   %c %c\n", i, P, L->Chars[i],
+        //L->Chars[i], L->Chars[P]);
+        if ((L->Chars[i] == '\b') && (P < (L->Count - 2)))
+            P += 2;
+    }
+    size_t len = 0;
+    if (P < L->Count) {
+        // To start of word,
+        while ((P > 0)
+               && ((L->Chars[P - 1] == '\b') || isalnum(L->Chars[P - 1])
+                   || (strchr(achr, L->Chars[P - 1]) != NULL)))
+            P--; // '_' for underline is hidden in achr
+        if ((P < (L->Count - 1)) && (L->Chars[P] == '\b'))
+            P++;
+        // To end of word,
+        while ((len < (sizeof(buf) - 1)) && (P < L->Count)) {
+            if (((P + 1) < L->Count) && (L->Chars[P + 1] == '\b'))
+                P += 2;
+            else if (isalnum(L->Chars[P])
+                     || (strchr(achr, L->Chars[P]) != NULL))
+                buf[len++] = L->Chars[P++];
+            else
+                break;
+        }
+    }
+    buf[len] = 0;
+    //printf("Word: %s\n", buf);
+    //if (buf[0] == 0) {
+    //    Msg(INFO, "No valid word under the cursor.");
+    //    return 0;
+    //}
+    return View->SysShowHelp(State, buf[0] ? buf : 0);
+}
+
+int EBuffer::GetStrVar(int var, char *str, int buflen) {
+    assert(buflen >= 0);
+    if (buflen == 0)
+        return 0;
+    //puts("variable EBuffer\x7");
+    switch (var) {
+    case mvFilePath:
+        //puts("variable FilePath\x7");
+        strncpy(str, FileName, buflen);
+        str[buflen - 1] = 0;
+        return 1;
+
+    case mvFileName:
+        JustFileName(FileName, str);
+        return 1;
+
+    case mvFileDirectory:
+        JustDirectory(FileName, str);
+        return 1;
+    case mvFileBaseName:
+        {
+            char buf[MAXPATH];
+            char *dot, *dot2;
+
+            JustFileName(FileName, buf);
+
+            dot = strchr(buf, '.');
+            while ((dot2 = strchr(dot + 1, '.')) != NULL)
+                dot = dot2;
+            if (dot)
+                *dot = 0;
+            strcpy(str, buf);
+        }
+        return 1;
+
+    case mvFileExtension:
+        {
+            char buf[MAXPATH];
+            char *dot, *dot2;
+
+            JustFileName(FileName, buf);
+
+            dot = strchr(buf, '.');
+            while ((dot2 = strchr(dot + 1, '.')) != NULL)
+                dot = dot2;
+            if (dot)
+                strcpy(str, dot);
+            else
+                str[0] = 0;
+        }
+        return 1;
+
+    case mvChar:
+    case mvWord:
+    case mvLine:
+        return 0;
+    }
+    return EModel::GetStrVar(var, str, buflen);
+}
+
+int EBuffer::GetIntVar(int var, int *value) {
+    switch (var) {
+    case mvCurRow: *value = VToR(CP.Row) + 1; return 1;
+    case mvCurCol: *value = CP.Col; return 1;
+    }
+    return EModel::GetIntVar(var, value);
+}
