@@ -10,9 +10,9 @@
 #include "fte.h"
 
 #ifdef CONFIG_OBJ_DIRECTORY
-EDirectory::EDirectory(EModel **ARoot, char *aPath): EList(ARoot, aPath) {
+EDirectory::EDirectory(int createFlags, EModel **ARoot, char *aPath): EList(createFlags, ARoot, aPath) {
     char XPath[MAXPATH];
-    
+
     Files = 0;
     FCount = 0;
     SearchLen = 0;
@@ -43,10 +43,10 @@ void EDirectory::DrawLine(PCell B, int Line, int Col, ChColor color, int Width)
         int Year, Mon, Day, Hour, Min, Sec;
         struct tm *t;
         time_t tim;
-        
+
         tim = Files[Line]->MTime();
         t = localtime(&tim);
-        
+
         if (t) {
             Year = t->tm_year + 1900;
             Mon = t->tm_mon + 1;
@@ -57,27 +57,34 @@ void EDirectory::DrawLine(PCell B, int Line, int Col, ChColor color, int Width)
         } else {
             Year = Mon = Day = Hour = Min = Sec = 0;
         }
-        
-        sprintf(s, " %04d/%02d/%02d %02d:%02d:%02d %8ld %.500s%c",
+
+        sprintf(s,
+                " %04d/%02d/%02d %02d:%02d:%02d %8ld ",
                 Year, Mon, Day, Hour, Min, Sec,
-                Files[Line]->Size(),
-                Files[Line]->Name(),
-                (Files[Line]->Type() == fiDIRECTORY)? '\\' : ' ');
-        if (Col < strlen(s))
-            MoveStr(B, 0, Width, s + Col, color, Width);
+                Files[Line]->Size());
+
+
+        strcat(s, Files[Line]->Name());
+        s[strlen(s) + 1] = '\0';
+        s[strlen(s)] = (Files[Line]->Type() == fiDIRECTORY)? SLASH : ' ';
+
+        if (Col < int(strlen(s)))
+            MoveStr(B, 0, Width, s + Col,
+                    (Files[Line]->Type() == fiDIRECTORY) ?
+                    color | (1<<3) : color, Width);
     }
 }
 
-int FileNameCmp(const void *a, const void *b) {
+int _LNK_CONV FileNameCmp(const void *a, const void *b) {
     FileInfo *A = *(FileInfo **)a;
     FileInfo *B = *(FileInfo **)b;
-    
+
     if (!(A->Type() == fiDIRECTORY) && (B->Type() == fiDIRECTORY))
         return 1;
-    
+
     if ((A->Type() == fiDIRECTORY) && !(B->Type() == fiDIRECTORY))
         return -1;
-    
+
     return filecmp(A->Name(), B->Name());
 }
 
@@ -120,10 +127,10 @@ void EDirectory::RescanList() {
         rc = ff->FindNext(&fi);
     }
     delete ff;
-    
+
     {
         char CTitle[256];
-        
+
         sprintf(CTitle, "%d files%c%d dirs%c%d bytes%c%-200.200s",
                 FCount, ConGetDrawChar(DCH_V),
                 DirCount, ConGetDrawChar(DCH_V),
@@ -132,6 +139,7 @@ void EDirectory::RescanList() {
         SetTitle(CTitle);
     }
     qsort(Files, FCount, sizeof(FileInfo *), FileNameCmp);
+    NeedsRedraw = 1;
 }
 
 void EDirectory::FreeList() {
@@ -143,13 +151,22 @@ void EDirectory::FreeList() {
     Files = 0;
 }
 
+int EDirectory::isDir(int No) {
+    char FilePath[256];
+
+    JustDirectory(Path, FilePath);
+    Slash(FilePath, 1);
+    strcat(FilePath, Files[No]->Name());
+    return IsDirectory(FilePath);
+}
+
 int EDirectory::ExecCommand(int Command, ExState &State) {
     switch (Command) {
     case ExActivateInOtherWindow:
         SearchLen = 0;
-        Msg(INFO, "");
+        Msg(S_INFO, "");
         if (Files && Row >= 0 && Row < FCount) {
-            if (Files[Row]->Type() == fiDIRECTORY) {
+            if (isDir(Row)) {
             } else {
                 return FmLoad(Files[Row]->Name(), View->Next);
             }
@@ -160,32 +177,32 @@ int EDirectory::ExecCommand(int Command, ExState &State) {
         if (RescanDir() == 0)
             return ErFAIL;
         return ErOK;
-        
+
     case ExDirGoUp:
         SearchLen = 0;
-        Msg(INFO, "");
+        Msg(S_INFO, "");
         FmChDir(SDOT SDOT);
         return ErOK;
-        
+
     case ExDirGoDown:
         SearchLen = 0;
-        Msg(INFO, "");
+        Msg(S_INFO, "");
         if (Files && Row >= 0 && Row < FCount) {
-            if (Files[Row]->Type() == fiDIRECTORY) {
+            if (isDir(Row)) {
                 FmChDir(Files[Row]->Name());
                 return ErOK;
             }
         }
         return ErFAIL;
-        
+
     case ExDirGoto:
         SearchLen = 0;
-        Msg(INFO, "");
+        Msg(S_INFO, "");
         return ChangeDir(State);
-    
+
     case ExDirGoRoot:
         SearchLen = 0;
-        Msg(INFO, "");
+        Msg(S_INFO, "");
         FmChDir(SSLASH);
         return ErOK;
     }
@@ -194,9 +211,9 @@ int EDirectory::ExecCommand(int Command, ExState &State) {
 
 int EDirectory::Activate(int No) {
     SearchLen = 0;
-    Msg(INFO, "");
+    Msg(S_INFO, "");
     if (Files && No >= 0 && No < FCount) {
-        if (Files[No]->Type() == fiDIRECTORY) {
+        if (isDir(No)) {
             FmChDir(Files[No]->Name());
             return 0;
         } else {
@@ -207,26 +224,30 @@ int EDirectory::Activate(int No) {
 }
 
 void EDirectory::HandleEvent(TEvent &Event) {
+    int resetSearch = 1;
     EModel::HandleEvent(Event);
     switch (Event.What) {
     case evKeyDown:
         switch (kbCode(Event.Key.Code)) {
         case kbBackSp:
+            resetSearch = 0;
             if (SearchLen > 0) {
                 SearchName[--SearchLen] = 0;
                 Row = SearchPos[SearchLen];
-                Msg(INFO, "Search: [%s]", SearchName);
+                Msg(S_INFO, "Search: [%s]", SearchName);
             } else
-                Msg(INFO, "");
+                Msg(S_INFO, "");
             break;
         case kbEsc:
-            Msg(INFO, "");
+            Msg(S_INFO, "");
             break;
         default:
+            resetSearch = 0; // moved here - its better for user
+            // otherwice there is no way to find files like i_ascii
             if (isAscii(Event.Key.Code) && (SearchLen < MAXISEARCH)) {
                 char Ch = (char) Event.Key.Code;
                 int Found;
-                
+
                 SearchPos[SearchLen] = Row;
                 SearchName[SearchLen] = Ch;
                 SearchName[++SearchLen] = 0;
@@ -240,11 +261,14 @@ void EDirectory::HandleEvent(TEvent &Event) {
                 }
                 if (Found == 0)
                     SearchName[--SearchLen] = 0;
-                Msg(INFO, "Search: [%s]", SearchName);
+                Msg(S_INFO, "Search: [%s]", SearchName);
             }
             break;
         }
     }
+    if (resetSearch) {
+        SearchLen = 0;
+    }
 }
 
 int EDirectory::RescanDir() {
@@ -266,10 +290,10 @@ int EDirectory::RescanDir() {
     return 1;
 }
 
-int EDirectory::FmChDir(char *Name) {
+int EDirectory::FmChDir(const char *Name) {
     char Dir[256];
     char CName[256] = "";
-    
+
     if (strcmp(Name, SSLASH) == 0) {
         JustRoot(Path, Dir);
     } else if (strcmp(Name, SDOT SDOT) == 0) {
@@ -301,11 +325,11 @@ int EDirectory::FmChDir(char *Name) {
 
 int EDirectory::FmLoad(char *Name, EView *XView) {
     char FilePath[256];
-    
+
     JustDirectory(Path, FilePath);
     Slash(FilePath, 1);
     strcat(FilePath, Name);
-    return FileLoad(FilePath, NULL, XView);
+    return FileLoad(0, FilePath, NULL, XView);
 }
 
 void EDirectory::GetName(char *AName, int MaxLen) {
@@ -321,7 +345,7 @@ void EDirectory::GetPath(char *APath, int MaxLen) {
 }
 
 void EDirectory::GetInfo(char *AInfo, int MaxLen) {
-    sprintf(AInfo, 
+    sprintf(AInfo,
             "%2d %04d/%03d %-150s",
             ModelNo,
             Row + 1, FCount,
@@ -329,15 +353,15 @@ void EDirectory::GetInfo(char *AInfo, int MaxLen) {
 }
 
 void EDirectory::GetTitle(char *ATitle, int MaxLen, char *ASTitle, int SMaxLen) {
-    
+
     strncpy(ATitle, Path, MaxLen - 1);
     ATitle[MaxLen - 1] = 0;
-    
+
     {
         char P[MAXPATH];
         strcpy(P, Path);
         Slash(P, 0);
-        
+
         JustDirectory(P, ASTitle);
         Slash(ASTitle, 1);
     }
@@ -346,14 +370,18 @@ void EDirectory::GetTitle(char *ATitle, int MaxLen, char *ASTitle, int SMaxLen)
 int EDirectory::ChangeDir(ExState &State) {
     char Dir[MAXPATH];
     char Dir2[MAXPATH];
-    
-    if (State.GetStrParam(Dir, sizeof(Dir)) == 0) {
+
+    if (State.GetStrParam(View, Dir, sizeof(Dir)) == 0) {
         strcpy(Dir, Path);
         if (View->MView->Win->GetStr("Set directory", sizeof(Dir), Dir, HIST_PATH) == 0)
             return 0;
     }
     if (ExpandPath(Dir, Dir2) == -1)
         return 0;
+#if 0
+    // is this needed for other systems as well ?
+    Slash(Dir2, 1);
+#endif
     if (Path)
         free(Path);
     Path = strdup(Dir2);
