@@ -21,7 +21,7 @@ public:
     char SearchName[MAXISEARCH];
     int SearchPos[MAXISEARCH];
     
-    EDirectory(EModel **ARoot, char *aPath);
+    EDirectory(int createFlags, EModel **ARoot, char *aPath);
     virtual ~EDirectory();
     
     virtual int GetContext();
@@ -43,7 +43,8 @@ public:
     virtual void GetInfo(char *AInfo, int MaxLen);
     virtual void GetTitle(char *ATitle, int MaxLen, char *ASTitle, int SMaxLen);
     
-    int FmChDir(char *Name);
+    int isDir(int No);
+    int FmChDir(const char *Name);
     int FmLoad(char *Name, EView *View);
     int ChangeDir(ExState &State);
     int RescanDir();
