@@ -22,7 +22,7 @@ struct {
     RxNode *rx;
 } CRegexp[MAXREGEXP];
 
-int AddCRegexp(int file, int line, int msg, char *regexp) {
+int AddCRegexp(int file, int line, int msg, const char *regexp) {
     if (NCRegexp >= MAXREGEXP) return 0;
     CRegexp[NCRegexp].RefFile = file;
     CRegexp[NCRegexp].RefLine = line;
@@ -34,7 +34,7 @@ int AddCRegexp(int file, int line, int msg, char *regexp) {
     return 1;
 }
 
-EMessages::EMessages(EModel **ARoot, char *ADir, char *ACommand): EList(ARoot, "Messages") {
+EMessages::EMessages(int createFlags, EModel **ARoot, char *ADir, char *ACommand): EList(createFlags, ARoot, "Messages") {
     CompilerMsgs = this;
     ErrCount = 0;
     ErrList = 0;
@@ -151,15 +151,21 @@ int EMessages::RunPipe(char *ADir, char *ACommand) {
     ReturnCode = -1;
     Running = 1;
     BufLen = BufPos = 0;
+    Row = ErrCount - 1;
 
     {
-        char s[MAXPATH * 4];
-        Error *pe;
+        char s[2 * MAXPATH * 4];
 
         sprintf(s, "[running '%s' in '%s']", Command, Directory);
         AddError(0, -1, 0, s);
     }
 
+    {
+        char s[MAXPATH * 2];
+        sprintf(s, "Messages [%s]: %s", Directory, Command);
+        SetTitle(s);
+    }
+    
     ChangeDir(Directory);
     PipeId = gui->OpenPipe(Command, this);
     return 0;
@@ -201,13 +207,14 @@ void EMessages::AddError(Error *p) {
 
     if (ErrCount > Count)
         if (Row >= Count - 1) {
-            Row = ErrCount - 1;
+            //if (ErrCount > 1 && !ErrList[TopRow]->file)
+                Row = ErrCount - 1;
         }
 
     UpdateList();
 }
 
-void EMessages::AddError(char *file, int line, char *msg, char *text) {
+void EMessages::AddError(char *file, int line, char *msg, const char *text) {
     Error *pe;
 
     pe = (Error *) malloc(sizeof(Error));
@@ -340,7 +347,6 @@ void EMessages::GetErrors() {
     char line[1024];
     RxMatchRes RM;
     //int retc;
-    Error *pe;
     int i, n;
     int didmatch = 0;
     int WasRunning = Running;
@@ -477,11 +483,11 @@ int EMessages::CompilePrevError(EView *V) {
             if (ErrList[Row]->line == -1 || !ErrList[Row]->file) goto bad;
             ShowError(V, Row);
         } else {
-            V->Msg(INFO, "No previous error.");
+            V->Msg(S_INFO, "No previous error.");
             return 0;
         }
     } else {
-        V->Msg(INFO, "No errors.");
+        V->Msg(S_INFO, "No errors.");
         return 0;
     }
     return 1;
@@ -496,22 +502,22 @@ int EMessages::CompileNextError(EView *V) {
             ShowError(V, Row);
         } else {
             if (Running)
-                V->Msg(INFO, "No more errors (yet).");
+                V->Msg(S_INFO, "No more errors (yet).");
             else
-                V->Msg(INFO, "No more errors.");
+                V->Msg(S_INFO, "No more errors.");
             return 0;
         }
     } else { 
         if (Running) 
-            V->Msg(INFO, "No errors (yet).");
+            V->Msg(S_INFO, "No errors (yet).");
         else
-            V->Msg(INFO, "No errors.");
+            V->Msg(S_INFO, "No errors.");
         return 0;
     }
     return 1;
 }
 
-int EMessages::Compile(char *Command) {
+int EMessages::Compile(char * /*Command*/) {
     return 0;
 }
 
@@ -528,22 +534,22 @@ void EMessages::ShowError(EView *V, int err) {
                 sprintf(bk, "_MSG.%d", err);
                 ErrList[err]->Buf->GotoBookmark(bk);
             } else {
-                if (FileLoad(ErrList[err]->file, 0, V) == 1) {
+                if (FileLoad(0, ErrList[err]->file, 0, V) == 1) {
                     V->SwitchToModel(ActiveModel);
                     ((EBuffer *)ActiveModel)->CenterNearPosR(0, ErrList[err]->line - 1);
                 }
             }
             if (ErrList[err]->msg != 0)
-                V->Msg(INFO, "%s", ErrList[err]->msg);
+                V->Msg(S_INFO, "%s", ErrList[err]->msg);
             else
-                V->Msg(INFO, "%s", ErrList[err]->text);
+                V->Msg(S_INFO, "%s", ErrList[err]->text);
         }
     }
 }
 
 void EMessages::DrawLine(PCell B, int Line, int Col, ChColor color, int Width) {
     if (Line < ErrCount)
-        if (Col < strlen(ErrList[Line]->text)) {
+        if (Col < int(strlen(ErrList[Line]->text))) {
             char str[1024];
             int len;
 
@@ -571,6 +577,8 @@ void EMessages::UpdateList() {
 }
 
 int EMessages::Activate(int No) {
+    //assert(No == Row);
+    //Row = No;
     ShowError(View, Row);
     return 1;
 }
