@@ -2,6 +2,7 @@
  *  portdos.c:  Portable calls for DOS
  *  ==================================
  *  (C)1996 by F.Jalvingh; Public domain.
+ *  (C)1997 by Markus F.X.J. Oberhumer; Public domain.
  *
  *  $Header$
  *
@@ -12,6 +13,15 @@
 #include    <string.h>
 
 #if defined(DOS) || defined(DOSP32)
+#include    <dos.h>
+#if defined(__DJGPP__)
+#include    <pc.h>
+#include    <dpmi.h>
+#include    <go32.h>
+#include    <sys/movedata.h>
+#define inp	inportb
+#define outp outportb
+#endif
 
 struct plScnInfo
 {
@@ -31,8 +41,8 @@ struct plScnInfo
 	
     UWORD           s_iobase;
 
-#ifdef  __DOS4G__
-	ULONG           s_pbase;            // Physical base address of video memory,
+#if defined(__DOS4G__) || defined(__DJGPP__)
+ 	ULONG           s_pbase;            // Physical base address of video memory
 #else
 	UWORD           s_pseg;             // Segment of video memory
 #endif
@@ -180,7 +190,9 @@ static void getType(void)
 	}
 
 	//** Determine hardware addresses..
-#ifdef  __DOS4G__
+#if defined(__DJGPP__)
+	plScnI.s_pbase  = ScreenPrimary;
+#elif defined(__DOS4G__)
 	plScnI.s_pbase  = plScnI.s_ismono ? 0xb0000 : 0xb8000;
 #elif defined(__MSDOS__)
 	plScnI.s_pseg   = plScnI.s_ismono ? 0xb000 : 0xb800;
@@ -275,6 +287,17 @@ void plScnSetFlash(boolean on)
 }
 
 
+#if defined(__DJGPP__)
+static void ScreenSetCursorShape(int shape)
+{
+	__dpmi_regs reg;
+
+	reg.h.ah = 0x01;
+	reg.h.al = ScreenMode();
+	reg.x.cx = shape & 0x7f1f;
+	__dpmi_int(0x10, &reg);
+}
+#else
 /*
  *  cardOut()..
  */
@@ -283,6 +306,7 @@ static void cardOut(int port, int val)
 	outp(plScnI.s_iobase, port);
 	outp(plScnI.s_iobase + 1, val);
 }
+#endif
 
 
 /*
@@ -297,6 +321,10 @@ void plScnCursorOn(boolean on)
 	plScnI.s_curon  = on;
 	v	= (UWORD) ( ((plScnI.s_curstart & 0xff) << 8) | (plScnI.s_curend & 0xff));
 	if(! on) v |= 0x2000;           // Disable if required..
+
+#if defined(__DJGPP__)
+	ScreenSetCursorShape(v);
+#else
 	dosxDisable();
 	cardOut(10, v >> 8);
 	cardOut(11, v & 0xff);
@@ -307,6 +335,7 @@ void plScnCursorOn(boolean on)
 		cardOut(15, 0);
 	}
 	dosxEnable();
+#endif
 }
 
 
@@ -339,6 +368,9 @@ void plScnCursorShapeGet(int* sp, int* ep)
  */
 void plScnCursorPos(int x, int y)
 {
+#if defined(__DJGPP__)
+	ScreenSetCursor(y,x);
+#else
 	UWORD       soff;
 
 	plScnInit();
@@ -350,6 +382,7 @@ void plScnCursorPos(int x, int y)
 	dosxEnable();
 	plScnI.s_cur_x  = x;
 	plScnI.s_cur_y  = y;
+#endif
 }
 
 
@@ -365,9 +398,63 @@ void plScnCursorPosGet(int* xp, int* yp)
 
 
 
+/****************************************************************************/
+/*																			*/
+/*	DJGPP implementation. 													*/
+/*																			*/
+/****************************************************************************/
+#if defined(__DJGPP__)              // Protected Mode
+
+void plScnWrite(unsigned x, unsigned y, unsigned short* buf, unsigned nch)
+{
+	plScnInit();
+ 	movedata(_my_ds(), (unsigned) buf, _dos_ds, plScnI.s_pbase+(((y * plScnI.s_wid) + x)*2), nch*2);
+}
+
+void plScnRead(unsigned x, unsigned y, unsigned short* buf, unsigned nch)
+{
+	plScnInit();
+	movedata(_dos_ds, plScnI.s_pbase+((y * plScnI.s_wid) + x)*2, _my_ds(), (unsigned) buf, nch*2);
+}
+
+static void moveVm(unsigned to, unsigned from, unsigned len)
+{
+	movedata(_dos_ds, plScnI.s_pbase + from*2, _dos_ds,  plScnI.s_pbase + to*2, len*2);
+}
 
 
-#if defined(__MSDOS__)              // REAL Mode
+/****************************************************************************/
+/*																			*/
+/*	CODING: Dos4GW implementation.. 										*/
+/*																			*/
+/****************************************************************************/
+#elif defined(__DOS4G__)
+
+void plScnWrite(unsigned x, unsigned y, unsigned short* buf, unsigned nch)
+{
+    plScnInit();
+	memcpy((UBYTE*) plScnI.s_pbase+(((y * plScnI.s_wid) + x)*2), buf, nch<<1);
+}
+
+void plScnRead(unsigned x, unsigned y, unsigned short* buf, unsigned nch)
+{
+	plScnInit();
+	memcpy(buf, (UBYTE*)plScnI.s_pbase+(((y * plScnI.s_wid) + x)*2), nch<<1);
+}
+
+static void moveVm(unsigned to, unsigned from, unsigned len)
+{
+    memcpy((UBYTE*) plScnI.s_pbase+to*2, (UBYTE*) plScnI.s_pbase+from*2, len*2);
+}
+
+
+/****************************************************************************/
+/*																			*/
+/*	REAL MODE implementation. 												*/
+/*																			*/
+/****************************************************************************/
+
+#elif defined(__MSDOS__)              // REAL Mode
 #include	<dos.h>
 
 void plScnWrite(unsigned x, unsigned y, unsigned short* buf, unsigned nch)
@@ -387,29 +474,6 @@ static void moveVm(unsigned to, unsigned from, unsigned len)
 	memcpy(MK_FP(plScnI.s_pseg, to*2), MK_FP(plScnI.s_pseg, from*2), len << 1);
 }
 
-
-/****************************************************************************/
-/*																			*/
-/*	CODING: Dos4GW implementation.. 										*/
-/*																			*/
-/****************************************************************************/
-#elif defined(__DOS4G__)
-void plScnWrite(unsigned x, unsigned y, unsigned short* buf, unsigned nch)
-{
-    plScnInit();
-	memcpy((UBYTE*) plScnI.s_pbase+(((y * plScnI.s_wid) + x)*2), buf, nch<<1);
-}
-
-void plScnRead(unsigned x, unsigned y, unsigned short* buf, unsigned nch)
-{
-	plScnInit();
-	memcpy(buf, (UBYTE*)plScnI.s_pbase+(((y * plScnI.s_wid) + x)*2), nch<<1);
-}
-
-static void moveVm(unsigned to, unsigned from, unsigned len)
-{
-    memcpy((UBYTE*) plScnI.s_pbase+to*2, (UBYTE*) plScnI.s_pbase+from*2, len*2);
-}
 #endif
 
 
