@@ -9,7 +9,6 @@
 
 // TODO: on OS/2 choose between UNIX and OS/2 style patterns (now OS/2 only)
 // TODO: on OS/2 fetch multiple entries at once and cache them for speed
-
 #include "fte.h"
 
 #ifdef OS2
@@ -26,6 +25,27 @@
 #   include "port.h"
 #endif
 
+
+#ifdef DJGPP
+#include <sys/stat.h>
+static int my_stat(const char *name, struct stat *s)
+{
+    unsigned short f = _djstat_flags;
+    // speed up directory access by turning off unused stat functionality
+    _djstat_flags |= _STAT_INODE;
+    _djstat_flags |= _STAT_EXEC_EXT;
+    _djstat_flags |= _STAT_EXEC_MAGIC;
+    _djstat_flags |= _STAT_DIRSIZE;
+    //_djstat_flags |= _STAT_ROOT_TIME;
+    _djstat_flags &= ~_STAT_ROOT_TIME;
+    _djstat_flags |= _STAT_WRITEBIT;
+    int r = stat(name,s);
+    _djstat_flags = f;
+    return r;
+}
+#endif
+
+
 FileInfo::FileInfo(char *Name, int Type, off_t Size, time_t MTime) {
     name = new char[strlen (Name) + 1];
     if (name)
@@ -39,7 +59,7 @@ FileInfo::~FileInfo() {
     delete [] name;
 }
 
-FileFind::FileFind(char *aDirectory, char *aPattern, int aFlags) {
+FileFind::FileFind(const char *aDirectory, const char *aPattern, int aFlags) {
     Directory = new char[strlen(aDirectory) + 1];
     if (Directory) {
         strcpy(Directory, aDirectory);
@@ -53,13 +73,13 @@ FileFind::FileFind(char *aDirectory, char *aPattern, int aFlags) {
         Pattern = 0;
     }
     Flags = aFlags;
-#if defined(USE_DIRENT) 
-//#if defined(UNIX) || defined(DOSP32) || defined(NT)
+#if defined(USE_DIRENT)
+    //#if defined(UNIX) || defined(DOSP32) || defined(NT)
     dir = 0;
 #elif defined(OS2) && !defined(USE_DIRENT)
     dir = 0;
 #elif defined(NT) && !defined(USE_DIRENT)
-	dir = 0;
+    dir = 0;
 #endif
 }
 
@@ -67,7 +87,7 @@ FileFind::~FileFind() {
     delete [] Directory;
     if (Pattern)
         delete [] Pattern;
-//#if defined(UNIX) || defined(DOSP32) || defined(NT)
+    //#if defined(UNIX) || defined(DOSP32) || defined(NT)
 #if defined(USE_DIRENT)
     if (dir)
         closedir(dir);
@@ -76,12 +96,12 @@ FileFind::~FileFind() {
         DosFindClose(dir);
 #elif defined(NT) && !defined(USE_DIRENT)
     if (dir)
-		FindClose((HANDLE)dir);
+        FindClose((HANDLE)dir);
 #endif
 }
 
 int FileFind::FindFirst(FileInfo **fi) {
-//#if defined(UNIX) || defined(NT) || defined(DOSP32)
+    //#if defined(UNIX) || defined(NT) || defined(DOSP32)
 #if defined(USE_DIRENT)
     if (dir)
         closedir(dir);
@@ -101,7 +121,7 @@ int FileFind::FindFirst(FileInfo **fi) {
 
     if (dir)
         DosFindClose(dir);
-    
+
     if (Flags & ffDIRECTORY)
         attr |= FILE_DIRECTORY;
 
@@ -146,7 +166,7 @@ int FileFind::FindFirst(FileInfo **fi) {
     return 0;
 #elif defined(NT) && !defined(USE_DIRENT)
 #if defined(USE_VCFIND)
-	char fullpattern[MAXPATH];
+    char fullpattern[MAXPATH];
 
     _finddata_t find; // need to improve to fetch multiple entries at once
     char fullpath[MAXPATH];
@@ -156,13 +176,13 @@ int FileFind::FindFirst(FileInfo **fi) {
 
     if (dir)
         _findclose(dir);
-    
-    /*if (Flags & ffDIRECTORY)
-        attr |= FILE_DIRECTORY;
 
-    if (Flags & ffHIDDEN)
-        attr |= FILE_HIDDEN | FILE_SYSTEM; // separate ?
-*/
+    /*if (Flags & ffDIRECTORY)
+     attr |= FILE_DIRECTORY;
+
+     if (Flags & ffHIDDEN)
+     attr |= FILE_HIDDEN | FILE_SYSTEM; // separate ?
+     */
     if (Pattern)
         JoinDirFile(fullpattern, Directory, Pattern);
     else
@@ -170,7 +190,7 @@ int FileFind::FindFirst(FileInfo **fi) {
 
     if ((rc = _findfirst(fullpattern, &find)) == -1)
     {
-//        fprintf(stderr, "%s: %d\n\n", fullpattern, rc);
+        //        fprintf(stderr, "%s: %d\n\n", fullpattern, rc);
         return -1;
     }
     dir = rc;
@@ -180,30 +200,31 @@ int FileFind::FindFirst(FileInfo **fi) {
         JoinDirFile(fullpath, Directory, name);
         name = fullpath;
     }
-    
+
     *fi = new FileInfo(name,
                        (find.attrib & _A_SUBDIR) ? fiDIRECTORY : fiFILE,
                        find.size,
                        find.time_create);
     return 0;
 #else
-	char fullpattern[MAXPATH];
+    char fullpattern[MAXPATH];
 
     WIN32_FIND_DATA find; // need to improve to fetch multiple entries at once
     char fullpath[MAXPATH];
     char *name;
     struct tm t;
+    SYSTEMTIME st;
     int rc;
 
     if (dir)
         _findclose(dir);
-    
-    /*if (Flags & ffDIRECTORY)
-        attr |= FILE_DIRECTORY;
 
-    if (Flags & ffHIDDEN)
-        attr |= FILE_HIDDEN | FILE_SYSTEM; // separate ?
-*/
+    /*if (Flags & ffDIRECTORY)
+     attr |= FILE_DIRECTORY;
+
+     if (Flags & ffHIDDEN)
+     attr |= FILE_HIDDEN | FILE_SYSTEM; // separate ?
+     */
     if (Pattern)
         JoinDirFile(fullpattern, Directory, Pattern);
     else
@@ -221,18 +242,29 @@ int FileFind::FindFirst(FileInfo **fi) {
         JoinDirFile(fullpath, Directory, name);
         name = fullpath;
     }
-    
+
+    FileTimeToSystemTime(&find.ftLastWriteTime, &st);
+
+    t.tm_year = st.wYear - 1900;
+    t.tm_mon = st.wMonth;
+    t.tm_mday = st.wDay;
+    t.tm_hour = st.wHour;
+    t.tm_min = st.wMinute;
+    t.tm_sec = st.wSecond;
+    t.tm_isdst = -1;
+
+
     *fi = new FileInfo(name,
                        (find.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) ? fiDIRECTORY : fiFILE,
                        find.nFileSizeLow, // !!!
-                       0); //find.time_create);
+                       mktime(&t));
     return 0;
 #endif
 #endif
 }
 
 int FileFind::FindNext(FileInfo **fi) {
-//#if defined(UNIX) || defined(NT) || defined(DOSP32)
+    //#if defined(UNIX) || defined(NT) || defined(DOSP32)
 #if defined(USE_DIRENT)
     struct dirent *dent;
     char fullpath[MAXPATH];
@@ -240,10 +272,9 @@ int FileFind::FindNext(FileInfo **fi) {
 
     *fi = 0;
 again:
-    if ((dent = readdir(dir)) == 0)
+    if ((dent = readdir(dir)) == NULL)
         return -1;
     name = dent->d_name;
-    //printf("name=%s\n", name);
 
     if (name[0] == '.')
         if (!(Flags & ffHIDDEN))
@@ -251,7 +282,7 @@ again:
 
     if (Pattern && fnmatch(Pattern, dent->d_name, 0) != 0)
         goto again;
-    
+
     if (Flags & ffFULLPATH) {
         JoinDirFile(fullpath, Directory, dent->d_name);
         name = fullpath;
@@ -265,13 +296,15 @@ again:
         if (!(Flags & ffFULLPATH)) // need it now
             JoinDirFile(fullpath, Directory, dent->d_name);
 
-	if (
-#ifdef UNIX
+        if (
+#if defined(DJGPP)
+            my_stat
+#elif defined(UNIX) // must use lstat if available
             lstat
-#else 
+#else
             stat
 #endif
-            (fullpath, &st) != 0) 
+            (fullpath, &st) != 0 && 0)
             goto again;
 
         if (!(Flags & ffDIRECTORY) && S_ISDIR(st.st_mode))
@@ -291,7 +324,7 @@ again:
     char *name;
     struct tm t;
     int rc;
-    
+
     if ((rc = DosFindNext(dir,
                           &find, sizeof(find),
                           &count)) != 0)
@@ -306,8 +339,8 @@ again:
         JoinDirFile(fullpath, Directory, name);
         name = fullpath;
     }
-	memset((void *)&t, 0, sizeof(t));
-    t.tm_year = find.fdateLastWrite.year + 80; // ugh!
+    memset((void *)&t, 0, sizeof(t));
+    t.tm_year = find.fdateLastWrite.year + 80;
     t.tm_mon = find.fdateLastWrite.month - 1;
     t.tm_mday = find.fdateLastWrite.day;
     t.tm_hour = find.ftimeLastWrite.hours;
@@ -318,7 +351,7 @@ again:
                        (find.attrFile & FILE_DIRECTORY) ? fiDIRECTORY : fiFILE,
                        find.cbFile,
                        mktime(&t));
-    
+
     return 0;
 #elif defined(NT) && !defined(USE_DIRENT)
 #if defined(USE_VCFIND)
@@ -327,14 +360,14 @@ again:
     char *name;
     struct tm t;
     int rc;
-    
+
     if ((rc = _findnext(dir,
-                          &find)) != 0)
+                        &find)) != 0)
     {
         // fprintf(stderr, "%d\n\n", rc);
         return -1;
     }
-    
+
     name = find.name;
     if (Flags & ffFULLPATH) {
         JoinDirFile(fullpath, Directory, name);
@@ -351,27 +384,38 @@ again:
     char fullpath[MAXPATH];
     char *name;
     struct tm t;
+    SYSTEMTIME st;
     int rc;
-    
+
     if ((rc = FindNextFile((HANDLE)dir,
                            &find)) != TRUE)
     {
-        fprintf(stderr, "%d\n\n", rc);
+        //fprintf(stderr, "%d\n\n", rc);
         return -1;
     }
-    
+
     name = find.cFileName;
     if (Flags & ffFULLPATH) {
         JoinDirFile(fullpath, Directory, name);
         name = fullpath;
     }
 
+    FileTimeToSystemTime(&find.ftLastWriteTime, &st);
+
+    t.tm_year = st.wYear - 1900;
+    t.tm_mon = st.wMonth;
+    t.tm_mday = st.wDay;
+    t.tm_hour = st.wHour;
+    t.tm_min = st.wMinute;
+    t.tm_sec = st.wSecond;
+    t.tm_isdst = -1;
+
     *fi = new FileInfo(name,
                        (find.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) ? fiDIRECTORY : fiFILE,
                        find.nFileSizeLow,
-                       0);//find.time_create);
+                       mktime(&t));
     return 0;
 #endif
-//    put code here
+    //    put code here
 #endif
 }
