@@ -15,6 +15,9 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
+#if defined(AIX) || defined(SCO) || defined(NCR)
+#include <strings.h>
+#endif
 #include <stdio.h>
 #include <fcntl.h>
 #include <ctype.h>
@@ -22,6 +25,10 @@
 #include <sys/stat.h>
 #include <time.h>
 
+#ifdef DBMALLOC
+#include <malloc.h>
+#endif
+
 #if !defined(OS2) && \
     !defined(NT) && \
     !defined(DOSP32) && \
@@ -29,11 +36,13 @@
     !defined(HPUX) && \
     !defined(AIX) && \
     !defined(IRIX) && \
-    !defined(SUNOS)
+    !defined(SCO) && \
+    !defined(SUNOS) && \
+    !defined(NCR)
 #    error Target not supported.
 #endif
 
-#if defined(UNIX)
+#if defined(UNIX) || defined(DJGPP)
 #    define USE_DIRENT
 #endif
 
@@ -44,21 +53,22 @@
 #if defined(UNIX)
 #    include <unistd.h>
 #    include <pwd.h>
-#    if defined(HPUX) || defined(AIX)
-#        include <fnmatch.h>             // IRIX seems to be missing this ??
-#    endif
+#    include <fnmatch.h>
 #    define strnicmp strncasecmp
 #    define stricmp strcasecmp
 #    define filecmp strcmp
-#    define memicmp strncasecmp   // FIX, fails for nulls
+     //#    define memicmp strncasecmp   // FIX, fails for nulls
+     extern "C" int memicmp(const void *s1, const void *s2, size_t n);
 #endif
 
 #if defined(OS2)
 #    include <malloc.h>
-#    include <dos.h>
+#    if !defined(__TOS_OS2__)
+#        include <dos.h>
+#    endif
 #    include <io.h>
 #    include <process.h>
-#    if defined(BCPP) || defined(WATCOM)
+#    if defined(BCPP) || defined(WATCOM) || defined(__TOS_OS2__)
 #        include <direct.h>
 #    endif
 #    if defined(BCPP)
@@ -78,6 +88,12 @@
 #    if defined(WATCOM)
 #        include <direct.h>
 #    endif
+#    if defined(DJGPP)
+#        include <dir.h>
+#        include <unistd.h>
+#        undef MAXPATH
+         extern "C" int memicmp(const void *s1, const void *s2, size_t n);
+#    endif
 #    define filecmp stricmp
 #endif
 
@@ -96,6 +112,8 @@
 #        include <dir.h>
 #    endif
 #    define filecmp stricmp
+#    define popen _popen
+#    define pclose _pclose
 #endif
 
 #ifndef MAXPATH
@@ -107,11 +125,15 @@
 #endif
 
 #if defined(OS2) || defined(NT)
-#if defined(EMX) || defined(WATCOM)
-#define FAKE_BEGINTHREAD_NULL NULL,
-#else
-#define FAKE_BEGINTHREAD_NULL
+#    if defined(EMX) || defined(WATCOM) || defined(__TOS_OS2__)
+#        define FAKE_BEGINTHREAD_NULL NULL,
+#    else
+#        define FAKE_BEGINTHREAD_NULL
+#    endif
 #endif
+
+#if !defined(__IBMC__) && !defined(__IBMCPP__)
+#    define _LNK_CONV
 #endif
 
 #define PT_UNIXISH   0
@@ -125,6 +147,11 @@
 #    endif
 #endif
 
+#ifndef S_IWGRP
+#define S_IWGRP 0
+#define S_IWOTH 0
+#endif
+
 #if defined(OS2) || defined(NT) || defined(DOSP32) || defined(DOS)
 #define PATHTYPE   PT_DOSISH
 #else
